#include <stdio.h>
#include <string.h>
#include <math.h>
#include <stdlib.h>
#include <umfpack.h>

/* INCORPORADO POR COMANDOS PARA LA INTERFAZ GRAFICA
// DIVISIONES EN DIRECCION CIRCUNFERENCIAL (THETA)
// NOTA: RESTAR 1 POR SER PERIODICO (P.E. 15 = 16 ELEMENTOS)
#define DIV_U 34			-1

// DIVISIONES EN DIRECCION LONGITUDINAL (PHI)
// NOTA: RESTAR 1 POR SER PERIODICO (P.E. 15 = 16 ELEMENTOS)
#define DIV_W 34			-1

// DIVISIONES EN PROFUNDIDAD (RHO)
#define DIV_R 10
*/
//TRANSFORMADAS LAS CONSTANTES EN VARIABLES
int DIV_U = 34;
int DIV_W = 34;
int DIV_R = 10;
char * ACTUAL_PATH= "";
// DIVISIONES DE CUADRICULA (SOLO CON DISCRETIZACION DE RECTANGULO INFLADO)
// NOTA: RESTAR 1 (P.E. 15 = 16 ELEMENTOS)
#define DIV_C 15


#define PI 3.14159265358979323846
#define T_MAX 200000
#define TOLERANCIA 0.00001
#define DISTANCE_WEIGHT_EXPONENT 2
double TANGENT_ANGLE_THRESHOLD_DEGREES=7.5; //El angulo de la tangente a la curva y el vector radio deben estar entre (este valor) y 180-(este valor)

#define NO_INICIALIZADO -9
#define CUADRANTE1	3
#define CUADRANTE2	2
#define CUADRANTE3	0
#define CUADRANTE4	1

//int dir_phi=-1; //0 si phi=u e 1 si phi=w


/*********************************************************/
/*************          **********************************/
/*************  TYPES   **********************************/
/*************          **********************************/
/*********************************************************/


typedef struct punto {
	double x;
	double y;
	double z;
} punto;

typedef punto vector;

typedef struct cara {
	long int id[3];
} cara;


typedef struct arista {
	long int inicio_id;
	long int final_id;
	int utilizada;
} arista;


typedef struct{
	long int n_total;
	struct punto coord[T_MAX];
} lista_puntos;


typedef struct{
	long int n_total;
	cara coord[T_MAX];
	int activeFlag[T_MAX];
} lista_caras; 


typedef struct{
	long int n_total;
	long int comienzo;
	arista aristas[T_MAX];
} lista_aristas;


typedef struct{
	long int 	*i;
	long int 	*j;
	double	*val;
	long int n_total;
} sparseMatrix;


typedef struct{
	long int ini;
	long int fin;
	long int medioBajo;
	long int medioAlto;
	double x;
	double y;
	double z;
} sustitucion;


/*********************************************************/
/*************                     ***********************/
/************* GEOMETRY FUNCTIONS  ***********************/
/*************                     ***********************/
/*********************************************************/

double	//expresado en radianes
anguloVectores	(	vector	u,
					vector	v)
{
	double productoEscalar = u.x * v.x + u.y * v.y + u.z * v.z;
	double moduloU=sqrt( (u.x * u.x) + (u.y * u.y) +(u.z * u.z) );
	double moduloV=sqrt( (v.x * v.x) + (v.y * v.y) +(v.z * v.z) );
	return acos(productoEscalar/moduloU/moduloV);
}

double
prodEscalar(	vector	u,
				vector	v)
{
	return u.x*v.x+u.y*v.y+u.z*v.z;
}

vector
vectorSuma(	vector	u,
			vector	v)
{
	vector aux;
	aux.x=u.x+v.x;
	aux.y=u.y+v.y;
	aux.z=u.z+v.z;
	return aux;
}

vector
vectorResta(	vector	u,
				vector	v)
{
	vector aux;
	aux.x=u.x-v.x;
	aux.y=u.y-v.y;
	aux.z=u.z-v.z;
	return aux;
}

vector
prodVectorial( 	vector 	u,
				vector 	v)
{
	vector result;
	result.x=u.y*v.z-u.z*v.y;
	result.y=u.z*v.x-u.x*v.z;
	result.z=u.x*v.y-u.y*v.x;
	return result;
}


vector
vectorPorEscalar(	vector 	p,
					double 	escalar)
{
	vector aux;
	aux.x=p.x*escalar;
	aux.y=p.y*escalar;
	aux.z=p.z*escalar;
	return aux;
}


double
moduloVector	(vector p)
{
	return sqrt( (p.x * p.x) + (p.y * p.y) + (p.z * p.z) );
}


vector
vectorUnitario (vector p)
{
	double modulo = moduloVector(p);
	p.x/=modulo;
	p.y/=modulo;
	p.z/=modulo;
	return p; 
}


int	//devuelve 1 si los vertices (2D) de un triangulo ABC se numeran en el sentido de las agujas (AB x BC va en direccion z-)
prodVectorialUnitario(	punto	a,
						punto	b,
						punto 	c)
{
	double r;
	r = (a.x - c.x) * (b.y - c.y) - (b.x-c.x) * (a.y-c.y);
	return (r<0) ? 1 : 0;
}


lista_puntos	//dada una polilinea y un plano (dado por su vector normal y un punto del mismo), crea una segunda polilinea producto de proyectar todos los puntos de la primera sobre el plano
proyectarSobrePlano3d(	lista_puntos	a,
						punto			centro,
						vector			normal)
{
	int i;
	lista_puntos b;
	punto C,P,X;
	vector CP,CX,XP, N;
	
	b.n_total=a.n_total;
	
	//a=polilínea a proyectar
	//b=polilínea proyectada
	//C=punto en plano proyectante
	//P=punto a proyectar
	//N=vector normal al plano
	//X=punto proyeccion
	
	C=centro;
	N=vectorUnitario(normal);
	
	for (i = 0; i < a.n_total; i++)
	{
		P=a.coord[i];
		CP=vectorResta(P,C);
		XP=vectorPorEscalar(N,prodEscalar(CP,N));
		CX=vectorResta(CP,XP);
		X=vectorSuma(C,CX);
		b.coord[i]=X;
	}
	
	return b;
}


lista_puntos	// reduce una serie de coordenadas 3d a 2d dado un punto del plano de proyeccion y dos direcciones ortogonales contenidas en el mismo
proyectarSobrePlano2d(	lista_puntos	a,
						punto			origen,
						vector			vectorX,
						vector			vectorY)
{
	int i;
	lista_puntos b; b.n_total=a.n_total;
	punto posicion;
	
	for (i = 0; i < a.n_total; i++)
	{
		posicion=vectorResta(a.coord[i],origen);
		b.coord[i].x=prodEscalar(posicion,vectorX);
		b.coord[i].y=prodEscalar(posicion,vectorY);
		b.coord[i].z=0;	
	}
	
	return b;
}


double //mide la distancia desde un plano (dado por su vector normal y un punto perteneciente al plano) y un punto exterior al plano. La distancia se mide perpendicular al plano y tiene signo
distanciaPuntoPlano(	punto 	x,
						vector	normal,
						punto	centro)
{
	vector cx=vectorResta(x,centro);
	return prodEscalar(vectorUnitario(normal),cx);
}


double
jacobianoElementoCuadrado	(	punto p1, // s- t-
								punto p2, // s+ t-
								punto p3, // s+ t+ 
								punto p4,
								double s,
								double t) // s- t+
{
	int i;
	double P[4];
	double J;
	double dxdslow,dxdtlow,dydslow,dydtlow,dxdshigh,dxdthigh,dydshigh,dydthigh;
	double dxds,dxdt,dyds,dydt;
	double avg, min;
	double x1,x2,x3,x4,y1,y2,y3,y4;
	
	y1=p1.y;
	y2=p2.y;
	y3=p3.y;
	y4=p4.y;
	x1=p1.x;
	x2=p2.x;
	x3=p3.x;
	x4=p4.x;
	
	//dxds=((-1 + t)*x1 + x2 - t*x2 + (1 + t)*(x3 - x4))/4;
	//dxdt= ((-1 + s)*x1 - (1 + s)*x2 + x3 + s*x3 + x4 - s*x4)/4;
	//dyds= ((-1 + t)*y1 + y2 - t*y2 + (1 + t)*(y3 - y4))/4;
	//dydt= ((-1 + s)*y1 - (1 + s)*y2 + y3 + s*y3 + y4 - s*y4)/4;
	
	dxdt=((1+s)*x1-(1+s)*x2-(1-s)*x3+(1-s)*x4)/4;
	dxds=((1+t)*x1+(1-t)*x2-(1-t)*x3-(1+t)*x4)/4;
	dydt=((1+s)*y1-(1+s)*y2-(1-s)*y3+(1-s)*y4)/4;
	dxdt=((1+t)*y1+(1-t)*y2-(1-t)*y3-(1+t)*y4)/4;
	
	
	J=dxdt*dyds-dxds*dydt;
	
	
	////derivadas parciales para el valor maximo y minimo de la variable dependiente
	//dxdslow=(p2.x-p1.x)/2;
	//dxdshigh=(p3.x-p4.x)/2;
	
	//dxdtlow=(p4.x-p1.x)/2;
	//dxdthigh=(p3.x-p2.x)/2;
	
	//dydslow=(p2.y-p1.y)/2;
	//dydshigh=(p3.y-p4.y)/2;
	
	//dydtlow=(p4.y-p1.y)/2;
	//dydthigh=(p3.y-p2.y)/2;
	
	//j1=dxdslow*dydtlow-dxdtlow*dydslow;
	//j2=dxdshigh*dydtlow-dxdtlow*dydshigh;
	//j3=dxdshigh*dydthigh-dxdthigh*dydshigh;
	//j4=dxdslow*dydthigh-dxdthigh*dydslow;

	//min=j1;
	//min=(j2<min)?j2:min;
	//min=(j3<min)?j3:min;
	//min=(j4<min)?j4:min;

	//avg=(j1+j2+j3+j4)/4;
	
	//return min;
	//return avg;
	
}


/*********************************************************/
/*************                     ***********************/
/*************     MESH FUNCTIONS  ***********************/
/*************                     ***********************/
/*********************************************************/


int //devuelve 1 si una cara tiene el nodo con ID dado como uno de sus vertices, 0 otherwise
nodoPerteneceACara(	int		nodo_id,
					cara	cara_elem)
{
	int i;
	int resultado=0;
	
	for (i = 0; i < 3; i++){
		if (cara_elem.id[i]==nodo_id){
			resultado=1;
			break;
		}
	}
	return resultado;
}

int	//devuelve 1 si una lista de aristas contiene almenos una que conecte los puntos con ID ini y fin, en cualquier orden
aristaPerteneceALista(	int				ini,
						int				fin,
						lista_aristas*	lista)
{
	long int i;
	int result;
	for (i = 0; i < lista->n_total; i++){
		result=0;
		result+=(ini==lista->aristas[i].inicio_id)?1:0;
		result+=(fin==lista->aristas[i].inicio_id)?1:0;
		result+=(ini==lista->aristas[i].final_id)?1:0;
		result+=(fin==lista->aristas[i].final_id)?1:0;
		if (result==2){
			return 1;
		}
		
	}
	return 0;
	
}

int //de una lista, devuelve 0 si hay alguna arista que contiene al nodo con ID dado
nodoPerteneceALista(	int 			nodo, 
						lista_aristas* 	lista)
{
						
	long int i;
	int resultado=0;
	
	for(i=0;i<lista->n_total;i++){
		if((lista->aristas[i].inicio_id==nodo) || (lista->aristas[i].final_id==nodo)){
			resultado=1;
			break;
		}
	}
	return resultado;
}


int //devuelve 1 si una cara tiene area 0 (vertices colineales)
caraTieneAreaCero(	punto a,
					punto b,
					punto c)
{
	double dist_ab=0;
	double dist_ca=0;
	double dist_bc=0;
	
	dist_ab=sqrt((a.x-b.x)*(a.x-b.x)+(a.y-b.y)*(a.y-b.y)+(a.z-b.z)*(a.z-b.z));
	dist_bc=sqrt((c.x-b.x)*(c.x-b.x)+(c.y-b.y)*(c.y-b.y)+(c.z-b.z)*(c.z-b.z));
	dist_ca=sqrt((a.x-c.x)*(a.x-c.x)+(a.y-c.y)*(a.y-c.y)+(a.z-c.z)*(a.z-c.z));
	
	if ((dist_ab == 0)	||
		(dist_ca == 0)	||
		(dist_bc == 0)	||
		(dist_ab >= dist_ca + dist_bc)	||
		(dist_ca >= dist_ab + dist_bc)	||
		(dist_bc >= dist_ca + dist_ab)	)
	{
		return 1;
	}else{
		return 0;
	}

	
}


int	//devuelve 1 si un punto esta en el interior de un triangulo dado por vertices (a,b,c). SE ASUME QUE EL PUNTO Y EL TRIANGULO SON COPLANARES
puntoPerteneceATriangulo(	punto	p,
							punto	a,
							punto	b,
							punto	c)
{
	if (caraTieneAreaCero(a,b,c)){
		return 0;
	}else{
		return ((prodVectorialUnitario(p,a,b) == prodVectorialUnitario(p,b,c)) && (prodVectorialUnitario(p,b,c) == prodVectorialUnitario(p,c,a)));
	}
}


void	//salida por pantalla del contenido de una lista de aristas
imprimirListaAristas(	lista_aristas a)
{
	int	i;
	printf("\nEDGE LIST:\n----------------------");
	for (i = 0; i < a.n_total+1; i++){
		printf("\n%ld\t%ld",a.aristas[i].inicio_id,a.aristas[i].final_id);
	}
	
}



int	//crea un triangulo uniendo los puntos (ID1,ID2,ID3) y lo anade al final de una lista de caras
anadirCara( long int 			id0,
			long int 			id1,
			long int 			id2,
			lista_caras* 		c)
{
	c->coord[c->n_total].id[0]=id0;
	c->coord[c->n_total].id[1]=id1;
	c->coord[c->n_total].id[2]=id2;
	c->n_total++;
	return 0;
}

int	//agrega una arista a una lista, dados los vertices que conecta dicha arista (el orden de los dos vertices ES relevante)
anadirArista(	long int 			id_i, 
				long int 			id_f, 
				lista_aristas*		lista)
{
	lista->aristas[ (lista->n_total) ].inicio_id=id_i;
	lista->aristas[ (lista->n_total) ].final_id=id_f;
	lista->aristas[ (lista->n_total) ].utilizada=0;
	lista->n_total++;
	//printf (" %d $ %d \n",id_i,id_f);
	return 0;
}

int	//elimina todas las aristas de una lista que unan el vertice i y el j
eliminarArista (	int				id_i,
					int				id_f,
					lista_aristas*	lista)
{
	long int i,j;
	int exito=0;

	for (i = 0; i < lista->n_total; i++){
		if (
			((lista->aristas[i].inicio_id==id_i) && (lista->aristas[i].final_id==id_f)) ||
			((lista->aristas[i].inicio_id==id_f) && (lista->aristas[i].final_id==id_i))
		){
			for (j = i; j < lista->n_total; j++){
				lista->aristas[j].inicio_id=lista->aristas[j+1].inicio_id;
				lista->aristas[j].final_id=lista->aristas[j+1].final_id;
				lista->aristas[j].utilizada=lista->aristas[j+1].utilizada;
			}
			lista->n_total--;
			exito=1;
			
		}
		
	}
	return exito;
	
}



lista_aristas  //en una lista de aristas, busca un circuito cerrado (de un vertice a si mismo sin pasar dos veces por la misma arista ni por el mismo punto)
ordenarAristas(	lista_aristas 	lista,
				lista_puntos*	p,
				int 			abierto) //0 para busqueda de ciclo cerrado por id de nodos, 1 para buscar inicio y final que tengan las mismas coordenadas
{
	long int i;
	int terminado=0;
	lista_aristas trayectoria[10]; for (i=0;i<10;i++) trayectoria[i].n_total=0;
	long int punto_actual, punto_inicial;	//inicio de la trayectoria
	int tray_id; //contabiliza cuantos ciclos cerrados se han formado con las aristas dadas
	long int n_aristas_procesadas=0;

	
	for(i=0;i<lista.n_total;i++) {
		if(!lista.aristas[i].utilizada) {
			punto_inicial=lista.aristas[i].inicio_id;
			break;
		}
	}
	punto_actual=punto_inicial;
	
	tray_id=0;
	
	do{
		for (i=0; i<lista.n_total; i++) {
			if (!lista.aristas[i].utilizada) {
				if (lista.aristas[i].inicio_id==punto_actual) {
					anadirArista(punto_actual,lista.aristas[i].final_id, &(trayectoria[tray_id]));
					punto_actual=lista.aristas[i].final_id;
					lista.aristas[i].utilizada=1;
					n_aristas_procesadas++;
					break;
				}
				if (lista.aristas[i].final_id==punto_actual) {
					anadirArista (punto_actual,lista.aristas[i].inicio_id, &(trayectoria[tray_id]) );
					punto_actual=lista.aristas[i].inicio_id;
					lista.aristas[i].utilizada=1;
					n_aristas_procesadas++;
					break;
				}
			}
		}
		if (abierto){ //condicion de terminacion para 
			double diff=0;
			diff+=(p->coord[punto_actual].x-p->coord[punto_inicial].x);
			diff+=(p->coord[punto_actual].y-p->coord[punto_inicial].y);
			diff+=(p->coord[punto_actual].z-p->coord[punto_inicial].z);
			diff=fabs(diff);
			terminado=(diff<TOLERANCIA) ? 1 : 0; 
		}else{
			terminado=(punto_inicial==punto_actual) ? 1: 0;
		}
	} while (!terminado);


	return trayectoria[0]; //posiblemente non necesite calcular varias trayectorias, 1 basta
	 
}


int	//devuelve 1 si dos puntos tienen las mismas coordenadas, partiendo de sus IDs
compararCoordenadas(	int				p1_id,
						int				p2_id,
						lista_puntos*	n)
{
	punto	p1,p2;
	double x[2];
	double y[2];
	double z[2];
	
	p1=n->coord[p1_id];
	p2=n->coord[p2_id];
	
	x[0]=p1.x;
	y[0]=p1.y;
	z[0]=p1.z;
	x[1]=p2.x;
	y[1]=p2.y;
	z[1]=p2.z;
	
	if(x[0]==x[1]){
		if(y[0]==y[1]){
			if(z[0]==z[1]){
				return 1;
			}
		}
	}
	
	return 0;
	
}

//TODO REVISAR: POR QUE SOLO LEE O ULTIMO PUNTO DA CARA
int	//para un triangulo de la malla y dos planos de corte, identifica con un numero del 0 al 3 en que cuadrante esta el triangulo: (-,-)=0 (+,-)=1 (-,+)=2 (+,+)=3 
identificarCuadranteCara(	cara			c,
							lista_puntos*	n,
							double 			posicionCorteY,
							double 			posicionCorteZ	)
{
	int semiplanoY=NO_INICIALIZADO; //1 si y>ycorte, 0 si no
	int semiplanoZ=NO_INICIALIZADO;
	int i;
	punto p;

	for (i = 0; i < 3; i++){
		p=n->coord[c.id[i]];
		if(p.y>posicionCorteY){
			semiplanoY=1;
		}else if (p.y<posicionCorteY){
			semiplanoY=0;
		}	
		if(p.z>posicionCorteZ){
			semiplanoZ=1;
		}else if (p.z<posicionCorteZ){
			semiplanoZ=0;
		}	
	
	}
	return semiplanoY+2*semiplanoZ;
}


cara	//crea una copia de una cara
copiarCara(cara c1)
{
	int i;
	cara c2;
	for(i=0;i<3;i++){
		c2.id[i]=c1.id[i];
	}
	return c2;
}


lista_aristas	//dadas dos listas de aristas, develve una tercera con aquellas aristas que estan en ambas (las coordenadas de los extremos son las mismas, independientemente de sus ID)
interseccionListas(	lista_aristas* 	a1,
					lista_aristas* 	a2,
					long int*		alt,
					lista_puntos*	n)
{
	long int i,j;
	lista_aristas aux; aux.n_total=0;
	for (i = 0; i < a1->n_total; i++){
		//if(aristaPerteneceALista(alt[a1->aristas[i].inicio_id],alt[a1->aristas[i].final_id],a2)){
			//anadirArista(alt[a1->aristas[i].inicio_id],alt[a1->aristas[i].final_id],&aux);
		//}
		for(j=0; j<a2->n_total;j++){
			if((compararCoordenadas(	a1->aristas[i].inicio_id,	a2->aristas[j].inicio_id, 	n) &&
				compararCoordenadas(	a1->aristas[i].final_id,	a2->aristas[j].final_id,	n)) 
				|| 
				(compararCoordenadas(	a1->aristas[i].inicio_id,	a2->aristas[j].final_id, 	n) &&
				compararCoordenadas(	a1->aristas[i].final_id,	a2->aristas[j].inicio_id,		n) ) )
			{
				anadirArista(a2->aristas[j].inicio_id,a2->aristas[j].final_id,&aux);
			}
		}
	}
	return aux;
}


/*********************************************************/
/*************                     ***********************/
/*************OPTIMIZATION CRITERIA***********************/
/*************                     ***********************/
/*********************************************************/


double	//para una polilinea2D y un centro, da una puntuacion de -1 a 1 al centro. Basado en el minimo angulo de incidencia de los rayos sobre el contorno
calidadCentro	(	punto			centro,
					lista_puntos	a)
{
	int i,j,k;
	vector vector1,vector2,vector3,producto1,aux1,aux2;
	lista_puntos poligono;
	double calidad=1; //entre 0 y 100%, vamos reduciendo segun hallamos puntos con una calidad menor
	
	punto origen = {0,0,0};
	vector normal = {0,0,1};
	poligono=proyectarSobrePlano3d(a,origen,normal);
	

	for (k = 0; k < poligono.n_total; k++)
	{
		
		i= (k) % poligono.n_total;
		j= (k+1) % poligono.n_total;
		
		aux1=poligono.coord[i];
		aux2=poligono.coord[j];
		
		vector1=vectorSuma(aux1,vectorPorEscalar(centro,-1));
		vector2=vectorSuma(aux2,vectorPorEscalar(aux1,-1));
		vector3=vectorSuma(aux2,vectorPorEscalar(centro,-1));
		
		double angulo=anguloVectores(vector1,vector2);
		calidad=(sin(angulo)<calidad) ? sin(angulo) : calidad;
		
	}

	return calidad;
}


double	//para una polilinea2D y un centro, da una puntuacion de -1 a 1 al centro. Basado en el minimo angulo abarcado entre puntos consecutivos
calidadBarrido	(	punto			centro,
					lista_puntos	a)
{
	int i,j,k;
	vector vector1,vector2,vector3,producto1;
	punto aux1,aux2;
	lista_puntos poligono;
	double calidad=1; //entre 0 y 100%, vamos reduciendo segun hallamos puntos con una calidad menor
	
	punto origen = {0,0,0};
	vector normal = {0,0,1};
	poligono=proyectarSobrePlano3d(a,origen,normal);
	

	for (k = 0; k < poligono.n_total; k++)
	{
		
		i= (k) % poligono.n_total;
		j= (k+1) % poligono.n_total;
		
		aux1=poligono.coord[i];
		aux2=poligono.coord[j];
		
		vector1=vectorSuma(aux1,vectorPorEscalar(centro,-1));
		vector2=vectorSuma(aux2,vectorPorEscalar(aux1,-1));
		vector3=vectorSuma(aux2,vectorPorEscalar(centro,-1));
		
		double angulo=anguloVectores(vector1,vector3);
		calidad=(sin(angulo)<calidad) ? sin(angulo) : calidad;
		
	}

	return calidad;
}

double
calidadJacobiano	( 	punto			centro,
						lista_puntos	a)
{
	int i,j,k;
	vector vector1,vector2,vector3,producto1;
	punto aux1,aux2,aux3,aux4;
	lista_puntos poligono;
	double calidad=9999;
	
	punto origen = {0,0,0};
	vector normal = {0,0,1};
	poligono=proyectarSobrePlano3d(a,origen,normal);
	

	for (k = 0; k < poligono.n_total; k++)
	{
		
		i= (k) % poligono.n_total;
		j= (k+1) % poligono.n_total;
		
		aux4=poligono.coord[i];
		aux3=poligono.coord[j];
		double ratio=1.0-1.0/(double)DIV_R;
		aux1=vectorPorEscalar(vectorSuma(centro,aux4),ratio);
		aux2=vectorPorEscalar(vectorSuma(centro,aux3),ratio);
		
		double area=moduloVector(vectorResta(aux3,aux4))*moduloVector(vectorResta(aux3,aux2));
		double calidadElemento=jacobianoElementoCuadrado(aux1,aux2,aux3,aux4,0,1)/area;
		calidad=(calidadElemento<calidad) ? calidadElemento : calidad;
		
	}

	return calidad;
	
}


double	//para una polilinea3D y un centro, da una puntuacion al centro. Mejor cuanto menores sean los angulos entre el plano y los vectores centro-contorno (minimos cuadrados)
calidadAltura	(	double		offset,
					double*		v,
					double*		h,
					int			n)
{
	double anguloRad;
	double calidad=0;
	int i;
	
	//INIT
	anguloRad=0;
	
	//Suma de cuadrados de los angulos, buscamos minimizarlo
	for (i = 0; i < n; i++){
		anguloRad=atan((v[i]+offset)/h[i]);
		calidad+=anguloRad*anguloRad;
	}
	
	return calidad;
}



/*********************************************************/
/*************                     ***********************/
/*************    INPUT OUTPUT     ***********************/
/*************                     ***********************/
/*********************************************************/




void	//extrae de un .ply las coordenadas de un punto concreto, como (x,y,z)
procesar_punto (	char 			*line,
					lista_puntos 	*nodos)
{
	double x=0,y=0,z=0;
	
	sscanf (line,"%lf %lf %lf ",&x,&y,&z);
	nodos->coord[nodos->n_total].x=x;
	nodos->coord[nodos->n_total].y=y;
	nodos->coord[nodos->n_total].z=z;
	nodos->n_total++;
}


void 	//extrae de un .ply la definicion de un triangulo concreto de la malla, como (vertice1,vertice2,vertice3)
procesar_cara(	char 			*line, 
				lista_caras 	*caras)
{
	long int id1,id2,id3;
	int n;
	
	sscanf (line,"%d %ld %ld %ld ",&n,&id1,&id2,&id3);
	caras->coord[caras->n_total].id[0]=id1;
	caras->coord[caras->n_total].id[1]=id2;
	caras->coord[caras->n_total].id[2]=id3;
	caras->n_total++;
}

void	//durante la lectura de un .ply, detecta en que seccion del archivo esta leyendo: header, lista de coordenadas o conectividad.
procesar_header(	char*			line,
					long int*		vertexSize,
					int*		region)
{
	long int aux=0;
	
	if (!strcmp(line,"end_header\n")){
		*region=1;
	}
	
	if (sscanf (line,"element vertex %ld",&aux)){
		*vertexSize=aux;
	}			
}

				

long int  //extrae de un .ply las coordenadas de todos sus vertices
leer_nodos(	char 			filename[],
			lista_puntos 	*nodos)
{
	int size = 1024, pos;
	long int lineas_leidas=0;
    int c;
    char *buffer = (char *)malloc (size);
	FILE *f = fopen (filename, "r"); //PROYECTO, IDENTIFICAR TRAMO DE COORDENADAS EN .ply
	if(f) {
		do { // read all lines in file
			pos = 0;
			do{ // read one line
				c = fgetc(f);
				if (c != EOF) buffer[pos++] = (char)c;
				if (pos >= size - 1) { // increase buffer length - leave room for 0
					size *=2;
					buffer = (char*)realloc(buffer, size);
				}
			}while (c != EOF && c != '\n');
			buffer[pos] = 0;
	        // line is now in buffer
			procesar_punto(buffer, nodos);
			lineas_leidas++;
		} while (c != EOF); 
		fclose (f);           
	}
	free (buffer);
	nodos->n_total--;
	printf("\nNodes section readed: %ld items found",lineas_leidas-1);
	return lineas_leidas-1;	
}


long int 	//extrae de un .ply la informacion sobre que vertices estan unidos a que otros vertices mediante una sola arista
leer_caras(	char 			filename[],
			lista_caras 	*caras)
{
	int size = 1024, pos;
    int c;
    long int lineas_leidas=0;
    char *buffer = (char *)malloc (size);
    
	FILE *f = fopen (filename, "r"); //PROYECTO, IDENTIFICAR TRAMO DE CARAS EN .ply
	if(f) {
		do { // read all lines in file
			pos = 0;
			do{ // read one line
				c = fgetc (f);
				if (c != EOF) buffer[pos++] = (char)c;
				if (pos >= size - 1) { // increase buffer length - leave room for 0
					size *=2;
					buffer = (char*)realloc (buffer, size);
				}
			}while (c != EOF && c != '\n');
			buffer[pos] = 0;
	        // line is now in buffer
	        procesar_cara (buffer, caras);
	        lineas_leidas++;
		} while (c != EOF); 
	    fclose (f);           
	}
	free (buffer);
	caras->n_total--;
	printf("\nFaces section readed: %ld item found",lineas_leidas-1);
	return lineas_leidas-1;
}



void	//lee de disco las coordenadas x,y,z de cada vertice de la malla triangulada (ply). Tambien lee que puntos estan unidos a cuales.
leerFichero(	char			filename[],
				lista_puntos*	nodos,
				lista_caras*	caras,
				long int*		n_nodos,
				long int*		n_caras)
{
	int size = 1024, pos;
    int c;
    long int lineas_leidas=0;
    long int vertexStartLine=0;
    long int vertexSize=0;
    int region=0; //0=header, 1=puntos, 2=caras
    
    char *buffer = (char *)malloc (size);
    
	FILE *f = fopen (filename, "r"); //PROYECTO, IDENTIFICAR TRAMO DE CARAS EN .ply
	if(f) {
		do { // read all lines in file
			pos = 0;
			do{ // read one line
				c = fgetc (f);
				if (c != EOF) buffer[pos++] = (char)c;
				if (pos >= size - 1) { // increase buffer length - leave room for 0
					size *=2;
					buffer = (char*)realloc (buffer, size);
				}
			}while (c != EOF && c != '\n');
			buffer[pos] = 0;
			
			if(region==0){
				procesar_header(buffer,&vertexSize,&region);
				if (region==1){
					vertexStartLine=lineas_leidas;
				}
			}else if(lineas_leidas <= vertexStartLine+vertexSize){
				procesar_punto(buffer,nodos);
			}else{
				procesar_cara(buffer,caras);
			}
			
	        // line is now in buffer
	        //if(lineas_leidas>=vertexStartLine){
				//procesar_punto (buffer, nodos);
			//}else if(lineas_leidas>=faceStartLine){
				//procesar_cara (buffer, caras);
			//}else{
				//procesar_header(buffer,&vertexStartLine,&faceStartLine);
			//}
	        lineas_leidas++;
		} while (c != EOF); 
	    fclose (f);
	    caras->n_total--; //aviso: antes estaba fora do if
		*n_nodos=nodos->n_total;
		*n_caras=caras->n_total;           
	}else{
		printf("\nFile\"%s\" not found, do you want to try other file?:\n",filename);
		scanf("%s",filename);
		leerFichero(filename,nodos,caras,n_nodos,n_caras);	
	}
	free (buffer);

	
}


void	//pregunta al usuario las coordenadas de los dos planos de corte, en Y y en Z
preguntarCentroGravedad(	lista_puntos*	nodos,
							double*			posY,
							double*			posZ)
{
	long int i;
	double posX=NO_INICIALIZADO;


	float input;
	int inputCount;
	
	
	double zMax=nodos->coord[0].z;
	double xMax=nodos->coord[0].x;
	double xMin=nodos->coord[0].x;
	double yMax=nodos->coord[0].y;
	double zMin=nodos->coord[0].z;
	double yMin=nodos->coord[0].y;
	
	
	long int n_nodos=nodos->n_total;
	for (i=0; i<n_nodos; i++) {
		*posZ+=nodos->coord[i].z;
		*posY+=nodos->coord[i].y;
		posX+=nodos->coord[i].x;
		xMin=(xMin > nodos->coord[i].y) ? nodos->coord[i].x : xMin;
		yMin=(yMin > nodos->coord[i].y) ? nodos->coord[i].y : yMin;
		zMin=(zMin > nodos->coord[i].z) ? nodos->coord[i].z : zMin;
		xMax=(xMax < nodos->coord[i].y) ? nodos->coord[i].y : xMax;
		yMax=(yMax < nodos->coord[i].y) ? nodos->coord[i].y : yMax;
		zMax=(zMax < nodos->coord[i].z) ? nodos->coord[i].z : zMax;

	}
	*posZ/=n_nodos;	
	posX/=n_nodos;	
	*posY/=n_nodos;
	
	
	printf("\nIndique posicion de corte en Y (escribir c para usar centro de gravedad):\n");
	inputCount=scanf("%f",&input);
	if (inputCount > 0){
		*posY=input;
	}

	
	printf("\nIndique posicion de corte en Z (escribir c para usar centro de gravedad):\n");
	inputCount=scanf("%f",&input);
	if (inputCount > 0){
		*posZ=input;
	}
	

	
}




/*********************************************************/
/*************                     ***********************/
/*************    MESH EDITING     ***********************/
/*************                     ***********************/
/*********************************************************/


void	//a partir de una cara, decide si esta está encima o debajo de un plano de corte. Si está en ambas partes a la vez, divide la cara en 3 fragmentos triangulares que estan cada uno totalmente contenido en una de las mitades
subdividirCara2(lista_puntos 	*nodos,
				lista_caras		*lista,
				int				cara_id,
				sustitucion*	r,
				int				n,
				double			altura,
				int				dir) 
{
	long int i,j;
	int a[6]; // 1 si un vertice de la cara pertenece a la arista estudiada
	long int id[3]; //id global de los nodos que forman la cara
	long int ladolargo1,ladolargo2,verticeAislado; //identidad de los nodos segun su posicion respecto al plano de corte
	int sustitucionAplicable[]={NO_INICIALIZADO,NO_INICIALIZADO}; //las dos aristas de la cara que resultan cortadas, y sus reemplazos
	int trianguloInvertido=0; //0 para triangulos con el vertice aislado en valores positivos "apunta arriba", 1 el resto
	sustitucion s1, s2;
	
	for(i=0;i<3;i++){
		id[i]=lista->coord[cara_id].id[i];
	}
	
	j=0;
	for (i=0; i<n; i++) {
		a[0]=(r[i].fin==id[0]) ? 1 : 0;
		a[1]=(r[i].fin==id[1]) ? 1 : 0;
		a[2]=(r[i].fin==id[2]) ? 1 : 0;
		a[3]=(r[i].ini==id[0]) ? 1 : 0;
		a[4]=(r[i].ini==id[1]) ? 1 : 0;
		a[5]=(r[i].ini==id[2]) ? 1 : 0;
		if( a[0]+a[1]+a[2]+a[3]+a[4]+a[5] ==  2){
			sustitucionAplicable[j++]=i;
		}
	}	
	
	if (sustitucionAplicable[0]==NO_INICIALIZADO) 
		return;
	else if (sustitucionAplicable[1]==NO_INICIALIZADO) 
		return;
	else{
		s1=r[sustitucionAplicable[0]];
		s2=r[sustitucionAplicable[1]];
			
			// localizar lado aislado y vertice aislado
			
			if(s1.ini==s2.ini || s1.ini==s2.fin){
				ladolargo1=s1.fin; //nota: ladolargo1 siempre pertenece a la arista de s1
				verticeAislado=s1.ini;
			}else{
				ladolargo1=s1.ini;
				verticeAislado=s1.fin;
			}
			
			if(s2.ini==s1.ini || s2.ini==s1.fin){
				ladolargo2=s2.fin;
			}else{
				ladolargo2=s2.ini;
			}
			
			switch(dir){
			case 0:
				trianguloInvertido=((nodos->coord[verticeAislado].x) > (nodos->coord[ladolargo1].x)) ? 0 : 1;
				break;
			case 1:
				trianguloInvertido=((nodos->coord[verticeAislado].y) > (nodos->coord[ladolargo1].y)) ? 0 : 1;
				break;
			case 2:
				trianguloInvertido=((nodos->coord[verticeAislado].z) > (nodos->coord[ladolargo1].z)) ? 0 : 1;
				break;
			}
			
			
			
			if (trianguloInvertido)
			{
				lista->coord[cara_id].id[0]=verticeAislado;
				lista->coord[cara_id].id[1]=s1.medioBajo;
				lista->coord[cara_id].id[2]=s2.medioBajo;
				anadirCara(s2.medioAlto,s1.medioAlto,ladolargo1,lista);
				anadirCara(s2.medioAlto,ladolargo1,ladolargo2,lista);
			}else{
				lista->coord[cara_id].id[0]=verticeAislado;
				lista->coord[cara_id].id[1]=s1.medioAlto;
				lista->coord[cara_id].id[2]=s2.medioAlto;
				anadirCara(s2.medioBajo,s1.medioBajo,ladolargo2,lista);
				anadirCara(s1.medioBajo,ladolargo1,ladolargo2,lista);
			}
			
			
	}
}


lista_aristas	//Con una lista de aristas, propone un orden de recorrido de nodos, en los que todas las aristas y nodos se recorren como máximo una vez. Crea una polilinea con esos nodos en orden
filtrarCiclos(lista_aristas p)
{
	long int i, j;
	long int lista[T_MAX]; for (i = 0; i < T_MAX; i++) lista[i]=0;
	long int pos=2;
	lista_aristas aux; aux.n_total=0;

	
	for(i=0;i<p.n_total;i++){
		p.aristas[i].utilizada=0;
	}
	
	lista[0]=p.aristas[0].inicio_id;
	lista[1]=p.aristas[0].final_id;
	p.aristas[0].utilizada=1;
	anadirArista(p.aristas[0].inicio_id,p.aristas[0].final_id,&aux);
	

	
	j=1;
	
	do{
		for(i=0;i<pos;i++){
			if ((p.aristas[j].inicio_id==lista[i]) && !p.aristas[j].utilizada){
				anadirArista(p.aristas[j].inicio_id,p.aristas[j].final_id,&aux);
				p.aristas[j].utilizada=1;
				lista[pos++]=p.aristas[j].final_id;
				j=1;
				break;
			}
			if ((p.aristas[j].final_id==lista[i]) && !p.aristas[j].utilizada){
				anadirArista(p.aristas[j].inicio_id,p.aristas[j].final_id,&aux);
				p.aristas[j].utilizada=1;
				lista[pos++]=p.aristas[j].inicio_id;
				j=1;
				break;
			}
		}
		j++;
	}while(j<p.n_total);
		
	return aux;
}


lista_aristas	//elimina de una lista todas aquellas aristas que sean coincidentes (extremos en las mismas coordenadas) a otra arista de la lista, dejando solo una copia.
filtrarRepetidas(lista_aristas p)
{
	lista_aristas aux; aux.n_total=0;
	long int i,j;
	long int ini, fin;
	for (i=0; i<p.n_total; i++){
		p.aristas[i].utilizada=0;
	}
	
	for (i=0; i<p.n_total; i++) {
		ini=p.aristas[i].inicio_id;
		fin=p.aristas[i].final_id;
		if(!p.aristas[i].utilizada){
			anadirArista(ini,fin,&aux);
			for (j=0; j<p.n_total; j++) {
				if ((ini==p.aristas[j].inicio_id) && (fin==p.aristas[j].final_id))
					p.aristas[j].utilizada=1;
				if ((fin==p.aristas[j].inicio_id) && (ini==p.aristas[j].final_id))
					p.aristas[j].utilizada=1;
			}
		}
	}
	
	return aux;
	
}

int *	//crea una lista de nodos homólogos. Cuando un vértice esta sobre el plano de corte, hay dos versiones del mismo, cada una de las versiones perteneciente a una de las mitades. Ambas versiones tienen las mismas coordenadas pero distinto ID
generarAlternativas (	lista_aristas 		c,
						lista_puntos*		n,
						double				altura,
						int					dir,
						sustitucion*		aux,
						long int*			alt)
{
	
	
	long int i;

	double x[2];
	double y[2];
	double z[2];
	double ratio;

	
	for (i=0;i<c.n_total;i++) {
		aux[i].ini=c.aristas[i].inicio_id;
		aux[i].fin=c.aristas[i].final_id;
		x[0]=n->coord[c.aristas[i].inicio_id].x;
		y[0]=n->coord[c.aristas[i].inicio_id].y;
		z[0]=n->coord[c.aristas[i].inicio_id].z;
		x[1]=n->coord[c.aristas[i].final_id].x;
		y[1]=n->coord[c.aristas[i].final_id].y;
		z[1]=n->coord[c.aristas[i].final_id].z;
		
		
		switch(dir){
			case 0:
				ratio=( x[0] - altura) / (x[0] - x[1]);
				//inferior= (x[0]>x[1]) ? 1 : 0;
				break;
			case 1:
				ratio=( y[0] - altura) / (y[0] - y[1]);
				//inferior= (y[0]>y[1]) ? 1 : 0;
				break;
			case 2:
				ratio=( z[0] - altura) / (z[0] - z[1]);
				//inferior= (z[0]>z[1]) ? 1 : 0;
				break;
		}
	
		aux[i].x= x[0] + (x[1] - x[0])*ratio;
		aux[i].y= y[0] + (y[1] - y[0])*ratio;
		aux[i].z= z[0] + (z[1] - z[0])*ratio;
		
		n->coord[n->n_total].x=aux[i].x;
		n->coord[n->n_total].y=aux[i].y;
		n->coord[n->n_total].z=aux[i].z;
		aux[i].medioBajo=n->n_total;
		n->n_total++;
		
		n->coord[n->n_total].x=aux[i].x;
		n->coord[n->n_total].y=aux[i].y;
		n->coord[n->n_total].z=aux[i].z;
		aux[i].medioAlto=n->n_total;
		n->n_total++;
		
		
		//vinculando nodos
		alt[aux[i].medioBajo]=aux[i].medioAlto;
		alt[aux[i].medioAlto]=aux[i].medioBajo;
		
	}
	
	return 0;
}




int //para una malla triangulada, comprueba todas sus caras para ver si se ven afectadas por el plano de corte. Si es asi, las divide en caras mas pequenas, contenidas completamente en una de las mitades
seccionar2(	lista_puntos* 	nodos,
			lista_caras*	caras, 
			double 			altura,
			int 			dir,
			long int*		alt)
{
	int semifigura[T_MAX];
	long int i;
	int a1, a2, a3;
	long int id1, id2, id3;
	lista_aristas corte; corte.n_total=0;	//lista de aristas que cruzan el plano de corte, sin duplicados
	lista_aristas corteSucio; corteSucio.n_total=0; //lista provisional de aristas que cruzan el plano de corte, puede contener duplicados
	sustitucion * reemplazos;	//tabla de equivalencias entre dos nodos con las mismas coordenadas pero que residen a distintos lados del corte
	int carasAfectadas[T_MAX]; for(i=0;i<T_MAX;i++) carasAfectadas[i]=NO_INICIALIZADO;	//lista de IDS de caras que tocan el plano de corte
	long int cara_id=0;
	
	
	//para cada punto de la malla, decide si esta encima o debajo del plano de corte. El plano de corte puede orientarse en una de las direcciones principales
	
		switch (dir){
			case 0: //plano YZ
				printf("\ncutting figure in YZ plame ... \n\t");
				for (i=0; i< nodos->n_total; i++) 
					semifigura[i]=nodos->coord[i].x>=altura?1:0;
				break;
			case 1: //plano XZ
				printf("\ncutting figure in XZ plane ... \n\t");
				for (i=0; i< nodos->n_total; i++) 
					semifigura[i]=nodos->coord[i].y>=altura?1:0;
				break;
			case 2: //plano XY
				printf("\ncutting figure in XY plane ... \n\t");
				for (i=0; i<nodos->n_total; i++) 
					semifigura[i]=nodos->coord[i].z>=altura?1:0;
				break;
	}
	
	//para cada cara, comprueba si sus 3 vertices estan o bien todos encima o bien todos debajo del plano. Las que no cumplen, se cortan en trozos
	
	
	long int lim=(caras->n_total)-1;
	
	for (i=0; i<lim; i++){
		
		//ids de los vertices de la cara
		id1=caras->coord[i].id[0];
		id2=caras->coord[i].id[1];
		id3=caras->coord[i].id[2];

		//flags de dichos puntos: 1 si estan encima, 0 si debajo
		a1=semifigura[id1];
		a2=semifigura[id2];
		a3=semifigura[id3];
		
		//casos problematicos, 2 puntos en un lado y 1 en otro. Se listan todas las combinaciones
		if(a1 && a2 && !a3){
			anadirArista(id1,id3,&corteSucio);
			anadirArista(id2,id3,&corteSucio);
			carasAfectadas[cara_id++]=i;
		}
		if(a1 && a3 && !a2){
			anadirArista(id1,id2,&corteSucio);
			anadirArista(id3,id2,&corteSucio);
			carasAfectadas[cara_id++]=i;
		}
		if(a3 && a2 && !a1){
			anadirArista(id3,id1,&corteSucio);
			anadirArista(id2,id1,&corteSucio);
			carasAfectadas[cara_id++]=i;
		}
		
		if(a1 && !a3 && !a2){
			anadirArista(id1,id2,&corteSucio);
			anadirArista(id3,id1,&corteSucio);
			carasAfectadas[cara_id++]=i;
		}
		
		if(!a1 && !a3 && a2){
			anadirArista(id1,id2,&corteSucio);
			anadirArista(id3,id2,&corteSucio);
			carasAfectadas[cara_id++]=i;
		}
		
		if(!a1 && a3 && !a2){
			anadirArista(id1,id3,&corteSucio);
			anadirArista(id3,id2,&corteSucio);
			carasAfectadas[cara_id++]=i;
		}
		
	}
	
	//una vez decididas que aristas son candidatas a ser cortadas, se crean nuevos nodos en el medio, se crean nuevas aristas uniendo con el nuevo nodo y se eliminan las aristas antiguas 
	corte=filtrarRepetidas(corteSucio);
	reemplazos=(sustitucion*)calloc(corte.n_total,sizeof(sustitucion));
	generarAlternativas(corte,nodos,altura,dir,reemplazos,alt);
	
	//se cortan todas las caras candidatas
	for(i=0;i<cara_id;i++){
		subdividirCara2(nodos,caras, carasAfectadas[i], reemplazos, corte.n_total, altura, dir);
	}
	
	printf("Completed\n\tNew number of faces: %ld ",caras->n_total);
	return caras->n_total;
	
}



void	//reemplaza todas las apariciones de un vertice por otro (las caras que contenian a ese vertice ahora contienen el nuevo)
sustituirId	(	long int 		id_orig,
				long int		id_dest,
				lista_caras*	c,
				lista_puntos*	n)
{
	long int i,j;
	for (i = 0; i < c->n_total; i++){
		for (j = 0; j < 3; j++) {
			if (c->coord[i].id[j]==id_orig){
				c->coord[i].id[j]=id_dest;
			}
		}
	}
}

void	//en una lista de aristas, asigna la misma ID a todos los vertices con las mismas coordenadas. USAR CON CUIDADO, PUEDE DESHAER .
reemplazarId(	long int 		a,
				lista_caras*	c,
				lista_puntos*	n)
{
	long int i,j;
	punto pt1,pt2;
	pt2=n->coord[a];
	for (i = 0; i < c->n_total; i++){
		for (j = 0; j < 3; j++) {
			pt1=n->coord[c->coord[i].id[j]];
			if ((pt1.x==pt2.x) && (pt1.y==pt2.y) && (pt1.z==pt2.z)){
				//printf("\nreeemplazado nodo %ld\tpor\t%ld",c->coord[i].id[j],a);
				c->coord[i].id[j]=a;
				
			}
			
		}
	}
	
}



void	//los vertices que se situan sobre ambos planos de corte pero no acaban en las "esquinas" del rectangulo parametrico son "muescas" y deben unirse manualmente. Esta funcion las detecta y las une a los puntos correspondientes
corregirMuescas(	lista_aristas*	a,
					lista_puntos*	n,
					lista_caras*	c)
{
	long int i,j,k;
	long int ini1,ini2,fin1,fin2;
	
	// barre el contorno del rectangulo parametrico obteniendo las coordenadas (x,y,z) previas a proyectar. Las muescas tendran el mismo (x,y,z) que otros 3 puntos pero no estaran en las esquinas 
	
	for (i = 0; i < a->n_total; i++){
		ini1=a->aristas[i].inicio_id;
		fin1=a->aristas[i].final_id;
		for( j=0; j < a->n_total; j++){
			ini2=a->aristas[j].inicio_id;
			fin2=a->aristas[j].final_id;
			//para una arista, comparar con todas las demas para ver si esta repetida (signo de que uno de sus dos vertices forma una muesca)
			if(compararCoordenadas(ini1,ini2,n) && (ini1!=ini2)){
				if(fin1==fin2){
					//si la arista esta repetida, nos "saltamos" la muesca eliminandola del contorno parametrico
					sustituirId(ini2,ini1,c,n);
					eliminarArista(ini1,fin1,a);
					eliminarArista(ini2,fin2,a);
					for(k=0;k<a->n_total;k++){
						if(a->aristas[k].inicio_id==ini2){
							a->aristas[k].inicio_id=ini1;
						} else if(a->aristas[k].final_id==ini2){
							a->aristas[k].final_id=ini1;
						}
					}
				}
			}
			if(compararCoordenadas(ini1,fin2,n) && (ini1!=fin2)){
				if(fin1==ini2){
					sustituirId(fin2,ini1,c,n);
					eliminarArista(ini1,fin1,a);
					eliminarArista(ini2,fin2,a);
					for(k=0;k<a->n_total;k++){
						if(a->aristas[k].inicio_id==fin2){
							a->aristas[k].inicio_id=ini1;
						} else if(a->aristas[k].final_id==fin2){
							a->aristas[k].final_id=ini1;
						}
					}
				}
			}
			if(compararCoordenadas(fin1,fin2,n) && (fin1!=fin2)){
				if(ini1==ini2){
					sustituirId(fin2,fin1,c,n);
					eliminarArista(ini1,fin1,a);
					eliminarArista(ini2,fin2,a);
					for(k=0;k<a->n_total;k++){
						if(a->aristas[k].inicio_id==fin2){
							a->aristas[k].inicio_id=fin1;
						} else if(a->aristas[k].final_id==fin2){
							a->aristas[k].final_id=fin1;
						}
					}
				}
			}
			if(compararCoordenadas(fin1,ini2,n) && (fin1!=ini2)){
				if(ini1==fin2){
					sustituirId(ini2,fin1,c,n);
					eliminarArista(ini1,fin1,a);
					eliminarArista(ini2,fin2,a);
					for(k=0;k<a->n_total;k++){
						if(a->aristas[k].inicio_id==ini2){
							a->aristas[k].inicio_id=fin1;
						} else if(a->aristas[k].final_id==ini2){
							a->aristas[k].final_id=fin1;
						}
					}
				}
			}
		}
	}	
	printf("\n\tCompleted");
}



int	//para una polilinea cerrada que representa el contorno de un patch, identifica otros vertices de la malla con las mismas coordenadas y que pueden fusionarse para dar continuidad a la superficie. Exporta la lista de equivalencias en variable sustituciones
coserContorno(	lista_aristas 	a,
				lista_caras*	c,
				lista_puntos*	n)
{
	long int i;
	long int buscado=NO_INICIALIZADO;
	long int buscado2=NO_INICIALIZADO;
	long int sustituciones=1;
	
	long int nodoIni=a.comienzo;
	
	for (i = 0; i < a.n_total; i++){
		a.aristas[i].utilizada=0;
	}
	
	
	buscado=nodoIni;
	buscado2=nodoIni;
	//reemplazarId(nodoIni,c,n);
	for (i = 0; i < a.n_total; i++){
		if(!a.aristas[i].utilizada){
			if (a.aristas[i].inicio_id==buscado){
				a.aristas[i].utilizada=1;
				reemplazarId(a.aristas[i].inicio_id,c,n);
				buscado=a.aristas[i].final_id;
				i=-1;
				sustituciones++;
			}
			if (a.aristas[i].final_id==buscado){
				a.aristas[i].utilizada=1;
				reemplazarId(a.aristas[i].final_id,c,n);
				buscado=a.aristas[i].inicio_id;
				i=-1;
				sustituciones++;
			}
			if (a.aristas[i].inicio_id==buscado2){
				a.aristas[i].utilizada=1;
				reemplazarId(a.aristas[i].inicio_id,c,n);
				buscado2=a.aristas[i].final_id;
				i=-1;
				sustituciones++;
			}
			if (a.aristas[i].final_id==buscado2){
				a.aristas[i].utilizada=1;
				reemplazarId(a.aristas[i].final_id,c,n);
				buscado2=a.aristas[i].inicio_id;
				i=-1;
				sustituciones++;
			}
		}
	}
	return sustituciones;
}


int	//propone un punto comun (mismo x,y,z) entre dos polilineas, devolviendo el orden que ocupa en la polilinea 1 (de 0 a n-1). Si no existe, devuelve -1
hallarNodoComun(	lista_aristas*	a1,
					lista_aristas*	a2,
					lista_puntos*	n)
{
	
	long int i,j;
	long int pt1a,pt1b,pt2a,pt2b;
	int aux[T_MAX]; for (i = 0; i < T_MAX; i++) aux[i]=0;
	
	
	for (i = 0; i < a1->n_total; i++){
		pt1a=a1->aristas[i].inicio_id;
		pt1b=a1->aristas[i].final_id;
		for (j = 0; j < a2->n_total; j++){
			pt2a=a2->aristas[j].inicio_id;
			pt2b=a2->aristas[j].final_id;
			if(compararCoordenadas(pt1a,pt2a,n)){
				aux[pt1a]=1;
				aux[pt2a]=1;
			}
			if(compararCoordenadas(pt1a,pt2b,n)){
				aux[pt1a]=1;
				aux[pt2b]=1;
			}
			if(compararCoordenadas(pt1b,pt2a,n)){
				aux[pt1b]=1;
				aux[pt2a]=1;
			}
			if(compararCoordenadas(pt1b,pt2b,n)){
				aux[pt1b]=1;
				aux[pt2b]=1;
			}
			
		}
	}
	
	for (i = 0; i < T_MAX; i++){
		if (aux[i]) {
			if(nodoPerteneceALista(i,a1)){
				return i;
			}
		}
	}
	
	return -1;
	
}
	
/*********************************************************/
/*************                     ***********************/
/*************    PARAMETRICS      ***********************/
/*************                     ***********************/
/*********************************************************/



void	//en base al numero de referencias a unas coordenadas, diferencia los puntos entre puntes de interior, borde o esquina de un patch. repeticiones=4 para borde, 8=midpoint de borde, 16=esquina
identificarEsquinas(	lista_aristas 	a,
						lista_puntos*	n,
						int*			repeticiones)
{
	long int i,j;
	
	
	for (i = 0; i < a.n_total; i++){
		repeticiones[a.aristas[i].inicio_id]++;
		repeticiones[a.aristas[i].final_id]++;
		for(j=0;j<i;j++){
			if(compararCoordenadas(a.aristas[i].inicio_id,a.aristas[j].inicio_id,n)){
				repeticiones[a.aristas[i].inicio_id]++;
				repeticiones[a.aristas[j].inicio_id]++;
			}
			if(compararCoordenadas(a.aristas[i].final_id,a.aristas[j].inicio_id,n)){
				repeticiones[a.aristas[i].final_id]++;
				repeticiones[a.aristas[j].inicio_id]++;
			}
			if(compararCoordenadas(a.aristas[i].inicio_id,a.aristas[j].final_id,n)){
				repeticiones[a.aristas[i].inicio_id]++;
				repeticiones[a.aristas[j].final_id]++;
			}
			if(compararCoordenadas(a.aristas[i].final_id,a.aristas[j].final_id,n)){
				repeticiones[a.aristas[i].final_id]++;
				repeticiones[a.aristas[j].final_id]++;
			}
		}
	}
	
}
						

void //dada una lista de puntos que se sabe que estan en el contorno de ALGUNO de los 4 patches, genera un patch conjunto que contiene a los 4 y es topologicamente asimilable a un rectangulo. Divide su contorno en 4 "lados" y los exporta
identificarLadoParametrico(	lista_aristas*	cp,
							int*			repeticiones,
							lista_aristas*	ch1,
							lista_aristas*	ch2,
							lista_aristas*	cv1,
							lista_aristas*	cv2)
{
	long int i;
	long int buscado=NO_INICIALIZADO;
	
		
	for (i = 0; i < cp->n_total; i++){
		cp->aristas[i].utilizada=0;
	}
	
	//las coordenadas que aparecen repetidas 16 veces pertenecen a los 4 patches a la vez. Hay 4 de esos puntos esquina
	//las polilineas que unen dos puntos esquina son la interseccion de dos patches
	
	
	//identificacion de puntos esquina
	
	for (i = 0; i < cp->n_total; i++){
		//anade aristas hasta encontrar otro punto esquina
		if(repeticiones[cp->aristas[i].inicio_id]==16){
			anadirArista(cp->aristas[i].inicio_id,cp->aristas[i].final_id,ch1);
			cp->aristas[i].utilizada=1;
			buscado=cp->aristas[i].final_id;
			break;
		}
		if(repeticiones[cp->aristas[i].final_id]==16){
			anadirArista(cp->aristas[i].final_id,cp->aristas[i].inicio_id,ch1);
			buscado=cp->aristas[i].inicio_id;
			cp->aristas[i].utilizada=1;
			break;
		}
	}
	
	//interseccion del contorno de dos patches, se asigna arbitrariamente a esta interseccion el "contorno horizontal inferior" del patch conjunto. 
	
	for (i = 0; i < cp->n_total; i++){
		if(!cp->aristas[i].utilizada){
			if (cp->aristas[i].inicio_id==buscado){
				cp->aristas[i].utilizada=1;
				anadirArista(cp->aristas[i].inicio_id,cp->aristas[i].final_id,ch1);
				buscado=cp->aristas[i].final_id;
				i=-1;
				if(repeticiones[buscado]==16)  break;
			} else if (cp->aristas[i].final_id==buscado){
				cp->aristas[i].utilizada=1;
				anadirArista(cp->aristas[i].final_id,cp->aristas[i].inicio_id,ch1);
				buscado=cp->aristas[i].inicio_id;
				i=-1;
				if(repeticiones[buscado]==16)  break;
			}
		}
	}
	
	
	for (i = 0; i < cp->n_total; i++){
		if(!cp->aristas[i].utilizada){
			if (cp->aristas[i].inicio_id==buscado){
				cp->aristas[i].utilizada=1;
				anadirArista(cp->aristas[i].inicio_id,cp->aristas[i].final_id,cv1);
				buscado=cp->aristas[i].final_id;
				i=-1;
				if(repeticiones[buscado]==16)  break;
			} else if (cp->aristas[i].final_id==buscado){
				cp->aristas[i].utilizada=1;
				anadirArista(cp->aristas[i].final_id,cp->aristas[i].inicio_id,cv1);
				buscado=cp->aristas[i].inicio_id;
				i=-1;
				if(repeticiones[buscado]==16)  break;
			}
		}
	}
	
	
	for (i = 0; i < cp->n_total; i++){
		if(!cp->aristas[i].utilizada){
			if (cp->aristas[i].inicio_id==buscado){
				cp->aristas[i].utilizada=1;
				anadirArista(cp->aristas[i].inicio_id,cp->aristas[i].final_id,ch2);
				buscado=cp->aristas[i].final_id;
				i=-1;
				if(repeticiones[buscado]==16)  break;
			} else if (cp->aristas[i].final_id==buscado){
				cp->aristas[i].utilizada=1;
				anadirArista(cp->aristas[i].final_id,cp->aristas[i].inicio_id,ch2);
				buscado=cp->aristas[i].inicio_id;
				i=-1;
				if(repeticiones[buscado]==16)  break;
			}
		}
	}
	
	
	for (i = 0; i < cp->n_total; i++){
		if(!cp->aristas[i].utilizada){
			if (cp->aristas[i].inicio_id==buscado){
				cp->aristas[i].utilizada=1;
				anadirArista(cp->aristas[i].inicio_id,cp->aristas[i].final_id,cv2);
				buscado=cp->aristas[i].final_id;
				i=-1;
				if(repeticiones[buscado]==16)  break;
			} else if (cp->aristas[i].final_id==buscado){
				cp->aristas[i].utilizada=1;
				anadirArista(cp->aristas[i].final_id,cp->aristas[i].inicio_id,cv2);
				buscado=cp->aristas[i].inicio_id;
				i=-1;
				if(repeticiones[buscado]==16)  break;
			}
		}
	}
	
	
}

lista_aristas	//una vez cortada una malla en 4 patches, identifica en que patch esta cada vertice y distingue vertices de borde de vertices de interior
identificarBordes(	lista_caras*	c,
					lista_puntos*	n,
					long int*		alt,
					double			posicionCorteY,
					double			posicionCorteZ	)
{
	long int i,j;
	lista_aristas contorno; contorno.n_total=0;
	lista_aristas contorno2; contorno2.n_total=0;
	lista_aristas contorno3; contorno3.n_total=0;
	lista_aristas contorno4; contorno4.n_total=0;
	long int nodoComun=NO_INICIALIZADO;
	long int carasAbarcadas[]={0,0,0,0};

//INIT
	
	for (i = 0; i < c->n_total; i++){
		c->activeFlag[i]=0;
	}
	
	//PATCH 1 (cuadrante (-,-)
	
	for (i = 0; i <  c->n_total; i++){
		//volcamos todas las aristas contenidas en un cuadrante en variable contorno
		if(!c->activeFlag[i]){
			if(identificarCuadranteCara(c->coord[i],n,posicionCorteY,posicionCorteZ)==CUADRANTE1){
				anadirArista(c->coord[i].id[0],c->coord[i].id[2],&contorno);
				anadirArista(c->coord[i].id[2],c->coord[i].id[1],&contorno);
				anadirArista(c->coord[i].id[1],c->coord[i].id[0],&contorno);
				c->activeFlag[i]=1;
				break;
			}
		}
	}
	
	
	j=0;
	
	do{	//eliminamos todas las aristas que aparezcan en 2 caras: las que quedan son parte del contorno del patch y sus vertices tambien
		for(i=0;i<contorno.n_total;i++){
			if(!c->activeFlag[j]){
				if (nodoPerteneceACara(	contorno.aristas[i].inicio_id,c->coord[j]) && nodoPerteneceACara( contorno.aristas[i].final_id,c->coord[j])){
					c->activeFlag[j]=1;
					carasAbarcadas[0]++;
					if (!eliminarArista(c->coord[j].id[0],c->coord[j].id[1],&contorno)){ //si existe, se elimina, si no, se anade
						anadirArista(c->coord[j].id[0],c->coord[j].id[1],&contorno);
					}
					if (!eliminarArista(c->coord[j].id[2],c->coord[j].id[1],&contorno)){ //si existe, se elimina, si no, se anade
						anadirArista(c->coord[j].id[2],c->coord[j].id[1],&contorno);
					}
					if (!eliminarArista(c->coord[j].id[0],c->coord[j].id[2],&contorno)){ //si existe, se elimina, si no, se anade
						anadirArista(c->coord[j].id[0],c->coord[j].id[2],&contorno);
					}
					j=-1;
					break;
				}
			}
		}
		j++;
	}while(j<c->n_total);
	
	printf("\n\tPATCH 1/4 completed");
	
	
	//PATCH 2
	
	
	for (i = 0; i <  c->n_total; i++){
		if(!c->activeFlag[i]){
			if(identificarCuadranteCara(c->coord[i],n,posicionCorteY,posicionCorteZ)==CUADRANTE2){
				anadirArista(c->coord[i].id[0],c->coord[i].id[2],&contorno2);
				anadirArista(c->coord[i].id[2],c->coord[i].id[1],&contorno2);
				anadirArista(c->coord[i].id[1],c->coord[i].id[0],&contorno2);
				c->activeFlag[i]=1;
				break;
			}
		}
	}
	
	
	j=0;
	
	do{
		for(i=0;i<contorno2.n_total;i++){
			if(!c->activeFlag[j]){
				if (nodoPerteneceACara(	contorno2.aristas[i].inicio_id,c->coord[j]) && nodoPerteneceACara( contorno2.aristas[i].final_id,c->coord[j])){
					c->activeFlag[j]=1;
					carasAbarcadas[1]++;
					if (!eliminarArista(c->coord[j].id[0],c->coord[j].id[1],&contorno2)){ //si existe, se elimina, si no, se anade
						anadirArista(c->coord[j].id[0],c->coord[j].id[1],&contorno2);
					}
					if (!eliminarArista(c->coord[j].id[2],c->coord[j].id[1],&contorno2)){ //si existe, se elimina, si no, se anade
						anadirArista(c->coord[j].id[2],c->coord[j].id[1],&contorno2);
					}
					if (!eliminarArista(c->coord[j].id[0],c->coord[j].id[2],&contorno2)){ //si existe, se elimina, si no, se anade
						anadirArista(c->coord[j].id[0],c->coord[j].id[2],&contorno2);
					}
					j=-1;
					break;
				}
			}
		}
		j++;
	}while(j<c->n_total);
	
	printf("\n\tPATCH 2/4 completed");
	
	//PATCH 3
	
	for (i = 0; i <  c->n_total; i++){
		if(!c->activeFlag[i]){
			if(identificarCuadranteCara(c->coord[i],n,posicionCorteY,posicionCorteZ)==CUADRANTE3){
				anadirArista(c->coord[i].id[0],c->coord[i].id[2],&contorno3);
				anadirArista(c->coord[i].id[2],c->coord[i].id[1],&contorno3);
				anadirArista(c->coord[i].id[1],c->coord[i].id[0],&contorno3);
				c->activeFlag[i]=1;
				break;
			}
		}
	}
	
	
	j=0;
	
	do{
		for(i=0;i<contorno3.n_total;i++){
			if(!c->activeFlag[j]){
				if (nodoPerteneceACara(	contorno3.aristas[i].inicio_id,c->coord[j]) && nodoPerteneceACara( contorno3.aristas[i].final_id,c->coord[j])){
					c->activeFlag[j]=1;
					carasAbarcadas[2]++;
					if (!eliminarArista(c->coord[j].id[0],c->coord[j].id[1],&contorno3)){ //si existe, se elimina, si no, se anade
						anadirArista(c->coord[j].id[0],c->coord[j].id[1],&contorno3);
					}
					if (!eliminarArista(c->coord[j].id[2],c->coord[j].id[1],&contorno3)){ //si existe, se elimina, si no, se anade
						anadirArista(c->coord[j].id[2],c->coord[j].id[1],&contorno3);
					}
					if (!eliminarArista(c->coord[j].id[0],c->coord[j].id[2],&contorno3)){ //si existe, se elimina, si no, se anade
						anadirArista(c->coord[j].id[0],c->coord[j].id[2],&contorno3);
					}
					j=-1;
					break;
				}
			}
		}
		j++;
	}while(j<c->n_total);
	
	printf("\n\tPATCH 3/4 completed");
	
	
	//PATCH 4
	
		
	for (i = 0; i <  c->n_total; i++){
		if(!c->activeFlag[i]){
			if(identificarCuadranteCara(c->coord[i],n,posicionCorteY,posicionCorteZ)==CUADRANTE4){
				anadirArista(c->coord[i].id[0],c->coord[i].id[2],&contorno4);
				anadirArista(c->coord[i].id[2],c->coord[i].id[1],&contorno4);
				anadirArista(c->coord[i].id[1],c->coord[i].id[0],&contorno4);
				c->activeFlag[i]=1;
				break;
			}
		}
	}
	
	j=0;
	
	do{
		for(i=0;i<contorno4.n_total;i++){
			if(!c->activeFlag[j]){
				if (nodoPerteneceACara(	contorno4.aristas[i].inicio_id,c->coord[j]) && nodoPerteneceACara( contorno4.aristas[i].final_id,c->coord[j])){
					c->activeFlag[j]=1;
					carasAbarcadas[3]++;
					if (!eliminarArista(c->coord[j].id[0],c->coord[j].id[1],&contorno4)){ //si existe, se elimina, si no, se anade
						anadirArista(c->coord[j].id[0],c->coord[j].id[1],&contorno4);
					}
					if (!eliminarArista(c->coord[j].id[2],c->coord[j].id[1],&contorno4)){ //si existe, se elimina, si no, se anade
						anadirArista(c->coord[j].id[2],c->coord[j].id[1],&contorno4);
					}
					if (!eliminarArista(c->coord[j].id[0],c->coord[j].id[2],&contorno4)){ //si existe, se elimina, si no, se anade
						anadirArista(c->coord[j].id[0],c->coord[j].id[2],&contorno4);
					}
					j=-1;
					break;
				}
			}
		}
		j++;
	}while(j<c->n_total);
	
	printf("\n\tPATCH 4/4 completed");
	
	//FILE *fcont1 = fopen("contorno1.csv", "w");
	//FILE *fcont2 = fopen("contorno2.csv", "w");
	//FILE *fcont3 = fopen("contorno3.csv", "w");
	//FILE *fcont4 = fopen("contorno4.csv", "w");
	
	//if (fcont1 == NULL)
	//{
		//printf("Error opening file!\n");
		//exit(1);
	//}
	//if (fcont2 == NULL)
	//{
		//printf("Error opening file!\n");
		//exit(1);
	//}
	//if (fcont3 == NULL)
	//{
		//printf("Error opening file!\n");
		//exit(1);
	//}
	//if (fcont4 == NULL)
	//{
		//printf("Error opening file!\n");
		//exit(1);
	//}
	
	//fprintf(fcont1,"x,y,z");
	//for (i = 0; i < contorno.n_total; i++){
		//punto p_ini=n->coord[contorno.aristas[i].inicio_id];
		//punto p_fin=n->coord[contorno.aristas[i].final_id];
		//fprintf(fcont1,"\n%lf,%lf,%lf",p_ini.x,p_ini.y,p_ini.z);
		//fprintf(fcont1,"\n%lf,%lf,%lf",p_fin.x,p_fin.y,p_fin.z);
	//}
	//fprintf(fcont2,"x,y,z");
	//for (i = 0; i < contorno2.n_total; i++){
		//punto p_ini=n->coord[contorno2.aristas[i].inicio_id];
		//punto p_fin=n->coord[contorno2.aristas[i].final_id];
		//fprintf(fcont2,"\n%lf,%lf,%lf",p_ini.x,p_ini.y,p_ini.z);
		//fprintf(fcont2,"\n%lf,%lf,%lf",p_fin.x,p_fin.y,p_fin.z);
	//}
	//fprintf(fcont3,"x,y,z");
	//for (i = 0; i < contorno3.n_total; i++){
		//punto p_ini=n->coord[contorno3.aristas[i].inicio_id];
		//punto p_fin=n->coord[contorno3.aristas[i].final_id];
		//fprintf(fcont3,"\n%lf,%lf,%lf",p_ini.x,p_ini.y,p_ini.z);
		//fprintf(fcont3,"\n%lf,%lf,%lf",p_fin.x,p_fin.y,p_fin.z);
	//}
	//fprintf(fcont4,"x,y,z");
	//for (i = 0; i < contorno4.n_total; i++){
		//punto p_ini=n->coord[contorno4.aristas[i].inicio_id];
		//punto p_fin=n->coord[contorno4.aristas[i].final_id];
		//fprintf(fcont4,"\n%lf,%lf,%lf",p_ini.x,p_ini.y,p_ini.z);
		//fprintf(fcont4,"\n%lf,%lf,%lf",p_fin.x,p_fin.y,p_fin.z);
	//}
	
	//printf("\nTERMINADA EXPORTACION DE CONTORNOS, c PARA CONTINUAR");
	//fclose(fcont1);fclose(fcont2);fclose(fcont3);fclose(fcont4);
	
	
	lista_aristas u12=interseccionListas(&contorno,&contorno2,alt,n);
	lista_aristas u13=interseccionListas(&contorno,&contorno3,alt,n);
	lista_aristas u14=interseccionListas(&contorno,&contorno4,alt,n);
	lista_aristas u23=interseccionListas(&contorno3,&contorno2,alt,n);
	lista_aristas u24=interseccionListas(&contorno4,&contorno2,alt,n);
	lista_aristas u34=interseccionListas(&contorno3,&contorno4,alt,n);
	lista_aristas* patch_ul; //cuadrante up-left del rectangulo parametrico
	lista_aristas* patch_ur;
	lista_aristas* patch_dl;
	lista_aristas* patch_dr;
	lista_aristas* uu;
	lista_aristas* ul;
	lista_aristas* ur;
	lista_aristas* ud;

	
	if(u12.n_total==0){
		patch_ul=&contorno;
		patch_ur=&contorno3;
		patch_dl=&contorno4;
		patch_dr=&contorno2;
		uu=&u13;
		ud=&u24;
		ul=&u14;
		ur=&u34;
	}
	if(u13.n_total==0){
		patch_ul=&contorno;
		patch_ur=&contorno2;
		patch_dl=&contorno4;
		patch_dr=&contorno3;
		uu=&u12;
		ud=&u34;
		ul=&u14;
		ur=&u23;
	}
	if(u14.n_total==0){
		patch_ul=&contorno;
		patch_ur=&contorno2;
		patch_dl=&contorno3;
		patch_dr=&contorno4;
		uu=&u12;
		ud=&u34;
		ul=&u13;
		ur=&u24;
	}
	
	nodoComun=hallarNodoComun(patch_ur,patch_dl,n);
	//for (i = 0; i < patch_ul->n_total; i++){
		//if(compararCoordenadas(nodoComun,patch_ul->aristas[i].inicio_id,n)){
			//nodoComun_ul=patch_ul->aristas[i].inicio_id;
			//break;
		//}
		//if(compararCoordenadas(nodoComun,patch_ul->aristas[i].final_id,n)){
			//nodoComun_ul=patch_ul->aristas[i].final_id;
			//break;
		//}
	//}
	//for (i = 0; i < patch_dl->n_total; i++){
		//if(compararCoordenadas(nodoComun,patch_dl->aristas[i].inicio_id,n)){
			//nodoComun_dl=patch_dl->aristas[i].inicio_id;
			//break;
		//}
		//if(compararCoordenadas(nodoComun,patch_dl->aristas[i].final_id,n)){
			//nodoComun_dl=patch_dl->aristas[i].final_id;
			//break;
		//}
	//}
	
	//for (i = 0; i < patch_dr->n_total; i++){
		//if(compararCoordenadas(nodoComun,patch_dr->aristas[i].inicio_id,n)){
			//nodoComun_dr=patch_dr->aristas[i].inicio_id;
			//patch_dr->comienzo=nodoComun_dr;
			//break;
		//}
		//if(compararCoordenadas(nodoComun,patch_dr->aristas[i].final_id,n)){
			//nodoComun_dr=patch_dr->aristas[i].final_id;
			//patch_dr->comienzo=nodoComun_dr;
			//break;
		//}
	//}
	
	for (i = 0; i < uu->n_total; i++){
		if(compararCoordenadas(uu->aristas[i].inicio_id,nodoComun,n)){
			uu->comienzo=uu->aristas[i].inicio_id;
			break;
		}
		if(compararCoordenadas(uu->aristas[i].final_id,nodoComun,n)){
			uu->comienzo=uu->aristas[i].final_id;
			break;
		}
	}
	
	for (i = 0; i < ud->n_total; i++){
		if(compararCoordenadas(ud->aristas[i].inicio_id,nodoComun,n)){
			ud->comienzo=ud->aristas[i].inicio_id;
			break;
		}
		if(compararCoordenadas(ud->aristas[i].final_id,nodoComun,n)){
			ud->comienzo=ud->aristas[i].final_id;
			break;
		}
	}
	
	for (i = 0; i < ur->n_total; i++){
		if(compararCoordenadas(ur->aristas[i].inicio_id,nodoComun,n)){
			ur->comienzo=ur->aristas[i].inicio_id;
			break;
		}
		if(compararCoordenadas(ur->aristas[i].final_id,nodoComun,n)){
			ur->comienzo=ur->aristas[i].final_id;
			break;
		}
	}
	
	for (i = 0; i < ul->n_total; i++){
		if(compararCoordenadas(ul->aristas[i].inicio_id,nodoComun,n)){
			ul->comienzo=ul->aristas[i].inicio_id;
			break;
		}
		if(compararCoordenadas(ul->aristas[i].final_id,nodoComun,n)){
			ul->comienzo=ul->aristas[i].final_id;
			break;
		}
	}
	
	coserContorno(*uu,c,n);
	coserContorno(*ud,c,n);
	coserContorno(*ur,c,n);
	coserContorno(*ul,c,n);

	printf("\n\tCommon edges identification completed");
	
	if(patch_ul->n_total==0){
		if(patch_ur->n_total==0){
			if(patch_dl->n_total==0){
				return *patch_dr;
			}else{
				return *patch_dl;
			}
		}else{
			return *patch_ur;
		}
	}else{
		return *patch_ul;
	}
	
}




/*********************************************************/
/*************                     ***********************/
/*************    MATRIX SYSTEM    ***********************/
/*************                     ***********************/
/*********************************************************/


void	// suma val al termino en la posicion i,j de una matriz (formato sparse)
sumarElementoSparse(sparseMatrix	*m,
					long int		i,
					long int		j,
					double			val)
{
	long int pos, lim;
	lim= m->n_total;
	
	for (pos=0; pos<lim; pos++) {
		if ((m->i[pos] == i) && (m->j[pos] == j)) {
			m->val[pos]+=val;
			return;
		}
	}
	
	pos=lim;
	m->val[pos]=val;
	m->i[pos]=i;
	m->j[pos]=j;
	m->n_total++;
	return;
	
	printf("\nERROR IN MATRIX GENERATION. Values i=%ld j=%ld val=%lf",i,j,val);
	
}


void	//inserta los nodos de coordenadas conocidas en la matriz de pesos. 0 en toda la fila y 1 en la diagonal
filaTrivial(	sparseMatrix	*m,
				long int		i)
{
	long int pos;
	
	for (pos=0; pos < m->n_total; pos++) {
		if (m->i[pos] == i) {
			m->val[pos]= (m->j[pos] == i) ? 1 : 0;
			return;
		}
	}
	printf("\nERROR IN MATRIX EDITION function filaTrivial. Values i=%ld",i);
}




void	//asigna a cada arista un peso en funcion de su longitud y los angulos de las caras que la comparten (ver chord length method)
calcularPeso (	lista_caras 	*caras,
				lista_puntos 	*nodos,
				sparseMatrix	*pesos)
{
	
	long int i,j,k,l;
	cara face;
	long int aristas_procesadas=0;
	long int id[3]; //ID de los 3 puntos que forman la cara
	double x[3];	//coords de los 3 puntos
	double y[3];
	double z[3];
	double vectores_x[3][3]; //vectores unitarios de cada una de las aristas, vector[0][1] = coordenadas del vector que va del punto 0 al 1
	double vectores_y[3][3];
	double vectores_z[3][3];
	double modulo[3][3];
	double angulo[3]; //angulo sobre el vertice con id 0,1 o 2
	double ctg; //contangente de angulo sobre el vertice




	for (k=0;k<caras->n_total;k++){

		face=caras->coord[k];

		//LECTURA DE COORDENADAS
	
		for (i=0;i<3;i++){
			id[i]=face.id[i];
			x[i]=nodos->coord[id[i]].x;
			y[i]=nodos->coord[id[i]].y;
			z[i]=nodos->coord[id[i]].z;
		}
		
		//CALCULO DE VECTORES UNITARIOS DE ARISTAS
		
		for (i=0;i<3;i++) {
			for (j=0;j<3;j++) {
				if (i!=j) {
					vectores_x[i][j]=x[i]-x[j];
					vectores_y[i][j]=y[i]-y[j];
					vectores_z[i][j]=z[i]-z[j];
					modulo[i][j]=sqrt(vectores_x[i][j]*vectores_x[i][j]+vectores_y[i][j]*vectores_y[i][j]+vectores_z[i][j]*vectores_z[i][j]);
					vectores_x[i][j]/=modulo[i][j];
					vectores_y[i][j]/=modulo[i][j];
					vectores_z[i][j]/=modulo[i][j];
				}
			}
		}
		
		//ANGULO ENTRE VECTORES
		
		angulo[0]=acos (vectores_x[0][1]*vectores_x[0][2]+vectores_y[0][1]*vectores_y[0][2]+vectores_z[0][1]*vectores_z[0][2]);
		angulo[1]=acos (vectores_x[1][2]*vectores_x[1][0]+vectores_y[1][2]*vectores_y[1][0]+vectores_z[1][2]*vectores_z[1][0]);
		angulo[2]=acos (vectores_x[2][1]*vectores_x[2][0]+vectores_y[2][1]*vectores_y[2][0]+vectores_z[2][1]*vectores_z[2][0]);
		
		//CALCULO DEL PESO
		
		ctg=1/tan(angulo[0]);
		i=id[1];
		j=id[2];
		
		for(l=0;l<3;l++){
				
			ctg=1/tan(angulo[l]);
			i=id[ (l+1) % 3];
			j=id[ (l+2) % 3];
			
			sumarElementoSparse(pesos,i,j,ctg);
			sumarElementoSparse(pesos,j,i,ctg);
			
			sumarElementoSparse(pesos,i,i,-ctg);
			sumarElementoSparse(pesos,j,j,-ctg);
			
			aristas_procesadas++;
			
		}
	}
	printf("\n\tCompleted, %ld edges processed",aristas_procesadas);
}


void	//para una figura parametrizada, sustituye aquellas filas de su matriz de pesos para los nodos con coordenadas conocidas (por estar en el borde del rectangulo parametrico) y genera el vector de terminos independientes del sistema
ecuacionesTriviales(	sparseMatrix*	pesos,
						long int		n_nodos,
						int*			repeticiones,
						lista_aristas*	contornoParametrico,
						lista_puntos*	nodos, 
						lista_caras*	caras,
						double*			termIndependientes_real,
						double*			termIndependientes_img)
{
	
	long int i, j;
	long int pos;
	double longTotal, longParcial;
	int existeNodo[T_MAX]; for (i=0;i<T_MAX;i++) existeNodo[i]=0;
	double *longitudAristas; longitudAristas=(double *)malloc(sizeof(double)*n_nodos); for (i=0; i<n_nodos; i++) longitudAristas[i]=0;
	long int id1, id2;
	double x1,x2,y1,y2,z1,z2;
	lista_aristas corteHInf,corteHSup,corteVDer,corteVIzq;
		corteHInf.n_total=0;
	corteHSup.n_total=0;
	corteVDer.n_total=0;
	corteVIzq.n_total=0;
	int filasTriviales[T_MAX]; for(i=0;i<T_MAX;i++) filasTriviales[i]=0;
	
//dividimos los vertices que estan sobre el rectangulo parametrico segun cual de sus 4 lados esten tocando.	
	
	identificarLadoParametrico(contornoParametrico,repeticiones,&corteHInf,&corteHSup,&corteVDer,&corteVIzq);
	
	
//INIT
	
	for (i=0; i<nodos->n_total; i++) {
		termIndependientes_real[i]=0;
		termIndependientes_img[i]=0;
	}
	longTotal=0;
	longParcial=0;
	i=0;
	j=0;

	
	
//Chord length param, horizontal inferior

	
	longTotal=0;
	for (i=0; i<corteHInf.n_total; i++) {
		id1=corteHInf.aristas[i].inicio_id;
		id2=corteHInf.aristas[i].final_id;
		filasTriviales[id1]=1;
		filasTriviales[id2]=1;
		x1=nodos->coord[id1].x;
		y1=nodos->coord[id1].y;
		z1=nodos->coord[id1].z;
		x2=nodos->coord[id2].x;
		y2=nodos->coord[id2].y;
		z2=nodos->coord[id2].z;
		longitudAristas[i]= sqrt ((x1-x2)*(x1-x2)+(y1-y2)*(y1-y2)+(z1-z2)*(z1-z2));
		longTotal+=longitudAristas[i];
	}
	
	longParcial=0;
	termIndependientes_real[corteHInf.aristas[0].inicio_id]=0;
	termIndependientes_img[corteHInf.aristas[0].inicio_id]=0;
	for (i=0; i<corteHInf.n_total; i++) {
		id1=corteHInf.aristas[i].final_id;
		longParcial += longitudAristas[i];
		termIndependientes_real[id1]= longParcial / longTotal; 
		termIndependientes_img[id1]= 0;

	}
	
	printf("\n\tFinished Bottom Edge: %ld nodes processed", corteHInf.n_total);
	

//Chord length param, vertical derecha	
	longTotal=0;
	for (i=0; i<corteVDer.n_total; i++) {
		id1=corteVDer.aristas[i].inicio_id;
		id2=corteVDer.aristas[i].final_id;
		filasTriviales[id1]=1;
		filasTriviales[id2]=1;
		x1=nodos->coord[id1].x;
		y1=nodos->coord[id1].y;
		z1=nodos->coord[id1].z;
		x2=nodos->coord[id2].x;
		y2=nodos->coord[id2].y;
		z2=nodos->coord[id2].z;
		longitudAristas[i]= sqrt ((x1-x2)*(x1-x2)+(y1-y2)*(y1-y2)+(z1-z2)*(z1-z2));
		longTotal+=longitudAristas[i];
	}
	
	longParcial=0;
	termIndependientes_real[corteVDer.aristas[0].inicio_id]=1;
	termIndependientes_img[corteVDer.aristas[0].inicio_id]=0;
	for (i=0; i<corteVDer.n_total; i++) {
		id1=corteVDer.aristas[i].final_id;
		longParcial += longitudAristas[i];
		termIndependientes_real[id1]= 1 ; 
		termIndependientes_img[id1]= longParcial / longTotal;

	}
	
	printf("\n\tFinished Right Edge: %ld nodes processed", corteVDer.n_total);
	
//Chord length param, horizontal superior	
	longTotal=0;
	for (i=0; i<corteHSup.n_total; i++) {
		id1=corteHSup.aristas[i].inicio_id;
		id2=corteHSup.aristas[i].final_id;
		filasTriviales[id1]=1;
		filasTriviales[id2]=1;
		x1=nodos->coord[id1].x;
		y1=nodos->coord[id1].y;
		z1=nodos->coord[id1].z;
		x2=nodos->coord[id2].x;
		y2=nodos->coord[id2].y;
		z2=nodos->coord[id2].z;
		longitudAristas[i]= sqrt ((x1-x2)*(x1-x2)+(y1-y2)*(y1-y2)+(z1-z2)*(z1-z2));
		longTotal+=longitudAristas[i];
	}
	
	longParcial=0;
	termIndependientes_real[corteHSup.aristas[0].inicio_id]=1;
	termIndependientes_img[corteHSup.aristas[0].inicio_id]=1;
	for (i=0; i<corteHSup.n_total; i++) {
		id1=corteHSup.aristas[i].final_id;
		longParcial += longitudAristas[i];
		termIndependientes_real[id1]= 1-longParcial / longTotal; 
		termIndependientes_img[id1]= 1;

	}
	
	printf("\n\tFinished Top Edge: %ld nodes processed", corteHSup.n_total);
		
//Chord length param, vertical izquierda
	longTotal=0;
	for (i=0; i<corteVIzq.n_total; i++) {
		id1=corteVIzq.aristas[i].inicio_id;
		id2=corteVIzq.aristas[i].final_id;
		filasTriviales[id1]=1;
		filasTriviales[id2]=1;
		x1=nodos->coord[id1].x;
		y1=nodos->coord[id1].y;
		z1=nodos->coord[id1].z;
		x2=nodos->coord[id2].x;
		y2=nodos->coord[id2].y;
		z2=nodos->coord[id2].z;
		longitudAristas[i]= sqrt ((x1-x2)*(x1-x2)+(y1-y2)*(y1-y2)+(z1-z2)*(z1-z2));
		longTotal+=longitudAristas[i];
	}
	
	longParcial=0;
	termIndependientes_real[corteVIzq.aristas[0].inicio_id]=0;
	termIndependientes_img[corteVIzq.aristas[0].inicio_id]=1;
	for (i=0; i<corteVIzq.n_total; i++) {
		id1=corteVIzq.aristas[i].final_id;
		longParcial += longitudAristas[i];
		termIndependientes_real[id1]= 0 ; //para puntos de la curva de corte horizontal, w € (0,1) , u = 0
		termIndependientes_img[id1]= 1-longParcial / longTotal;

	}
	
	printf("\n\tFinished Left Edge: %ld nodes processed", corteVIzq.n_total);
	

//LIMPIEZA
	
	//eliminamos nodos renumerados durante el procesado de la malla
	
	for (i = 0; i < caras->n_total; i++){
		existeNodo[caras->coord[i].id[0]]=1;
		existeNodo[caras->coord[i].id[1]]=1;
		existeNodo[caras->coord[i].id[2]]=1;
	}
	
	int eliminados=0;
	for (i = 0; i < nodos->n_total; i++){
		if(!existeNodo[i]){
			eliminados++;
			filasTriviales[i]=1;
			sumarElementoSparse(pesos,i,i,1);
			termIndependientes_real[i]=0;
			termIndependientes_img[i]=0;
		}
	}
	
	printf("\n\tDeleted %d nodes unreferenced", eliminados);
	
	//insertamos nodos de coordenadas conocidas en matriz de pesos

	for (pos=0; pos < pesos->n_total; pos++) {
		i=pesos->i[pos];
		j=pesos->j[pos];
		if (filasTriviales[i])
			pesos->val[pos]= (i==j) ? 1 : 0;	
	} 

	printf("\n\tAssembly of the system of equations");
}


void	//resuelve un sistema matricial A·x=b. (b y x son coordenadas parametricas (u,w) representadas por numeros complejos ej: b+bz·i)
solver(	sparseMatrix*	pesos,
		double* 		b, 
		double*			bz,
		long int 		n_filas,
		double*			x,
		double*			xz)
{
	double *nullDouble = (double *) NULL ;
	SuiteSparse_long *nullInt= (SuiteSparse_long *) NULL;
	void *Symbolic, *Numeric ;
	double Info[UMFPACK_INFO]; 
	long int nz=pesos->n_total;
	long int pos;
	int status;
	SuiteSparse_long *Arow;
	SuiteSparse_long *Xpoi;
	SuiteSparse_long *Acol;
	SuiteSparse_long *Xcol;
	double * Aval;
	double * Xval;
	double * Avalz;
	double * Xvalz;
	
	Arow=(SuiteSparse_long*)malloc(sizeof(SuiteSparse_long)*nz);
	Acol=(SuiteSparse_long*)malloc(sizeof(SuiteSparse_long)*nz);
	Aval=(double*)malloc(sizeof(double)*nz);
	Avalz=(double*)malloc(sizeof(double)*nz);
	Xpoi=(SuiteSparse_long*)malloc(sizeof(SuiteSparse_long)*(n_filas+1));
	Xcol=(SuiteSparse_long*)malloc(sizeof(SuiteSparse_long)*nz);
	Xval=(double*)malloc(sizeof(double)*nz);
	Xvalz=(double*)malloc(sizeof(double)*nz);
	
	for (pos=0; pos<nz; pos++) {
		Arow[pos]=pesos->i[pos];
		Acol[pos]=pesos->j[pos];
		Aval[pos]=pesos->val[pos];
		Avalz[pos]=0;
	}
	
	// m, n = tamaño da matriz
	status=umfpack_zl_triplet_to_col (n_filas, n_filas, nz, Arow, Acol, Aval, Avalz, Xpoi, Xcol, Xval, Xvalz, nullInt) ;
	(void) umfpack_zl_symbolic (n_filas, n_filas, Xpoi, Xcol, Xval,Xvalz, &Symbolic, nullDouble, nullDouble) ;
	(void) umfpack_zl_numeric (Xpoi, Xcol, Xval, Xvalz, Symbolic, &Numeric, nullDouble, nullDouble) ;
	umfpack_zl_free_symbolic (&Symbolic) ;
	(void) umfpack_zl_solve (UMFPACK_A, Xpoi, Xcol, Xval, Xvalz, x,xz, b,bz, Numeric, nullDouble, Info) ;
    umfpack_zl_free_numeric (&Numeric) ;
    //printf("\nSystem Solve Status=%s",Info[0]==0?"Ok":"Error");
    //for (i = 0 ; i < n_filas ; i++) {
		////if (!isnan(x[i]) && !isnan(xz[i])) {
			//printf ("[%ld] = %lf \t%lf\n", i, x [i],xz[i]) ;
		////}
	//}

	//free(Arow);
	//free(Acol);
	//free(Aval);
	//free(Avalz);
	//free(Xpoi);
	//free(Xcol);
	//free(Xval);
	//free(Xvalz);
    
    return;
}

punto	//invierte la transformacion parametrica. Partiendo de un punto2D p sobre el rectangulo parametrico {[0,1],[0,1]} obtiene sus coordenadas (x,y,z) originales
coordParametricasAFisicas(	lista_caras*		caras,
							lista_puntos*		nodos,
							double*				vectorU,
							double*				vectorW,
							punto				p)
{
	long int i;
	long int caraContenedora=NO_INICIALIZADO;
	punto v1,v2,v3; //coordenadas parametricas
	punto a1,a2,a3; //coordenadas cartesianas
	double aux1,aux2;
	double lambda1,lambda2,lambda3; //coordenadas baricentricas
	punto result;
	
	for (i = 0; i < caras->n_total; i++){
		v1.x=vectorU[caras->coord[i].id[0]];
		v1.y=vectorW[caras->coord[i].id[0]];
		v2.x=vectorU[caras->coord[i].id[1]];
		v2.y=vectorW[caras->coord[i].id[1]];
		v3.x=vectorU[caras->coord[i].id[2]];
		v3.y=vectorW[caras->coord[i].id[2]];
		if(puntoPerteneceATriangulo(p,v1,v2,v3)){
			caraContenedora=i;
			aux1=(v2.y-v3.y)*(p.x-v3.x)+(v3.x-v2.x)*(p.y-v3.y);
			aux2=(v2.y-v3.y)*(v1.x-v3.x)+(v3.x-v2.x)*(v1.y-v3.y);
			lambda1=aux1/aux2;
			aux1=(v3.y-v1.y)*(p.x-v3.x)+(v1.x-v3.x)*(p.y-v3.y);
			aux2=(v2.y-v3.y)*(v1.x-v3.x)+(v3.x-v2.x)*(v1.y-v3.y);
			lambda2=aux1/aux2;
			lambda3=1-lambda1-lambda2;
			break;
		}
	}
	
	a1=nodos->coord[caras->coord[caraContenedora].id[0]];
	a2=nodos->coord[caras->coord[caraContenedora].id[1]];
	a3=nodos->coord[caras->coord[caraContenedora].id[2]];

	result.x=lambda1*a1.x+lambda2*a2.x+lambda3*a3.x;
	result.y=lambda1*a1.y+lambda2*a2.y+lambda3*a3.y;
	result.z=lambda1*a1.z+lambda2*a2.z+lambda3*a3.z;
	
	if(caraContenedora==NO_INICIALIZADO) 
		printf("\nError: the parameter point u=%lf, w=%lf does not belong to any face",p.x,p.y);
	return result;
	
}





/*********************************************************/
/*************                     ***********************/
/*************  CONTROL POINTS     ***********************/
/*************                     ***********************/
/*********************************************************/



void	//crea el rectangulo parametrico asociado a una superficie 3d abierta sin huecos y asocia a cada (x,y,z) de sus vertices unas coordenadas parametricas (u,w)
discretizarFigura2d(	punto	toroDeformado[DIV_U][DIV_C+1][DIV_C+1],
						int		n,
						int		u
						)
{
	int i,j,k;
	double* termIndependientes_real;
	double* termIndependientes_img;
	sparseMatrix pesos;
	double* solucionU;
	double* solucionW;
	long int nonzero=320*(DIV_C+1)*(DIV_C+1)*3;
	termIndependientes_real=(double*)malloc(sizeof(double)*(DIV_C+1)*(DIV_C+1)*3);for (i = 0; i < (DIV_C+1)*(DIV_C+1)*3; i++){termIndependientes_real[i]=0;};
	termIndependientes_img=(double*)malloc(sizeof(double)*(DIV_C+1)*(DIV_C+1)*3); for (i = 0; i < (DIV_C+1)*(DIV_C+1)*3; i++){termIndependientes_img[i]=0;};

//GENERAMOS MATRIZ DE PESOS, DONDE LOS INDICES DE LA MATRIZ SON i Y j (NODOS QUE UNE LA ARISTA)

	pesos.i=(long int*)malloc(sizeof(long int)*nonzero);
	pesos.j=(long int*)malloc(sizeof(long int)*nonzero);
	pesos.val=(double*)malloc(sizeof(double)*nonzero);
	solucionU=(double*)malloc(sizeof(double)*(DIV_C+1)*(DIV_C+1)*3); 
	solucionW=(double*)malloc(sizeof(double)*(DIV_C+1)*(DIV_C+1)*3);
	
	for (i = 0; i < u; i++){
		
		
		//INICIALIZACION
		
		for (k=0; k<nonzero; k++){
			pesos.i[k]=NO_INICIALIZADO;
			pesos.j[k]=NO_INICIALIZADO;
			pesos.val[k]=0;
		}
		pesos.n_total=0;
		for (k = 0; k < (DIV_C+1)*(DIV_C+1)*3; k++){
			termIndependientes_real[k]=0;
			termIndependientes_img[k]=0;
		}

		//CREACIÓN DE MATRIZ DE PESOS

		for (j = 0; j <= DIV_C ; j++){
			for (k = 0; k <= DIV_C ; k++){
				int pt_id=j+(DIV_C+1)*k;
				if ((j==0) || (j==DIV_C) || (k==0) || (k==DIV_C)) {
					sumarElementoSparse(&pesos,3*pt_id,3*pt_id,1);
					sumarElementoSparse(&pesos,3*pt_id+1,3*pt_id+1,1);
					sumarElementoSparse(&pesos,3*pt_id+2,3*pt_id+2,1);
					termIndependientes_real[3*pt_id]=toroDeformado[i][j][k].x;
					termIndependientes_real[3*pt_id+1]=toroDeformado[i][j][k].y;
					termIndependientes_real[3*pt_id+2]=toroDeformado[i][j][k].z;
					termIndependientes_img[3*pt_id]=0;
					termIndependientes_img[3*pt_id+1]=0;
					termIndependientes_img[3*pt_id+2]=0;
				}else{
					sumarElementoSparse(&pesos,3*pt_id,3*pt_id,-8);
					sumarElementoSparse(&pesos,3*pt_id+1,3*pt_id+1,-8);
					sumarElementoSparse(&pesos,3*pt_id+2,3*pt_id+2,-8);
					
					sumarElementoSparse(&pesos,3*pt_id,3*pt_id+3*(DIV_C+1),2);
					sumarElementoSparse(&pesos,3*pt_id+1,3*pt_id+1+3*(DIV_C+1),2);
					sumarElementoSparse(&pesos,3*pt_id+2,3*pt_id+2+3*(DIV_C+1),2);
					
					sumarElementoSparse(&pesos,3*pt_id,3*pt_id-3*(DIV_C+1),2);
					sumarElementoSparse(&pesos,3*pt_id+1,3*pt_id+1-3*(DIV_C+1),2);
					sumarElementoSparse(&pesos,3*pt_id+2,3*pt_id+2-3*(DIV_C+1),2);
					
					sumarElementoSparse(&pesos,3*pt_id,3*pt_id-3,2);
					sumarElementoSparse(&pesos,3*pt_id+1,3*pt_id+1-3,2);
					sumarElementoSparse(&pesos,3*pt_id+2,3*pt_id+2-3,2);
					
					sumarElementoSparse(&pesos,3*pt_id,3*pt_id+3,2);
					sumarElementoSparse(&pesos,3*pt_id+1,3*pt_id+1+3,2);
					sumarElementoSparse(&pesos,3*pt_id+2,3*pt_id+2+3,2);
				
					termIndependientes_real[3*pt_id]=0;
					termIndependientes_real[3*pt_id+1]=0;
					termIndependientes_real[3*pt_id+2]=0;
					termIndependientes_img[3*pt_id]=0;
					termIndependientes_img[3*pt_id+1]=0;
					termIndependientes_img[3*pt_id+2]=0;
				}	
			}
		}
		
		
//RESOLUCION DEL SISTEMA	
	
		solver(	&pesos,
				termIndependientes_real,
				termIndependientes_img,
				(DIV_C+1)*(DIV_C+1)*3,
				solucionU,
				solucionW);
		
		for (j = 1; j < DIV_C; j++){
			for (k = 1; k < DIV_C ; k++){
				int pt_id=j+(DIV_C+1)*k;
				toroDeformado[i][j][k].x=solucionU[pt_id*3];
				toroDeformado[i][j][k].y=solucionU[pt_id*3+1];
				toroDeformado[i][j][k].z=solucionU[pt_id*3+2];
			}
		}
	}
}



void	//dados los control points de la superficie, agrega UxW niveles en el interior formando una cuadricula
ControlPointsOrto(			lista_caras*		caras,
							lista_puntos*		nodos,
							double*				vectorU,
							double*				vectorW)
{
	int u; //recorrido longitudinal al eje del toro (lo acompana)
	int w; //recorrido tangencial al eje del toro (gira a su alrededor)
	int r; //recorrido radial al eje (se aparta de el)
	int i,j; //contador
	punto toroParam[DIV_U+1][DIV_W+1][DIV_R+1];
	punto superficieDeformada[DIV_U+1][DIV_W+1];
	double umax=DIV_U+1;
	double wmax=DIV_W+1;
	double rmax=DIV_R+1;
	
	for (u = 0; u < umax; u++){
		for (w = 0; w < wmax ; w++){
			for (r = 0; r < rmax ; r++){
				toroParam[u][w][r].x= u/umax+(1/umax)/2;
				toroParam[u][w][r].y= w/wmax+(1/wmax)/2;
				toroParam[u][w][r].z=1-r/(rmax-1); //ojo, la capa r=0 es la exterior!		
			}
		}
	}
	
	for (u = 0; u < umax; u++){
		for (w = 0; w < wmax ; w++){			
			superficieDeformada[u][w]=coordParametricasAFisicas(caras,nodos,vectorU,vectorW,toroParam[u][w][0]);
		}
	}
	for (w = 0; w < wmax; w++){
		superficieDeformada[0][w].x=(superficieDeformada[1][w].x+superficieDeformada[DIV_U][w].x)/2;
		superficieDeformada[0][w].y=(superficieDeformada[1][w].y+superficieDeformada[DIV_U][w].y)/2;
		superficieDeformada[0][w].z=(superficieDeformada[1][w].z+superficieDeformada[DIV_U][w].z)/2;
	}
	for (u = 0; u < umax; u++){
		superficieDeformada[u][0].x=(superficieDeformada[u][1].x+superficieDeformada[u][DIV_W].x)/2;	
		superficieDeformada[u][0].y=(superficieDeformada[u][1].y+superficieDeformada[u][DIV_W].y)/2;	
		superficieDeformada[u][0].z=(superficieDeformada[u][1].z+superficieDeformada[u][DIV_W].z)/2;	
	}
	
	punto toroDeformado[DIV_U][DIV_C+1][DIV_C+1];
	
	
	if (fabs(superficieDeformada[0][0].z - superficieDeformada[0][1].z) > fabs(superficieDeformada[0][0].z - superficieDeformada[1][0].z)){
		// si anillos en W longitudinales 
		for (u = 0; u < umax; u++){
			for (w = 0; w < DIV_C; w++){
				toroDeformado[u][w][0]=superficieDeformada[u][w];
			}
			for (w = DIV_C; w < DIV_C*2; w++){
				toroDeformado[u][DIV_C][w-DIV_C]=superficieDeformada[u][w];
			}
			for (w = DIV_C*2; w < DIV_C*3; w++){
				toroDeformado[u][3*DIV_C-w][DIV_C]=superficieDeformada[u][w];
			}
			for (w = DIV_C*3; w < DIV_C*4; w++){
				toroDeformado[u][0][4*DIV_C-w]=superficieDeformada[u][w];
			}
			
			toroDeformado[u][0][0].x=(toroDeformado[u][1][0].x+toroDeformado[u][0][1].x)/2;
			toroDeformado[u][0][0].y=(toroDeformado[u][1][0].y+toroDeformado[u][0][1].y)/2;
			toroDeformado[u][0][0].z=(toroDeformado[u][1][0].z+toroDeformado[u][0][1].z)/2;
			
			toroDeformado[u][DIV_C][0].x=(toroDeformado[u][DIV_C-1][0].x+toroDeformado[u][DIV_C][1].x)/2;
			toroDeformado[u][DIV_C][0].y=(toroDeformado[u][DIV_C-1][0].y+toroDeformado[u][DIV_C][1].y)/2;
			toroDeformado[u][DIV_C][0].z=(toroDeformado[u][DIV_C-1][0].z+toroDeformado[u][DIV_C][1].z)/2;
			
			toroDeformado[u][DIV_C][DIV_C].x=(toroDeformado[u][DIV_C-1][DIV_C].x+toroDeformado[u][DIV_C][DIV_C-1].x)/2;
			toroDeformado[u][DIV_C][DIV_C].y=(toroDeformado[u][DIV_C-1][DIV_C].y+toroDeformado[u][DIV_C][DIV_C-1].y)/2;
			toroDeformado[u][DIV_C][DIV_C].z=(toroDeformado[u][DIV_C-1][DIV_C].z+toroDeformado[u][DIV_C][DIV_C-1].z)/2;
			
			toroDeformado[u][0][DIV_C].x=(toroDeformado[u][1][DIV_C].x+toroDeformado[u][0][DIV_C-1].x)/2;
			toroDeformado[u][0][DIV_C].y=(toroDeformado[u][1][DIV_C].y+toroDeformado[u][0][DIV_C-1].y)/2;
			toroDeformado[u][0][DIV_C].z=(toroDeformado[u][1][DIV_C].z+toroDeformado[u][0][DIV_C-1].z)/2;
			
			for (i = 1; i < DIV_C; i++){
				for (j = 1; j < DIV_C; j++){
					toroDeformado[u][i][j].x=0;
					toroDeformado[u][i][j].x += toroDeformado[u][0][j].x/i/i;
					toroDeformado[u][i][j].x += toroDeformado[u][DIV_C][j].x/(DIV_C-i)/(DIV_C-i);
					toroDeformado[u][i][j].x += toroDeformado[u][i][0].x/j/j;
					toroDeformado[u][i][j].x += toroDeformado[u][i][DIV_C].x/(DIV_C-j)/(DIV_C-j);
					toroDeformado[u][i][j].x /= (1.0/(double)i/(double)i+1.0/(double)(DIV_C-i)/(double)(DIV_C-i)+1.0/(double)j/(double)j+1.0/(double)(DIV_C-j)/(double)(DIV_C-j));
					
					toroDeformado[u][i][j].y=0;
					toroDeformado[u][i][j].y += toroDeformado[u][0][j].y/i/i;
					toroDeformado[u][i][j].y += toroDeformado[u][DIV_C][j].y/(DIV_C-i)/(DIV_C-i);
					toroDeformado[u][i][j].y += toroDeformado[u][i][0].y/j/j;
					toroDeformado[u][i][j].y += toroDeformado[u][i][DIV_C].y/(DIV_C-j)/(DIV_C-j);
					toroDeformado[u][i][j].y /= (1.0/(double)i/(double)i+1.0/(double)(DIV_C-i)/(double)(DIV_C-i)+1.0/(double)j/(double)j+1.0/(double)(DIV_C-j)/(double)(DIV_C-j));
					
					toroDeformado[u][i][j].z=0;
					toroDeformado[u][i][j].z += toroDeformado[u][0][j].z/i/i;
					toroDeformado[u][i][j].z += toroDeformado[u][DIV_C][j].z/(DIV_C-i)/(DIV_C-i);
					toroDeformado[u][i][j].z += toroDeformado[u][i][0].z/j/j;
					toroDeformado[u][i][j].z += toroDeformado[u][i][DIV_C].z/(DIV_C-j)/(DIV_C-j);
					toroDeformado[u][i][j].z /= (1.0/(double)i/(double)i+1.0/(double)(DIV_C-i)/(double)(DIV_C-i)+1.0/(double)j/(double)j+1.0/(double)(DIV_C-j)/(double)(DIV_C-j));
					
				}
			}
		}	
	}else{
		 // si anillos en W circunferenciales
		 for (w = 0; w < wmax; w++){
			for (u = 0; u < DIV_C; u++){
				toroDeformado[w][u][0]=superficieDeformada[u][w];
			}
			for (u = DIV_C; u < DIV_C*2; u++){
				toroDeformado[w][DIV_C][u-DIV_C]=superficieDeformada[u][w];
			}
			for (u = DIV_C*2; u < DIV_C*3; u++){
				toroDeformado[w][3*DIV_C-u][DIV_C]=superficieDeformada[u][w];
			}
			for (u = DIV_C*3; u < DIV_C*4; u++){
				toroDeformado[w][0][4*DIV_C-u]=superficieDeformada[u][w];
			}
			
			toroDeformado[u][0][0].x=(toroDeformado[u][1][0].x+toroDeformado[u][0][1].x)/2;
			toroDeformado[u][0][0].y=(toroDeformado[u][1][0].y+toroDeformado[u][0][1].y)/2;
			toroDeformado[u][0][0].z=(toroDeformado[u][1][0].z+toroDeformado[u][0][1].z)/2;
			
			toroDeformado[u][DIV_C][0].x=(toroDeformado[u][DIV_C-1][0].x+toroDeformado[u][DIV_C][1].x)/2;
			toroDeformado[u][DIV_C][0].y=(toroDeformado[u][DIV_C-1][0].y+toroDeformado[u][DIV_C][1].y)/2;
			toroDeformado[u][DIV_C][0].z=(toroDeformado[u][DIV_C-1][0].z+toroDeformado[u][DIV_C][1].z)/2;
			
			toroDeformado[u][DIV_C][DIV_C].x=(toroDeformado[u][DIV_C-1][DIV_C].x+toroDeformado[u][DIV_C][DIV_C-1].x)/2;
			toroDeformado[u][DIV_C][DIV_C].y=(toroDeformado[u][DIV_C-1][DIV_C].y+toroDeformado[u][DIV_C][DIV_C-1].y)/2;
			toroDeformado[u][DIV_C][DIV_C].z=(toroDeformado[u][DIV_C-1][DIV_C].z+toroDeformado[u][DIV_C][DIV_C-1].z)/2;
			
			toroDeformado[u][0][DIV_C].x=(toroDeformado[u][1][DIV_C].x+toroDeformado[u][0][DIV_C-1].x)/2;
			toroDeformado[u][0][DIV_C].y=(toroDeformado[u][1][DIV_C].y+toroDeformado[u][0][DIV_C-1].y)/2;
			toroDeformado[u][0][DIV_C].z=(toroDeformado[u][1][DIV_C].z+toroDeformado[u][0][DIV_C-1].z)/2;
			
			for (i = 1; i < DIV_C; i++){
				for (j = 1; j < DIV_C; j++){
					toroDeformado[w][i][j].x=0;
					toroDeformado[w][i][j].x += toroDeformado[w][0][j].x/i/i;
					toroDeformado[w][i][j].x += toroDeformado[w][DIV_C][j].x/(DIV_C-i)/(DIV_C-i);
					toroDeformado[w][i][j].x += toroDeformado[w][i][0].x/j/j;
					toroDeformado[w][i][j].x += toroDeformado[w][i][DIV_C].x/(DIV_C-j)/(DIV_C-j);
					toroDeformado[w][i][j].x /= (1.0/(double)i/(double)i+1.0/(double)(DIV_C-i)/(double)(DIV_C-i)+1.0/(double)j/(double)j+1.0/(double)(DIV_C-j)/(double)(DIV_C-j));
					
					toroDeformado[w][i][j].y=0;
					toroDeformado[w][i][j].y += toroDeformado[w][0][j].y/i/i;
					toroDeformado[w][i][j].y += toroDeformado[w][DIV_C][j].y/(DIV_C-i)/(DIV_C-i);
					toroDeformado[w][i][j].y += toroDeformado[w][i][0].y/j/j;
					toroDeformado[w][i][j].y += toroDeformado[w][i][DIV_C].y/(DIV_C-j)/(DIV_C-j);
					toroDeformado[w][i][j].y /= (1.0/(double)i/(double)i+1.0/(double)(DIV_C-i)/(double)(DIV_C-i)+1.0/(double)j/(double)j+1.0/(double)(DIV_C-j)/(double)(DIV_C-j));
					
					toroDeformado[w][i][j].z=0;
					toroDeformado[w][i][j].z += toroDeformado[w][0][j].z/i/i;
					toroDeformado[w][i][j].z += toroDeformado[w][DIV_C][j].z/(DIV_C-i)/(DIV_C-i);
					toroDeformado[w][i][j].z += toroDeformado[w][i][0].z/j/j;
					toroDeformado[w][i][j].z += toroDeformado[w][i][DIV_C].z/(DIV_C-j)/(DIV_C-j);
					toroDeformado[w][i][j].z /= (1.0/(double)i/(double)i+1.0/(double)(DIV_C-i)/(double)(DIV_C-i)+1.0/(double)j/(double)j+1.0/(double)(DIV_C-j)/(double)(DIV_C-j));
				}
			}
		}
	}
	//edicion para carpeta
	char aux_path[150];
	strcpy(aux_path, ACTUAL_PATH); 
	strcat(aux_path, "/controlpoints-orto.csv");
	FILE *f2 = fopen(aux_path, "w");
	if (f2 == NULL){
		printf("Error opening file!\n");
		exit(1);
	}
	fprintf(f2,"U,I,J,x,y,z");
	for (u = 0; u < umax; u++){
		for (i = 0; i <= DIV_C ; i++){
			for (j = 0; j <= DIV_C ; j++){
				fprintf(f2,"\n%d,%d,%d,%lf,%lf,%lf",u,i,j,toroDeformado[u][i][j].x,toroDeformado[u][i][j].y,toroDeformado[u][i][j].z);
			}
		}
	}
	fclose(f2);
}



vector	//a partir de una polilinea cerrada, ESTIMA la normal del plano cuya distancia a los puntos es menor
estimarPlanoRegresion(	lista_puntos 	a)
{
	int i;
	vector vector1,vector2,vector3,aux1,aux2;
	
	vector promedio={0,0,0};
	
	for (i = 0; i < a.n_total; i++)	{
		int zero=		 i;
		int quarter = 	(i+ (int) ceil( a.n_total / 4)) 		% a.n_total;
		int mid= 		(i+ (int) ceil( a.n_total / 2)) 		% a.n_total;
		int three_q=	(i+ (int) ceil(3 * a.n_total / 4))		% a.n_total;
		
		//calculo pseudonormal al plano, 1 componente
		aux1=a.coord[zero];
		aux2=a.coord[mid];
		vector1=vectorResta(aux2,aux1);
		
		//calculo pseudonormal al plano, 2 componente
		aux1=a.coord[quarter];
		aux2=a.coord[three_q];
		vector2=vectorResta(aux2,aux1);
		
		//calculo pseudonormal al plano, producto vectorial
		promedio=vectorSuma(promedio,vectorUnitario(prodVectorial(vector1,vector2)));
	}
	return vectorUnitario(promedio);
}


punto
refinarCentroRebanadaXYHillClimb(	lista_puntos 	a,
									punto			centro,
									int				n_rebanada)
{
	int i, nCandidatos=32;
	vector normal,vectorXproy,vectorYproy;
	double xmax, xmin,ymax,ymin;
	lista_puntos poligono;
	
	//printf("\n--Rebanada %d-- Refinamiento en XY, basado en algoritmo Hill Climbing",n_rebanada);
	
	
	//INIT
	
	//printf("\nCentro antes=(%f,%f,%f)",centro.x,centro.y,centro.z);
	punto actual={0,0,-1};
	punto mejorCandidato={0,0,-1};
	vector vectorX={1,0,0};
	vector vectorY={0,1,0};
	
	xmax=0;
	xmin=0;
	
	ymax=0;
	ymin=0;

	
	//PROYECTAR SOBRE PLANO
	
	normal=estimarPlanoRegresion(a);
	vectorXproy=vectorUnitario(prodVectorial(vectorY,normal));
	vectorYproy=vectorUnitario(prodVectorial(normal,vectorXproy));
	poligono=proyectarSobrePlano2d(a,centro,vectorXproy,vectorYproy);
		
	//CALCULAR BOUNDING BOX (Xmin,Ymin) a (Xmax,Ymax)
	
	double x,y,z;
	double sizex,sizey;
	double step;
	
	for (i = 0; i < poligono.n_total; i++)
	{
		x=poligono.coord[i].x;
		y=poligono.coord[i].y;
		xmin=(x<xmin) ? x : xmin;
		xmax=(x>xmax) ? x : xmax;
		ymin=(y<ymin) ? y : ymin;
		ymax=(y>ymax) ? y : ymax;
		
	}
	
	sizex=xmax-xmin;
	sizey=ymax-ymin;
	step= (sizex<sizey) ? sizex/25 : sizey/25;
	

	//BUSQUEDA DE MINIMOS LOCALES PARA LA FUNCION DE calidad
	
	lista_puntos listaCandidatos; listaCandidatos.n_total=nCandidatos;
	double anguloEntreCandidatos=2*M_PI/nCandidatos;
	int niveles=5;
	int direccionesDescenso=0;
	
	while(niveles>=0){
		direccionesDescenso=0;
		while(direccionesDescenso<nCandidatos){ //condicion de parada
			
			//CALCULAR calidad EN n DIRECCIONES DESDE PUNTO ACTUAL. AVISO: SE USA LA COORDENADA Z PARA ALMACENAR LA calidad DE UN PUNTO
			
			direccionesDescenso=0;
			mejorCandidato=actual;
			
			for (i = 0; i < nCandidatos; i++){
				listaCandidatos.coord[i].x= actual.x + step * cos(anguloEntreCandidatos*i);
				listaCandidatos.coord[i].y= actual.y + step * sin(anguloEntreCandidatos*i);
				listaCandidatos.coord[i].z= calidadCentro(listaCandidatos.coord[i],poligono);
				if (listaCandidatos.coord[i].z<actual.z){
					direccionesDescenso++;
				}
				if (listaCandidatos.coord[i].z>mejorCandidato.z){
					mejorCandidato=listaCandidatos.coord[i];
					//printf("\nNuevo centro (%f,%f), calidad %fº",mejorCandidato.x,mejorCandidato.y,180*asin(mejorCandidato.z)/M_PI);
				}
				
			}
			
			//MOVER CENTRO EN LA DIRECCION DE MAYOR AUMENTO (PROPORCIONAL A TAMANO DE BBOX)
			
			actual=mejorCandidato;

		}
		niveles--;
		step=step/4;
		//printf("\nStep down to %f",step);
	}
	
	//DESPROYECTAR
	
	punto nuevoCentro;
	nuevoCentro=vectorSuma(centro,vectorPorEscalar(vectorXproy,actual.x));
	nuevoCentro=vectorSuma(nuevoCentro,vectorPorEscalar(vectorYproy,actual.y));
	//printf("\nCentro despues=(%f,%f,%f)",nuevoCentro.x,nuevoCentro.y,nuevoCentro.z);
	return nuevoCentro;
	
}



punto	//para una polilinea cerrada a y un centro propuesto, busca moverse en perpendicular al plano para minimizar el maximo angulo de los rayos (centro-contorno) respecto al plano
refinarCentroRebanadaZ	(	lista_puntos 	a, 
							punto 			centro,
							int				n_rebanada)
{
	
	//printf("\n--Rebanada %d-- Refinamiento en Z, basado en minimos cuadrados de alabeos",n_rebanada);
	
	int i;
	int divisiones=100;
	double min,max,span,step;
	double* hList;
	double* vList;
	double alabeo, offset, alabeoMinimo, offsetOptimo, h, v, d ;
	hList=(double*)malloc(sizeof(double)*a.n_total);
	vList=(double*)malloc(sizeof(double)*a.n_total);
	vector normal=estimarPlanoRegresion(a);
		
	min=distanciaPuntoPlano(a.coord[0],centro,normal);
	max=min;
	for (i = 0; i < a.n_total; i++){
		v = vList[i] = distanciaPuntoPlano(a.coord[i],normal,centro); //aviso: va con signo, dependiendo de la direccion de la normal
		d = moduloVector(vectorResta(a.coord[i],centro));
		h = hList[i] = sqrt(d*d-v*v); 
		min=(v<min)?v:min;
		max=(v>max)?v:max;
	}
	span=max-min;
	step=span/(double)divisiones;
	
	
	alabeoMinimo=M_PI/2; //90 grados como cota superior
	offsetOptimo=min;
	for (i = 1; i < divisiones; i++){
		offset = max-i*step;
		alabeo = calidadAltura(offset,vList,hList,a.n_total);
		//printf("\noffset[%d]=%f, alabeo=%f",i,offset,alabeo);
		if(alabeo < alabeoMinimo){
			alabeoMinimo=alabeo;
			offsetOptimo=offset;
		}
	}
	
	free(vList);
	free(hList);
	
	centro=vectorSuma(centro,vectorPorEscalar(normal,offsetOptimo));

	//printf("\nDesviacion elegida = %f mm",offsetOptimo);

	return centro;
}


punto
refinarCentroRebanadaXYAnguloBarrido	(	lista_puntos 	a, 
											punto 			centro,
											int				n_rebanada)
{
	int i, nCandidatos=32;
	vector normal,vectorXproy,vectorYproy;
	double xmax, xmin,ymax,ymin;
	lista_puntos poligono;
	
	//printf("\n--Rebanada %d-- Refinamiento en XY, basado en angulo barrido",n_rebanada);
	
	
	//INIT
	
	//printf("\nCentro antes=(%f,%f,%f)",centro.x,centro.y,centro.z);
	punto actual={0,0,-1};
	punto mejorCandidato={0,0,-1};
	vector vectorX={1,0,0};
	vector vectorY={0,1,0};
	
	xmax=0;
	xmin=0;
	
	ymax=0;
	ymin=0;

	
	//PROYECTAR SOBRE PLANO
	
	normal=estimarPlanoRegresion(a);
	vectorXproy=vectorUnitario(prodVectorial(vectorY,normal));
	vectorYproy=vectorUnitario(prodVectorial(normal,vectorXproy));
	poligono=proyectarSobrePlano2d(a,centro,vectorXproy,vectorYproy);
		
	//CALCULAR BOUNDING BOX (Xmin,Ymin) a (Xmax,Ymax)
	
	double x,y,z;
	double sizex,sizey;
	double step;
	
	for (i = 0; i < poligono.n_total; i++)
	{
		x=poligono.coord[i].x;
		y=poligono.coord[i].y;
		xmin=(x<xmin) ? x : xmin;
		xmax=(x>xmax) ? x : xmax;
		ymin=(y<ymin) ? y : ymin;
		ymax=(y>ymax) ? y : ymax;
		
	}
	
	sizex=xmax-xmin;
	sizey=ymax-ymin;
	step= (sizex<sizey) ? sizex/25 : sizey/25;
	

	//BUSQUEDA DE MINIMOS LOCALES PARA LA FUNCION DE calidad
	
	lista_puntos listaCandidatos; listaCandidatos.n_total=nCandidatos;
	double anguloEntreCandidatos=2*M_PI/nCandidatos;
	int niveles=5;
	int direccionesDescenso=0;
	
	while(niveles>=0){
		direccionesDescenso=0;
		while(direccionesDescenso<nCandidatos){ //condicion de parada
			
			//CALCULAR calidad EN n DIRECCIONES DESDE PUNTO ACTUAL. AVISO: SE USA LA COORDENADA Z PARA ALMACENAR LA calidad DE UN PUNTO
			
			direccionesDescenso=0;
			mejorCandidato=actual;
			
			for (i = 0; i < nCandidatos; i++){
				listaCandidatos.coord[i].x= actual.x + step * cos(anguloEntreCandidatos*i);
				listaCandidatos.coord[i].y= actual.y + step * sin(anguloEntreCandidatos*i);
				listaCandidatos.coord[i].z= calidadBarrido(listaCandidatos.coord[i],poligono);
				if (listaCandidatos.coord[i].z<actual.z){
					direccionesDescenso++;
				}
				if (listaCandidatos.coord[i].z>mejorCandidato.z){
					mejorCandidato=listaCandidatos.coord[i];
					//printf("\nNuevo centro (%f,%f), calidad %fº",mejorCandidato.x,mejorCandidato.y,180*asin(mejorCandidato.z)/M_PI);
				}
				
			}
			
			//MOVER CENTRO EN LA DIRECCION DE MAYOR AUMENTO (PROPORCIONAL A TAMANO DE BBOX)
			
			actual=mejorCandidato;

		}
		niveles--;
		step=step/4;
		//printf("\nStep down to %f",step);
	}
	
	//DESPROYECTAR
	
	punto nuevoCentro;
	nuevoCentro=vectorSuma(centro,vectorPorEscalar(vectorXproy,actual.x));
	nuevoCentro=vectorSuma(nuevoCentro,vectorPorEscalar(vectorYproy,actual.y));
	//printf("\nCentro despues=(%f,%f,%f)",nuevoCentro.x,nuevoCentro.y,nuevoCentro.z);
	return nuevoCentro;
	
}



punto
refinarCentroRebanadaXYJacobiano		(	lista_puntos 	a, 
											punto 			centro,
											int				n_rebanada)
{
	int i, nCandidatos=32;
	vector normal,vectorXproy,vectorYproy;
	double xmax, xmin,ymax,ymin;
	lista_puntos poligono;
	
	//printf("\n--Rebanada %d-- Refinamiento en XY, basado en angulo barrido",n_rebanada);
	
	
	//INIT
	
	//printf("\nCentro antes=(%f,%f,%f)",centro.x,centro.y,centro.z);
	punto actual={0,0,-1};
	punto mejorCandidato={0,0,-1};
	vector vectorX={1,0,0};
	vector vectorY={0,1,0};
	
	xmax=0;
	xmin=0;
	
	ymax=0;
	ymin=0;

	
	//PROYECTAR SOBRE PLANO
	
	normal=estimarPlanoRegresion(a);
	vectorXproy=vectorUnitario(prodVectorial(vectorY,normal));
	vectorYproy=vectorUnitario(prodVectorial(normal,vectorXproy));
	poligono=proyectarSobrePlano2d(a,centro,vectorXproy,vectorYproy);
		
	//CALCULAR BOUNDING BOX (Xmin,Ymin) a (Xmax,Ymax)
	
	double x,y,z;
	double sizex,sizey;
	double step;
	
	for (i = 0; i < poligono.n_total; i++)
	{
		x=poligono.coord[i].x;
		y=poligono.coord[i].y;
		xmin=(x<xmin) ? x : xmin;
		xmax=(x>xmax) ? x : xmax;
		ymin=(y<ymin) ? y : ymin;
		ymax=(y>ymax) ? y : ymax;
		
	}
	
	sizex=xmax-xmin;
	sizey=ymax-ymin;
	step= (sizex<sizey) ? sizex/25 : sizey/25;
	

	//BUSQUEDA DE MINIMOS LOCALES PARA LA FUNCION DE calidad
	
	lista_puntos listaCandidatos; listaCandidatos.n_total=nCandidatos;
	double anguloEntreCandidatos=2*M_PI/nCandidatos;
	int niveles=5;
	int direccionesDescenso=0;
	
	while(niveles>=0){
		direccionesDescenso=0;
		while(direccionesDescenso<nCandidatos){ //condicion de parada
			
			//CALCULAR calidad EN n DIRECCIONES DESDE PUNTO ACTUAL. AVISO: SE USA LA COORDENADA Z PARA ALMACENAR LA calidad DE UN PUNTO
			
			direccionesDescenso=0;
			mejorCandidato=actual;
			
			for (i = 0; i < nCandidatos; i++){
				listaCandidatos.coord[i].x= actual.x + step * cos(anguloEntreCandidatos*i);
				listaCandidatos.coord[i].y= actual.y + step * sin(anguloEntreCandidatos*i);
				listaCandidatos.coord[i].z= calidadJacobiano(listaCandidatos.coord[i],poligono);
				if (listaCandidatos.coord[i].z<actual.z){
					direccionesDescenso++;
				}
				if (listaCandidatos.coord[i].z>mejorCandidato.z){
					mejorCandidato=listaCandidatos.coord[i];
					printf("\nNew center (%f,%f), quality %f",mejorCandidato.x,mejorCandidato.y,mejorCandidato.z);
				}
				
			}
			
			//MOVER CENTRO EN LA DIRECCION DE MAYOR AUMENTO (PROPORCIONAL A TAMANO DE BBOX)
			
			actual=mejorCandidato;

		}
		niveles--;
		step=step/4;
		//printf("\nStep down to %f",step);
	}
	
	//DESPROYECTAR
	
	punto nuevoCentro;
	nuevoCentro=vectorSuma(centro,vectorPorEscalar(vectorXproy,actual.x));
	nuevoCentro=vectorSuma(nuevoCentro,vectorPorEscalar(vectorYproy,actual.y));
	//printf("\nCentro despues=(%f,%f,%f)",nuevoCentro.x,nuevoCentro.y,nuevoCentro.z);
	return nuevoCentro;
	
}



punto	//para una polilinea cerrada a y un centro propuesto, mueve ese centro intentando que ningun rayo desde el centro hasta un punto del contorno cruze el contorno mas de una vez ("mordedura") 
refinarCentroRebanadaXYTangente	(	lista_puntos 	a, 
									punto 			centro,
									int				n_rebanada)
{
	
	punto aux1,aux2;
	vector vector1,vector2,vector3,producto1;
	vector correccion;
	int iteracion=0;
	int i,j;
	
	//printf("\n--Rebanada %d-- Refinamiento en XY, basado en angulo minimo. Cota inferior %d",n_rebanada,TANGENT_ANGLE_THRESHOLD_DEGREES);
	
	double cotaInferiorAngulo=TANGENT_ANGLE_THRESHOLD_DEGREES*(M_PI/180); //aviso: radians
	
	//sacamos borde de rebanada a fichero, para debugging
	
	//edicion para carpeta
	char aux_path[150];
	strcpy(aux_path, ACTUAL_PATH); 
	strcat(aux_path, "/iteracion_centro/rebanada%d.csv");
	
	char filename[100];
	sprintf(filename,aux_path,n_rebanada);
	FILE *out_file = fopen(filename, "w+");
	fprintf(out_file,"n,iter,x,y,z");
	for(i=0;i<a.n_total;i++){
		fprintf(out_file,"\n%d,0,%lf,%lf,%lf",i,a.coord[i].x,a.coord[i].y,a.coord[i].z);
	}
	fclose(out_file);
	
	//proyección de la polilinea sobre un plano de regresion
	
	vector normal=estimarPlanoRegresion(a);
	lista_puntos b=proyectarSobrePlano3d(a,centro,normal);
	
	
	//corrección del centro propuesto, en paralelo al plano de regresion
	
	int racha=0;
	
	//Si no logra encontrar solucion, parar a este numero de iteraciones
	int act_iterations = 0;
	int max_iterations = 200000;
	while( (racha<2*b.n_total) && (act_iterations<max_iterations)){
		i= (i) % b.n_total;
		j= (i+1) % b.n_total;
		
		aux1=b.coord[i];
		aux2=b.coord[j];
		
		vector1=vectorSuma(aux1,vectorPorEscalar(centro,-1));
		vector2=vectorSuma(aux2,vectorPorEscalar(aux1,-1));
		vector3=vectorSuma(aux2,vectorPorEscalar(centro,-1));
		
		double angulo=anguloVectores(vector1,vector2);
		producto1=prodVectorial(vector1,vector2);
		double sentidoGiro= prodEscalar(normal,producto1);
		
		if( (sentidoGiro<=0) || (sin(angulo)<=sin(cotaInferiorAngulo)) ){
			correccion=prodVectorial(normal,vector2);	//aviso: quitada correccion unitaria
			correccion=vectorPorEscalar(correccion,0.5);
			centro=vectorSuma(centro,correccion);
			//print to file
			
				FILE *out_file=fopen(filename, "a");
				printf("\nIter:%d\tSlice  %d\tEdge:%d-%d \t angle(deg):%f",iteracion,n_rebanada,i,j,angulo*180/M_PI);
				fprintf(out_file,"\n-1,%d,%lf,%lf,%lf",iteracion,centro.x,centro.y,centro.z);
				fclose(out_file);
			
			racha=0;
			iteracion++;
		}else{
			i++;
		}
		racha++;
		act_iterations++;		
	}
	//Comprobación de que las iteraciones no entran en bucle
	if(act_iterations>=max_iterations){
		printf("\nError, max iterations reached with no solution. Please, consider to change the tolerance.");
		exit(EXIT_FAILURE);
	}
	return centro;
}



punto	//para una polilinea cerrada, propone un centro desde el que mallar radialmente los elementos interiores
centroRebanada(	punto		toroDeformado[DIV_U][DIV_W][DIV_R],
				int			size,
				int			dir,
				int			u,
				int			w)
{

	double pesoTotal=0;
	double peso;
	int n_rebanada=NO_INICIALIZADO;

	punto centro= {0,0,0};
	
	lista_puntos anillo; anillo.n_total=0;
	
	//centro de gravedad: sistema de n masas puntuales situadas en el centro de cada arista y con peso igual a la longitud de la arista elevada a DISTANCE_WEIGHT_EXPONENT 
	
	if (dir == 0) {
		for (w = 0; w < DIV_W-1 ; w++){
			peso	=	pow	(	pow(toroDeformado[u][w][0].x-toroDeformado[u][w+1][0].x,2)+   //aviso: ponderado por longitud de arista al cuadrado
								pow(toroDeformado[u][w][0].y-toroDeformado[u][w+1][0].y,2)+
								pow(toroDeformado[u][w][0].z-toroDeformado[u][w+1][0].z,2),		DISTANCE_WEIGHT_EXPONENT/2);
			pesoTotal+=peso;
			centro.x+=peso*(toroDeformado[u][w][0].x+toroDeformado[u][w+1][0].x)/2;
			centro.y+=peso*(toroDeformado[u][w][0].y+toroDeformado[u][w+1][0].y)/2;
			centro.z+=peso*(toroDeformado[u][w][0].z+toroDeformado[u][w+1][0].z)/2;
			anillo.coord[anillo.n_total++]=toroDeformado[u][w][0];
		
		}
		centro.x/=pesoTotal;
		centro.y/=pesoTotal;
		centro.z/=pesoTotal;
		n_rebanada=u;
	}else{
		for (u = 0; u < DIV_U-1 ; u++){
			peso	=	pow	(	pow(toroDeformado[u][w][0].x-toroDeformado[u+1][w][0].x,2)+
								pow(toroDeformado[u][w][0].y-toroDeformado[u+1][w][0].y,2)+
								pow(toroDeformado[u][w][0].z-toroDeformado[u+1][w][0].z,2),		DISTANCE_WEIGHT_EXPONENT/2);
			pesoTotal+=peso;
			
			centro.x+=peso*(toroDeformado[u][w][0].x+toroDeformado[u+1][w][0].x)/2;
			centro.y+=peso*(toroDeformado[u][w][0].y+toroDeformado[u+1][w][0].y)/2;
			centro.z+=peso*(toroDeformado[u][w][0].z+toroDeformado[u+1][w][0].z)/2;
			anillo.coord[anillo.n_total++]=toroDeformado[u][w][0];
		}
		centro.x/=pesoTotal;
		centro.y/=pesoTotal;
		centro.z/=pesoTotal;
		n_rebanada=w;
	}
	
	centro=refinarCentroRebanadaXYTangente(anillo,centro,n_rebanada);
	//centro=refinarCentroRebanadaXYHillClimb(anillo,centro,n_rebanada);
	//centro=refinarCentroRebanadaXYJacobiano(anillo,centro,n_rebanada);
	//centro=refinarCentroRebanadaXYAnguloBarrido(anillo,centro,n_rebanada);
	centro=refinarCentroRebanadaZ(anillo,centro,n_rebanada);
	//printf("\n");
	
	return centro;
}



void
ControlPointsRadial(		lista_caras*		caras,
							lista_puntos*		nodos,
							double*				vectorU,
							double*				vectorW)
{
	int u; //recorrido longitudinal al eje del toro (lo acompana)
	int w; //recorrido tangencial al eje del toro (gira a su alrededor)
	int r; //recorrido radial al eje (se aparta de el)
	punto toroParam[DIV_U][DIV_W][DIV_R];
	punto toroDeformado[DIV_U][DIV_W][DIV_R];
	punto centroDeformado;
	double umax=DIV_U;
	double wmax=DIV_W;
	double rmax=DIV_R;
	
	
	for (u = 0; u < DIV_U; u++){
		for (w = 0; w < DIV_W ; w++){
			for (r = 0; r < DIV_R ; r++){
				toroParam[u][w][r].x= u/umax+(1/umax)/2;
				toroParam[u][w][r].y= w/wmax+(1/wmax)/2;
				toroParam[u][w][r].z=1-r/(rmax-1); //ojo, la capa r=0 es la exterior!		
			}
		}
	}
	
	for (u = 0; u < DIV_U; u++){
		for (w = 0; w < DIV_W ; w++){			
			toroDeformado[u][w][0]=coordParametricasAFisicas(caras,nodos,vectorU,vectorW,toroParam[u][w][0]);
		}
	}
	
	//DESCOMENTAR PARA SACAR A DISCO LOS CONTROL POINTS DE LA SUPERFICIE EN UN ARCHIVO APARTE
	
	/*
	FILE *f4 = fopen("controlpoints-superficie.csv", "w");
	if (f4 == NULL){
		printf("Error opening file!\n");
		exit(1);
	}
	fprintf(f4,"U,W,R,x,y,z");
	for (u = 0; u < DIV_U; u++){
		for (w = 0; w < DIV_W ; w++){
			//for (r = 0; r < DIV_R ; r++){
				fprintf(f4,"\n%d,%d,%d,%lf,%lf,%lf",u,w,r,toroDeformado[u][w][0].x,toroDeformado[u][w][0].y,toroDeformado[u][w][0].z);
			//}
		}
	}
	*/
	
	
	//calculo de la orientacion del agujero (buscamos que sea paralelo a z en lo posible)
	
	if (fabs(toroDeformado[0][0][0].z - toroDeformado[0][1][0].z) > fabs(toroDeformado[0][0][0].z - toroDeformado[1][0][0].z)){
		for (u = 0; u < DIV_U; u++){ // u diferencia los distintos anillos	
			centroDeformado=centroRebanada(toroDeformado,DIV_U,0,u,w);
			for (w = 0; w < DIV_W ; w++){ // w diferencia puntos dentro de un anillo
				for (r = 1; r < DIV_R; r++){
					toroDeformado[u][w][r].x=toroDeformado[u][w][0].x+(centroDeformado.x-toroDeformado[u][w][0].x)*((double)r/(double)(DIV_R-1));
					toroDeformado[u][w][r].y=toroDeformado[u][w][0].y+(centroDeformado.y-toroDeformado[u][w][0].y)*((double)r/(double)(DIV_R-1));
					toroDeformado[u][w][r].z=toroDeformado[u][w][0].z+(centroDeformado.z-toroDeformado[u][w][0].z)*((double)r/(double)(DIV_R-1));
				}
			}
		}
	} else {
		for (w = 0; w < DIV_W; w++){ // w diferencia los distintos anillos
			centroDeformado=centroRebanada(toroDeformado,DIV_W,1,u,w);
			for (u = 0; u < DIV_U ; u++){ // u diferencia puntos dentro de un anillo
				for (r = 1; r < DIV_R; r++){
					toroDeformado[u][w][r].x=toroDeformado[u][w][0].x+(centroDeformado.x-toroDeformado[u][w][0].x)*((double)r/(double)(DIV_R-1));
					toroDeformado[u][w][r].y=toroDeformado[u][w][0].y+(centroDeformado.y-toroDeformado[u][w][0].y)*((double)r/(double)(DIV_R-1));
					toroDeformado[u][w][r].z=toroDeformado[u][w][0].z+(centroDeformado.z-toroDeformado[u][w][0].z)*((double)r/(double)(DIV_R-1));
				}
			}
		}
	}
	
	//suavizado
	
	for (w = 0; w < DIV_W; w++){
		for(r = 0; r < DIV_R ; r++){
			toroDeformado[0][w][r].x=(toroDeformado[1][w][r].x+toroDeformado[DIV_U-1][w][r].x)/2;
			toroDeformado[0][w][r].y=(toroDeformado[1][w][r].y+toroDeformado[DIV_U-1][w][r].y)/2;
			toroDeformado[0][w][r].z=(toroDeformado[1][w][r].z+toroDeformado[DIV_U-1][w][r].z)/2;
		}
	}
	
	for (u = 0; u < DIV_U; u++){
		for(r = 0; r < DIV_R ; r++){
			toroDeformado[u][0][r].x=(toroDeformado[u][1][r].x+toroDeformado[u][DIV_W-1][r].x)/2;	
			toroDeformado[u][0][r].y=(toroDeformado[u][1][r].y+toroDeformado[u][DIV_W-1][r].y)/2;	
			toroDeformado[u][0][r].z=(toroDeformado[u][1][r].z+toroDeformado[u][DIV_W-1][r].z)/2;	
		}
	}
	
	//Salida a disco
	
	//edicion para carpeta
	char aux_path[150];
	strcpy(aux_path, ACTUAL_PATH); 
	strcat(aux_path, "/controlpoints-radial.csv");
	
	FILE *f2 = fopen(aux_path, "w");
	if (f2 == NULL){
		printf("Error opening file!\n");
		exit(1);
	}
	fprintf(f2,"U,W,R,x,y,z");

	for (u = 0; u < DIV_U; u++){
		for (w = 0; w < DIV_W ; w++){
			for (r = 0; r < DIV_R ; r++){
				fprintf(f2,"\n%d,%d,%d,%lf,%lf,%lf",u,w,r,toroDeformado[u][w][r].x,toroDeformado[u][w][r].y,toroDeformado[u][w][r].z);	
			}
		}
	}
	
}




void
ControlPointsChord(			lista_caras*		caras,
							lista_puntos*		nodos,
							double*				vectorU,
							double*				vectorW)
{
	int u; //recorrido longitudinal al eje del toro (lo acompana)
	int w; //recorrido tangencial al eje del toro (gira a su alrededor)
	int r; //recorrido radial al eje (se aparta de el)
	int i,j; //contador
	punto toroParam[DIV_U+1][DIV_W+1][DIV_R+1];
	punto superficieDeformada[DIV_U+1][DIV_W+1];
	double umax=DIV_U+1;
	double wmax=DIV_W+1;
	double rmax=DIV_R+1;
	
	for (u = 0; u <= DIV_U; u++){
		for (w = 0; w <= DIV_W ; w++){
			for (r = 0; r < DIV_R ; r++){
				toroParam[u][w][r].x= u/umax+(1/umax)/2;
				toroParam[u][w][r].y= w/wmax+(1/wmax)/2;
				toroParam[u][w][r].z=1-r/(rmax-1); //ojo, la capa r=0 es la exterior!		
			}
		}
	}
	
	for (u = 0; u < umax; u++){
		for (w = 0; w < wmax ; w++){			
			superficieDeformada[u][w]=coordParametricasAFisicas(caras,nodos,vectorU,vectorW,toroParam[u][w][0]);
		}
	}
	
	punto toroDeformado[DIV_U+1][DIV_C+1][DIV_C+1]; 
	if (fabs(superficieDeformada[0][0].z - superficieDeformada[0][1].z) > fabs(superficieDeformada[0][0].z - superficieDeformada[1][0].z)){
		// si anillos en W longitudinales 
		for (u = 0; u <= DIV_U; u++){
			for (w = 0; w < DIV_C; w++){
				toroDeformado[u][w][0]=superficieDeformada[u][w];
			}
			for (w = DIV_C; w < DIV_C*2; w++){
				toroDeformado[u][DIV_C][w-DIV_C]=superficieDeformada[u][w];
			}
			for (w = DIV_C*2; w < DIV_C*3; w++){
				toroDeformado[u][3*DIV_C-w][DIV_C]=superficieDeformada[u][w];
			}
			for (w = DIV_C*3; w < DIV_C*4; w++){
				toroDeformado[u][0][4*DIV_C-w]=superficieDeformada[u][w];
			}
			
			toroDeformado[u][0][0].x=(toroDeformado[u][1][0].x+toroDeformado[u][0][1].x)/2;
			toroDeformado[u][0][0].y=(toroDeformado[u][1][0].y+toroDeformado[u][0][1].y)/2;
			toroDeformado[u][0][0].z=(toroDeformado[u][1][0].z+toroDeformado[u][0][1].z)/2;
			
			toroDeformado[u][DIV_C][0].x=(toroDeformado[u][DIV_C-1][0].x+toroDeformado[u][DIV_C][1].x)/2;
			toroDeformado[u][DIV_C][0].y=(toroDeformado[u][DIV_C-1][0].y+toroDeformado[u][DIV_C][1].y)/2;
			toroDeformado[u][DIV_C][0].z=(toroDeformado[u][DIV_C-1][0].z+toroDeformado[u][DIV_C][1].z)/2;
			
			toroDeformado[u][DIV_C][DIV_C].x=(toroDeformado[u][DIV_C-1][DIV_C].x+toroDeformado[u][DIV_C][DIV_C-1].x)/2;
			toroDeformado[u][DIV_C][DIV_C].y=(toroDeformado[u][DIV_C-1][DIV_C].y+toroDeformado[u][DIV_C][DIV_C-1].y)/2;
			toroDeformado[u][DIV_C][DIV_C].z=(toroDeformado[u][DIV_C-1][DIV_C].z+toroDeformado[u][DIV_C][DIV_C-1].z)/2;
			
			toroDeformado[u][0][DIV_C].x=(toroDeformado[u][1][DIV_C].x+toroDeformado[u][0][DIV_C-1].x)/2;
			toroDeformado[u][0][DIV_C].y=(toroDeformado[u][1][DIV_C].y+toroDeformado[u][0][DIV_C-1].y)/2;
			toroDeformado[u][0][DIV_C].z=(toroDeformado[u][1][DIV_C].z+toroDeformado[u][0][DIV_C-1].z)/2;
		}
		
		discretizarFigura2d(toroDeformado,DIV_C,DIV_U);
		
		for (i = 0; i <= DIV_C; i++){
			for (j = 0; j <= DIV_C; j++){
				toroDeformado[0][i][j].x=(toroDeformado[1][i][j].x+toroDeformado[DIV_U][i][j].x)/2;
				toroDeformado[0][i][j].y=(toroDeformado[1][i][j].y+toroDeformado[DIV_U][i][j].y)/2;
				toroDeformado[0][i][j].z=(toroDeformado[1][i][j].z+toroDeformado[DIV_U][i][j].z)/2;
			}
		}
	}else{
		 // si anillos en W circunferenciales
		 for (w = 0; w <= DIV_W; w++){
			for (u = 0; u < DIV_C; u++){
				toroDeformado[w][u][0]=superficieDeformada[w][u];
			}
			for (u = DIV_C; u < DIV_C*2; u++){
				toroDeformado[w][DIV_C][u-DIV_C]=superficieDeformada[w][u];
			}
			for (u = DIV_C*2; u < DIV_C*3; u++){
				toroDeformado[w][3*DIV_C-u][DIV_C]=superficieDeformada[w][u];
			}
			for (u = DIV_C*3; u < DIV_C*4; u++){
				toroDeformado[w][0][4*DIV_C-u]=superficieDeformada[w][u];
			}
			
			toroDeformado[w][0][0].x=(toroDeformado[w][1][0].x+toroDeformado[w][0][1].x)/2;
			toroDeformado[w][0][0].y=(toroDeformado[w][1][0].y+toroDeformado[w][0][1].y)/2;
			toroDeformado[w][0][0].z=(toroDeformado[w][1][0].z+toroDeformado[w][0][1].z)/2;
			
			toroDeformado[w][DIV_C][0].x=(toroDeformado[w][DIV_C-1][0].x+toroDeformado[w][DIV_C][1].x)/2;
			toroDeformado[w][DIV_C][0].y=(toroDeformado[w][DIV_C-1][0].y+toroDeformado[w][DIV_C][1].y)/2;
			toroDeformado[w][DIV_C][0].z=(toroDeformado[w][DIV_C-1][0].z+toroDeformado[w][DIV_C][1].z)/2;
			
			toroDeformado[w][DIV_C][DIV_C].x=(toroDeformado[w][DIV_C-1][DIV_C].x+toroDeformado[w][DIV_C][DIV_C-1].x)/2;
			toroDeformado[w][DIV_C][DIV_C].y=(toroDeformado[w][DIV_C-1][DIV_C].y+toroDeformado[w][DIV_C][DIV_C-1].y)/2;
			toroDeformado[w][DIV_C][DIV_C].z=(toroDeformado[w][DIV_C-1][DIV_C].z+toroDeformado[w][DIV_C][DIV_C-1].z)/2;
			
			toroDeformado[w][0][DIV_C].x=(toroDeformado[w][1][DIV_C].x+toroDeformado[w][0][DIV_C-1].x)/2;
			toroDeformado[w][0][DIV_C].y=(toroDeformado[w][1][DIV_C].y+toroDeformado[w][0][DIV_C-1].y)/2;
			toroDeformado[w][0][DIV_C].z=(toroDeformado[w][1][DIV_C].z+toroDeformado[w][0][DIV_C-1].z)/2;
		}
		
		discretizarFigura2d(toroDeformado,DIV_C,DIV_W);	
					
		for (i = 0; i <= DIV_C; i++){
			for (j = 0; j <= DIV_C; j++){
				toroDeformado[0][i][j].x=(toroDeformado[1][i][j].x+toroDeformado[DIV_W][i][j].x)/2;
				toroDeformado[0][i][j].y=(toroDeformado[1][i][j].y+toroDeformado[DIV_W][i][j].y)/2;
				toroDeformado[0][i][j].z=(toroDeformado[1][i][j].z+toroDeformado[DIV_W][i][j].z)/2;
			}
		}
	}
	
	//salida a disco
	
	//edicion para carpeta
	char aux_path[150];
	strcpy(aux_path, ACTUAL_PATH); 
	strcat(aux_path, "/controlpoints-chord.csv");
	
	FILE *f2 = fopen(aux_path, "w");
	
	if (f2 == NULL){
		printf("Error opening file!\n");
		exit(1);
	}
	fprintf(f2,"U,I,J,x,y,z");
	for (u = 0; u <= DIV_U; u++){
		for (i = 0; i <= DIV_C ; i++){
			for (j = 0; j <= DIV_C ; j++){
				fprintf(f2,"\n%d,%d,%d,%lf,%lf,%lf",u,i,j,toroDeformado[u][i][j].x,toroDeformado[u][i][j].y,toroDeformado[u][i][j].z);
			}
		}
	}
	fclose(f2);
}



/*********************************************************/
/*************                     ***********************/
/*************      MAIN           ***********************/
/*************                     ***********************/
/*********************************************************/

int 
main(int argc, char *argv[])
{

	char filename[256];
	long int i,k;
	long int alternativas[T_MAX]; 
	long int n_nodos;
	long int nonzero;
	long int n_caras;
	sparseMatrix pesos;
	double* termIndependientes_real;
	double* termIndependientes_img;
	double* solucionU;
	double* solucionW;
	double posicionCorteZ;
	double posicionCorteY;
	lista_aristas bordeParametrico; bordeParametrico.n_total=0;
	int* repeticiones;
	lista_puntos nodos;
	lista_caras caras;
	
	
	
//LEEMOS DATOS DE FICHERO

	strncpy(filename,(argc>1)?argv[1]:"input.ply",256); //en caso de aportar nombre de fichero por linea de comandos, usarlo, si no buscar "input.ply"
	printf("\nReading file \"%s\"...",filename);
	leerFichero(filename,&nodos,&caras,&n_nodos,&n_caras);	
	printf("Listo");
	
//OBTENEMOS LA POSICION DE LOS DOS PLANOS DE CORTE
	if(argc==4){
		posicionCorteY=atof(argv[2]);
		posicionCorteZ=atof(argv[3]);
		if(posicionCorteY==0.0 || posicionCorteZ==0.0){ //sucede error al leer por linea de comandos, preguntar igualmente
			preguntarCentroGravedad(&nodos,&posicionCorteY,&posicionCorteZ);
		}else{
			printf("\nUsing cutting planes: y=%f, z=%f",posicionCorteY,posicionCorteZ);
		}
	}else if(argc==5){
		posicionCorteY=atof(argv[2]);
		posicionCorteZ=atof(argv[3]);
		if(posicionCorteY==0.0 || posicionCorteZ==0.0){ //sucede error al leer por linea de comandos, preguntar igualmente
			preguntarCentroGravedad(&nodos,&posicionCorteY,&posicionCorteZ);
		}else{
			printf("\nUsing cutting planes: y=%f, z=%f",posicionCorteY,posicionCorteZ);
		}
		double ANGLE_INPUT=atof(argv[4]);
		if(ANGLE_INPUT!=0) TANGENT_ANGLE_THRESHOLD_DEGREES= ANGLE_INPUT;
	}else if(argc==9){
		posicionCorteY=atof(argv[2]);
		posicionCorteZ=atof(argv[3]);
		if(posicionCorteY==0.0 || posicionCorteZ==0.0){ //sucede error al leer por linea de comandos, preguntar igualmente
			preguntarCentroGravedad(&nodos,&posicionCorteY,&posicionCorteZ);
		}else{
			printf("\nUsing cutting planes: y=%f, z=%f",posicionCorteY,posicionCorteZ);
		}
		double ANGLE_INPUT=atof(argv[4]);
		DIV_U = atoi(argv[5]) + 2;
		DIV_W = atoi(argv[6]) + 2;
		DIV_R = atoi(argv[7]) + 2;
		ACTUAL_PATH = argv[8];
		printf("\nInitial mesh size: %dx%dx%d",(DIV_U-2),(DIV_W-2), (DIV_R-2));
		if(ANGLE_INPUT!=0) TANGENT_ANGLE_THRESHOLD_DEGREES= ANGLE_INPUT;
	}else{
		preguntarCentroGravedad(&nodos,&posicionCorteY,&posicionCorteZ);
	}
	
//CORTAMOS LA FIGURA POR UN PLANO XY Y POR UNO ZX, OBTENEMOS 2 POLILINEAS 3D

	n_caras=seccionar2 (&nodos,&caras,posicionCorteZ,2,alternativas); 	
	n_caras=seccionar2 (&nodos,&caras,posicionCorteY,1,alternativas);
	printf("\nDetection of related components ..."); fflush(stdout);
	identificarBordes(&caras, &nodos, alternativas, posicionCorteY, posicionCorteZ);
	printf("\nDifferentiation of edge and interior elements ...");fflush(stdout);
	bordeParametrico=identificarBordes(&caras, &nodos, alternativas, posicionCorteY,posicionCorteZ);
	printf("\nBorder continuity control ...");fflush(stdout);
	corregirMuescas(&bordeParametrico,&nodos,&caras);
	repeticiones=(int*)malloc(sizeof(int)*nodos.n_total); for (i = 0; i < nodos.n_total; i++){repeticiones[i]=0;}
	identificarEsquinas(bordeParametrico,&nodos,repeticiones);


//ESTIMACION DEL NIVEL DE CONECTIVIDAD

	nonzero=16*2*n_caras;


//GENERAMOS MATRIZ DE PESOS, DONDE LOS INDICES DE LA MATRIZ SON i Y j (NODOS QUE UNE LA ARISTA)

	pesos.i=(long int*)malloc(sizeof(long int)*nonzero);
	pesos.j=(long int*)malloc(sizeof(long int)*nonzero);
	pesos.val=(double*)malloc(sizeof(double)*nonzero);
	for (k=0; k<nonzero; k++){
		pesos.i[k]=NO_INICIALIZADO;
		pesos.j[k]=NO_INICIALIZADO;
		pesos.val[k]=0;
	}
	pesos.n_total=0;
	printf("\nGenerating weight matrix...");
	calcularPeso(&caras,&nodos,&pesos);
	

	//FILE *FX=fopen("pesos.txt","w");
	//for(k=0;k<pesos.n_total;k++){
		//if (pesos.i[k]==pesos.j[k]){	
			//fprintf(FX,"[%ld] =\t%ld\t%ld\t%lf\n",k,pesos.i[k],pesos.j[k],pesos.val[k]);
		//}
	//}

//ENSAMBLADO DEL SISTEMA
	
	termIndependientes_real=(double*)malloc(sizeof(double)*nodos.n_total);
	termIndependientes_img=(double*)malloc(sizeof(double)*nodos.n_total);

	printf("\nInsertion of known coordinates (parametric rectangle border) ...");
	ecuacionesTriviales(	&pesos,
							nodos.n_total, 
							repeticiones,
							&bordeParametrico,
							&nodos, 
							&caras,
							termIndependientes_real,
							termIndependientes_img);

	//free(repeticiones);
	solucionU=(double*)malloc(sizeof(double)*nodos.n_total);
	solucionW=(double*)malloc(sizeof(double)*nodos.n_total);
	
	
	
//RESOLUCION DEL SISTEMA	
	printf("\nSolving linear system ...");
	
	solver(	&pesos,
			termIndependientes_real,
			termIndependientes_img,
			nodos.n_total,
			solucionU,
			solucionW);
	
 //SALIDA A FICHEIRO DO RECTANGULO PARAMETRICO
	
	//edicion para carpeta
	char aux_path[150];
	strcpy(aux_path, ACTUAL_PATH); 
	strcat(aux_path, "/param_rect.ply");
	
	FILE *f1 = fopen(aux_path, "w");
	if (f1 == NULL){
		printf("Error opening file!\n");
		exit(1);
	}
	fprintf(f1,"ply\n");
	fprintf(f1,"format ascii 1.0\n");
	fprintf(f1,"comment VTK generated PLY File\n");
	fprintf(f1,"obj_info vtkPolyData points and polygons: vtk4.0\n");
	fprintf(f1,"element vertex %ld \n",nodos.n_total);
	fprintf(f1,"property float x\n");
	fprintf(f1,"property float y\n");
	fprintf(f1,"property float z\n");
	fprintf(f1,"element face %ld \n",caras.n_total);
	fprintf(f1,"property list uchar int vertex_indices\n");
	fprintf(f1,"end_header\n");
	for (i = 0; i < nodos.n_total; i++){
		fprintf(f1, "%.4lf %.4lf %.4lf \n", solucionU[i],solucionW[i],0.0);
	}
	for (i = 0; i < caras.n_total; i++){
		fprintf(f1, "%d %ld %ld %ld \n", 3,caras.coord[i].id[0],caras.coord[i].id[1],caras.coord[i].id[2]);
	}
	
	fclose(f1);
	
// RELLENO DO INTERIOR DO SOLIDO CON CONTROL POINTS
	
	ControlPointsRadial(&caras,&nodos,solucionU,solucionW);
	//ControlPointsOrto(&caras,&nodos,solucionU,solucionW);
	//ControlPointsChord(&caras,&nodos,solucionU,solucionW);
	
	
	printf("\nEnd of program\n");
	
	//INFO ------ compilar con:
	//gcc <nome archivo>.o -lm -lumfpack -lamd -lcholmod -lcolamd -lccolamd -lcamd -lmetis -I$HOME/music/SuiteSparse/UMFPACK/Include/ -I$HOME/music/SuiteSparse/AMD/Include/ -o test; gdb -q test

	return 0;
	
}



