!--------------------------------------------------------------------
! subroutine to generate connectivities between processors.
! if the decomposition is unknown, set ConNP = 1, i.e. loop 
! through all the processors checking overlapping nodes.
!--------------------------------------------------------------------
subroutine gen_conn(ConNP,eps,xg,NP,NSD)

  integer,      intent(out) :: ConNP(:,:)
  real(kind=8), intent(in)  :: xg (:,:,:)
  integer,      intent(in)  :: NP,NSD
  
  real(kind=8) :: boxmax(NSD,NP),boxmin(NSD,NP)     
  real(kind=8) :: eps, tmp(3)
  integer :: p , p1, p2,i
  integer :: found
    
  eps = 1d-5 
    
  do p = 1, NP
    boxmin(:,p) = MINVAL(xg(:,:,p), DIM=1)
    boxmax(:,p) = MAXVAL(xg(:,:,p), DIM=1) 

   write(*,'(1I3,1x,3F13.7)') p, boxmin(:,p)
   write(*,'(1I3,1x,3F13.7)') p, boxmax(:,p)
  enddo
  
  tmp = MAXVAL(boxmax(:,:), DIM=2)&
      - MINVAL(boxmin(:,:), DIM=2) 

  write(*,*) MAXVAL(boxmax(:,:), DIM=2)
  write(*,*) MINVAL(boxmin(:,:), DIM=2)
  write(*,*) tmp

  eps = sqrt(sum(tmp*tmp))*1d-6
  write(*,*) "eps = ",eps

  ConNP = 0
  
  do p1 = 1, NP
    do p2 = p1+1, NP

      if ( (boxmin(1,p1).lt.(boxmax(1,p2)+eps)).and. &
           (boxmin(2,p1).lt.(boxmax(2,p2)+eps)).and. &
           (boxmin(3,p1).lt.(boxmax(3,p2)+eps)).and. &
           (boxmax(1,p1).gt.(boxmin(1,p2)-eps)).and. & 
           (boxmax(2,p1).gt.(boxmin(2,p2)-eps)).and. &
           (boxmax(3,p1).gt.(boxmin(3,p2)-eps)) )  then  

        ConNP(p1,p2) = 1
        ConNP(p2,p1) = 1       
      endif
      
      if (p2.eq.NP) then
         write (*,"(I2)", ADVANCE = "YES") ConNP(p1,p2)
      else
         write (*,"(I2)", ADVANCE = "NO" ) ConNP(p1,p2)
      endif	

    enddo    
  enddo
 
 

  


end subroutine gen_conn


!--------------------------------------------------------------------
! subroutine to generate connectivities between processors.
! if the decomposition is unknown, set ConNP = 1, i.e. loop 
! through all the processors checking overlapping nodes.
!--------------------------------------------------------------------
subroutine set_conn(ConNP)

  integer, intent(out) :: ConNP(:,:)

  ConNP = 0
  
  !---------------------
  ConNP(56:60,51:60) = 1
  ConNP(56:60,21:30) = 1

  ConNP(26:30,51:60) = 1
  ConNP(26:30,21:30) = 1

  !---------------------
  ConNP(51:55,46:60) = 1
  ConNP(51:55,16:30) = 1

  ConNP(21:25,46:60) = 1
  ConNP(21:25,16:30) = 1

  !---------------------
  ConNP(46:50,41:55) = 1
  ConNP(46:50,11:25) = 1

  ConNP(16:20,41:55) = 1
  ConNP(16:20,11:25) = 1

  !---------------------
  ConNP(41:45,36:50) = 1
  ConNP(41:45, 6:20) = 1

  ConNP(11:15,36:50) = 1
  ConNP(11:15, 6:20) = 1

  !---------------------
  ConNP(36:40,31:45) = 1
  ConNP(36:40, 1:15) = 1

  ConNP( 6:10,31:45) = 1
  ConNP( 6:10, 1:15) = 1

  !---------------------
  ConNP(31:35,31:40) = 1
  ConNP(31:35, 1:10) = 1

  ConNP( 1: 5,31:40) = 1
  ConNP( 1: 5, 1:10) = 1
  
end subroutine set_conn
