!--------------------------------------------------------------------
! program to read in the mesh files and generate the lwork files
!--------------------------------------------------------------------
program main

  implicit none
 
  integer :: NP, NSD 

  integer :: i, j, k, l, mf, ier, p
  integer :: MaxNNODZ
  integer :: Pu,Qu,Ru
  real(kind=8) :: tmp1, tmp2, tmp3

  real(kind=8) :: eps
  
  integer,      allocatable :: NNODZ(:)
  integer,      allocatable :: IPER(:,:), ConNP(:,:), IBC(:,:)
  integer, allocatable :: MCP(:), NCP(:), OCP(:)  
  real(kind=8), allocatable :: xg(:,:,:)

  character(len=80) :: fname


  i = iargc()
    
  if (i.eq.1) then
    call getarg(1,fname); read(fname,*) NP
  else   if (i.eq.3) then
    call getarg(1,fname); read(fname,*) Pu
    call getarg(2,fname); read(fname,*) Qu
    call getarg(3,fname); read(fname,*) Ru
    NP = Pu*Qu*Ru    
  else
    write (*,*) "Wrong input"
    call exit(-1)
  endif
  
  allocate(NNODZ(NP),MCP(NP), NCP(NP), OCP(NP))


  ! open all the mesh files and readin NNODZ
  ! figure out the max NNODZ of all processors
  do p = 1, NP
    mf = 11
    fname = trim('mesh'// trim(cname(p))//'.dat')
    open(mf, file = fname, status = 'old')

    read(mf,*) NSD
    read(mf,*) Pu, Qu, Ru
    read(mf,*) MCP(p), NCP(p), OCP(p)

    NNODZ(p) = MCP(p)*NCP(p)*OCP(p)

    close(mf)
  end do

  MaxNNODZ = maxval(NNODZ)

  write(*,*) '!== end of reading all NNODZ ==!', MaxNNODZ

  ! Allocate coord xg and IPER for all processors
  allocate(xg(MaxNNODZ,NSD,NP), IPER(MaxNNODZ,NP), &           
           IBC(MaxNNODZ,NP), ConNP(NP,NP), stat=ier)
  if (ier /= 0) stop 'Allocation Error: xg, IPER'
  xg   = 0.0d0
  IPER = 0
  IBC  = 0

  ! now loop through all mesh files and read in necessary information
  ! for generating lworks (need xg and iper only)
  do p = 1, NP

    mf = 11
    fname = trim('mesh'// trim(cname(p))//'.dat')
    open(mf, file = fname, status = 'old')
    read(mf,*) NSD
    read(mf,*) Pu, Qu, Ru
    read(mf,*) MCP(p), NCP(p), OCP(p)
    write(*,*) "Meta data for proc ",p
    ! Allocate arrays for knot vectors and for control net
!    allocate(U_KNOT(MCP+P+1), V_KNOT(NCP+Q+1), W_KNOT(OCP+R+1), stat=ier) 
!    if (ier /= 0) stop 'Allocation Error 1'
 
    read(mf,*) (tmp1, i = 1, MCP(p)+Pu+1)
    read(mf,*) (tmp1, i = 1, NCP(p)+Qu+1)
    read(mf,*) (tmp1, i = 1, OCP(p)+Ru+1)
    l = 0
    do k = 1, OCP(p)
      do j = 1, NCP(p)
        do i = 1, MCP(p)
          l = l + 1
          read(mf,'(3F13.9)') xg(l,1,p), xg(l,2,p), xg(l,3,p)
        end do
      end do
    end do     
    
    tmp1 =   xg(l,1,p) 
    tmp2 =   xg(l,2,p) 
    tmp3 =   xg(l,3,p) 
            
    ! Fill up with dummy data 
    do i = l+1, MaxNNODZ
      xg(i,1,p) = tmp1
      xg(i,2,p) = tmp2
      xg(i,3,p) = tmp3
    enddo

    do i = 1, NNODZ(p)
      read(mf,*) IPER(i,p)
    end do

    close(mf)
  end do



  write(*,*) '!== end of reading all meshes ==!'
   


  !call set_conn(ConNP)
  call gen_conn(ConNP,eps,xg,NP,NSD)
  call gen_lwork(eps,NP, NNODZ, xg, IPER, ConNP, MCP, NCP, OCP)

  deallocate(xg, IPER, ConNP, NNODZ,MCP, NCP, OCP)

contains

  include "inc_gen_conn.f"
  include "inc_gen_lwork.f"
  include "inc_cname.f"

end program main
