c     This routine links each element in the h-refined mesh with its
c        parent element in the unrefined mesh.
c
c     EL_CON(i) = j implies that element j of the OLD mesh is the parent
c           of element i in the new mesh.
c
c     modified for user specified knot insertion June 25, 2004
c
c     
c       Feb. 10, 2004
c       J. Austin Cottrell
c       CAM Graduate Student
c       Institute for Computational Engineering Science
c       The University of Texas at Austin

      subroutine gen_EL_CON_3D(EL_CON,new_NEL,new_MCP,new_NCP,
     &     new_OCP,uflag,vflag,wflag)

      use aAdjKeep
      include "common.h"

      integer e,new_NEL,i,j,k,icount,jcount,new_MCP,new_NCP
      integer  EL_CON(new_NEL), new_OCP,kcount,uflag,vflag,wflag
      

      
c     uflag = 1 denotes that we are refining in the u direction. That is
c        we are inserting knots into U_KNOT. Likewise for vflag and wflag      
      
      EL_CON = 0
      e = 0
      kcount = 0
      do k = 1,OCP-R
        jcount = 0
        do j = 1,NCP-Q
          icount = 0
          e = (MCP-P)*(NCP-Q)*(k-1)+(MCP-P)*(j-1)
          do i = 1,MCP-P
            e = e+1
            EL_CON(i+icount+(j-1+jcount)*(new_MCP-P)+
     &        (k-1+kcount)*(new_MCP-P)*(new_NCP-Q)) = e
            
                  ! check to see if the element has been split in u
            if (((icount+1).le.(new_MCP-MCP)).and.
     &        ((U_KNOT(i+P).le.x_knot_u(icount+1)).and.
     &        (x_knot_u(icount+1).lt.U_KNOT(i+P+1))).and.
     &        (uflag.eq.1)) then
 1000         continue
              icount = icount + 1
              EL_CON(i+icount+(j-1+jcount)*(new_MCP-P)+
     &          (k-1+kcount)*(new_MCP-P)*(new_NCP-Q)) = e
              if ((icount+1).le.(new_MCP-MCP).and.((U_KNOT(i+P).le.
     &          x_knot_u(icount+1)).and.
     &          (x_knot_u(icount+1).lt.U_KNOT(i+P+1)))) then
                goto 1000
              endif
            endif
          enddo

            ! check to see if the element has been split in v
          if (((jcount+1).le.(new_NCP-NCP)).and.
     &      ((V_KNOT(j+Q).le.(x_knot_v(jcount+1))).and.
     &      (x_knot_v(jcount+1)).lt.V_KNOT(j+1+Q)).and.
     &      (vflag.eq.1)) then
 1100       continue
            icount = 0
            jcount = jcount+1
            e = (MCP-P)*(NCP-Q)*(k-1)+
     &        (j-1)*(MCP-P)
            do i = 1,MCP-P
              e = e+1
              EL_CON(i+icount+(j-1+jcount)*(new_MCP-P)+
     &          (k-1+kcount)*(new_MCP-P)*(new_NCP-Q)) = e
                
                   ! check to see if the element has been split in u
              if (((icount+1).le.(new_MCP-MCP)).and.
     &          ((U_KNOT(i+P).le.x_knot_u(icount+1)).and.
     &          (x_knot_u(icount+1).lt.U_KNOT(i+P+1))).and.
     &          (uflag.eq.1)) then
 1200           continue
                icount = icount + 1
                EL_CON(i+icount+(j-1+jcount)*(new_MCP-P)+
     &            (k-1+kcount)*(new_MCP-P)*(new_NCP-Q)) = e
                if ((icount+1).le.(new_MCP-MCP).and.((U_KNOT(i+P).le.
     &            x_knot_u(icount+1)).and.
     &            (x_knot_u(icount+1).lt.U_KNOT(i+P+1)))) then
                  goto 1200
                endif
              endif
            enddo
            if (((jcount+1).le.(new_NCP-NCP)).and.((V_KNOT(j+Q).le.
     &        (x_knot_v(jcount+1))).and.
     &        (x_knot_v(jcount+1)).lt.V_KNOT(j+1+Q)))  then
              goto 1100
            endif
          endif
        enddo

                       ! check to see if the element has been split in w
        if (((kcount+1).le.(new_OCP-OCP)).and.
     &    ((W_KNOT(k+R).le.x_knot_w(kcount+1)).and.
     &    (x_knot_w(kcount+1).lt.W_KNOT(k+1+R))).and.
     &    (wflag.eq.1)) then
 1300     continue
          kcount = kcount+1
          jcount = 0 
          do j = 1,NCP-Q
            icount = 0
            e = (MCP-P)*(NCP-Q)*(k-1)+(MCP-P)*(j-1)
            do i = 1,MCP-P
              e = e+1
              EL_CON(i+icount+(j-1+jcount)*(new_MCP-P)+
     &          (k-1+kcount)*(new_MCP-P)*(new_NCP-Q)) = e
                
                     ! check to see if the element has been split in u
              if (((icount+1).le.(new_MCP-MCP)).and.
     &          ((U_KNOT(i+P).le.x_knot_u(icount+1)).and.
     &          (x_knot_u(icount+1).lt.U_KNOT(i+P+1))).and.
     &          (uflag.eq.1)) then
 2000           continue
                icount = icount + 1
                EL_CON(i+icount+(j-1+jcount)*(new_MCP-P)+
     &            (k-1+kcount)*(new_MCP-P)*(new_NCP-Q)) = e
                if ((icount+1).le.(new_MCP-MCP).and.((U_KNOT(i+P).le.
     &            x_knot_u(icount+1)).and.
     &            (x_knot_u(icount+1).lt.U_KNOT(i+P+1)))) then
                  goto 2000
                endif
              endif
            enddo
              
                  ! check to see if the element has been split in v
            if (((jcount+1).le.(new_NCP-NCP)).and.
     &        ((V_KNOT(j+Q).le.(x_knot_v(jcount+1))).and.
     &        (x_knot_v(jcount+1)).lt.V_KNOT(j+1+Q)).and.
     &        (vflag.eq.1)) then
 2100         continue
              icount = 0
              jcount = jcount+1
              e = (MCP-P)*(NCP-Q)*(k-1)+
     &          (j-1)*(MCP-P)
              do i = 1,MCP-P
                e = e+1
                EL_CON(i+icount+(j-1+jcount)*(new_MCP-P)+
     &            (k-1+kcount)*(new_MCP-P)*(new_NCP-Q)) = e
                
                      ! check to see if the element has been split in u
                if (((icount+1).le.(new_MCP-MCP)).and.
     &            ((U_KNOT(i+P).le.x_knot_u(icount+1)).and.
     &            (x_knot_u(icount+1).lt.U_KNOT(i+P+1))).and.
     &            (uflag.eq.1)) then
 2200             continue
                  icount = icount + 1
                  EL_CON(i+icount+(j-1+jcount)*(new_MCP-P)+
     &              (k-1+kcount)*(new_MCP-P)*(new_NCP-Q)) = e
                  if ((icount+1).le.(new_MCP-MCP).and.((U_KNOT(i+P).le.
     &              x_knot_u(icount+1)).and.
     &              (x_knot_u(icount+1).lt.U_KNOT(i+P+1)))) then
                    goto 2200
                  endif
                endif
              enddo
              if (((jcount+1).le.(new_NCP-NCP)).and.((V_KNOT(j+Q).le.
     &          (x_knot_v(jcount+1))).and.
     &          (x_knot_v(jcount+1)).lt.V_KNOT(j+1+Q)))  then
                goto 2100
              endif
            endif
          enddo
          if (((kcount+1).le.(new_OCP-OCP)).and.((W_KNOT(k+R).le.
     &      (x_knot_w(kcount+1))).and.
     &        (x_knot_w(kcount+1)).lt.W_KNOT(k+1+R)))  then
            goto 1300
          endif   
        endif
      enddo

         
      return
      end
