

      use aAdjKeep
      include "common.h"

!
!     User specified knot insertion for structures code      
!     adapted from h_insert_AD.f July 05, 2004
!         
!       J. Austin Cottrell
!       CAM Graduate Student
!       Institute for Computational Engineering Science
!       The University of Texas at Austin  


c     -------------------------------------------------------
      integer i, icount, new_MCP, new_NCP, new_OCP, new_NNODZ, new_NEL,
     &     new_NFACE, jcount, j, kcount,l,k,uflag,vflag,wflag
      
      integer, allocatable :: new_IEN(:,:), new_INN(:,:), new_IBC(:,:),
     &     new_FACE_IEN(:,:), new_FACE_OR(:), new_IBC_FACE(:,:),
     &     FACE_CON(:), EL_CON(:), new_IPER(:)
      real*8, allocatable :: new_U_KNOT(:),new_V_KNOT(:),new_W_KNOT(:),
     &     new_B_NET(:,:,:,:),
     &     new_BW(:,:,:,:), Pw_1(:,:), Pw_2(:,:),Pw_3(:,:), Pw_4(:,:),
     &     temp(:), temp2(:,:,:,:),new_LD_FACE(:,:), new_LD_EL(:,:),
     &     new_EY(:), new_NU(:),new_DIR_BC(:,:),
     &     temp3(:,:,:,:)
    

c     -------------------------------------------------------



c...  Get initial mesh data
      call input_3D

      
      allocate(INN(NNODZ,3))
      NSHL = (P+1)*(Q+1)*(R+1)
      allocate(IEN(NEL,NSHL))
c...  moved from genIEN_INN_3D for to fix bugs
      
c...  Form projective control net
      BW = B_NET
      do i = 1,NSD
         BW(:,:,:,i) = BW(:,:,:,i)*B_NET(:,:,:,NSD+1)
      enddo
      
      
c...  Figure out number of knots to be inserted in each direction
c     and new knot values

      call input_knots_3D(uflag,vflag,wflag,icount,jcount,kcount)
      
      new_MCP = MCP+icount      ! new number of control points in u
      allocate(new_U_KNOT(new_MCP+P+1))
      
      new_NCP = NCP+jcount      ! new number of control points in v
      allocate(new_V_KNOT(new_NCP+Q+1))
      
      new_OCP = OCP+kcount      ! new number of control points in w
      allocate(new_W_KNOT(new_OCP+R+1))
      
      
      allocate(new_B_NET(new_MCP,new_NCP,new_OCP,NSD+1))

      
      
c...  Set number of global nodes
      new_NNODZ = new_MCP*new_NCP*new_OCP
c...  Set number of elements
      new_NEL = (new_MCP-P)*(new_NCP-Q)*(new_OCP-R)
      new_NFACE = 2*(new_MCP-P)*(new_NCP-Q)*(1-CLOSED_W_flag) +
     &     2*(new_NCP-Q)*(new_OCP-R)*(1-CLOSED_U_flag) +
     &     2*(new_MCP-P)*(new_OCP-R)*(1-CLOSED_V_flag)
      allocate(new_FACE_IEN(new_NFACE,(P+1)*(Q+1)*(R+1)))
      allocate(new_FACE_OR(new_NFACE))
      allocate(FACE_CON(new_NFACE))
      allocate(EL_CON(new_NEL))
      allocate(new_LD_EL(new_NEL,NSD))
      allocate(new_EY(new_NEL))
      allocate(new_NU(new_NEL))
      allocate(new_INN(new_NNODZ,NSD))
      allocate(new_IEN(new_NEL,(P+1)*(Q+1)*(R+1)))
      allocate(new_IBC(new_NNODZ,NSD))
      allocate(new_IBC_FACE(new_NFACE,NSD))
      allocate(new_IPER(new_NNODZ))
      allocate(new_LD_FACE(new_NFACE,NSD))
      allocate(new_DIR_BC(new_NNODZ,NSD))
      
c     
c...  Form new Control Net in Projective coordinates
c     
      allocate(temp2(new_MCP,NCP,OCP,NSD+1))
      allocate(temp3(new_MCP,new_NCP,OCP,NSD+1))
      
      allocate(new_BW(new_MCP,new_NCP,new_OCP,NSD+1))
      temp2 = 0d+0
      new_BW = 0d+0

c...  refine first in u
      if (uflag.eq.1) then
        call RefineKnotVectSolid_3D(MCP,P,U_KNOT,NCP,Q,V_KNOT,OCP,R,
     &    W_KNOT,BW,x_knot_u,icount,0,0,1,temp2,new_U_KNOT,
     &    V_KNOT,W_KNOT)
      else
        temp2 = BW
        new_U_KNOT = U_KNOT
      endif
c...  now refine in v
      if (vflag.eq.1) then
        call RefineKnotVectSolid_3D(new_MCP,P,new_U_KNOT,NCP,Q,V_KNOT,
     &    OCP,R,W_KNOT,temp2,x_knot_v,0,jcount,0,2,temp3,new_U_KNOT,
     &    new_V_KNOT, W_KNOT)
      else
        temp3 = temp2
        new_V_KNOT = V_KNOT
      endif
c...    now refine in w
      if (wflag.eq.1) then       
        call RefineKnotVectSolid_3D(new_MCP,P,new_U_KNOT,new_NCP,Q,
     &    new_V_KNOT,OCP,R,W_KNOT,temp3,x_knot_w,0,0,kcount,3,
     &    new_BW,new_U_KNOT,new_V_KNOT,new_W_KNOT)
      else
        new_BW = temp3
        new_W_KNOT = W_KNOT
      endif
        deallocate(temp2)
        deallocate(temp3)

      
c...  Convert projective coordinates into physical coordinates
      new_B_NET = new_BW
      do i = 1,NSD
         new_B_NET(:,:,:,i) = new_B_NET(:,:,:,i)/new_B_NET(:,:,:,NSD+1)
      enddo
           
c      ---------------------------------------------------------------

c
c     At this point we have a new control net and new knot vectors in each
c       direction. We must now create connectivities such that we may
c       transfer load and boundary conditions, periodicities, etc. from the
c       original mesh to the new one.

c     We need the connectivities from our original mesh in order to 
c      assign old data to the correct locations in the new mesh.
      call genIEN_INN_3D
      call gen_FACE_IEN_3D


c     Generate FACE_CON matrix connecting old faces to new ones. This is
c      structured so that FACE_CON(new_face_num) equals
c      the old face number.      
      call gen_FACE_CON_3D(FACE_CON,new_NFACE,new_MCP,new_NCP,new_OCP,
     &  uflag,vflag,wflag)



c     Generate EL_CON matrix connecting old elements to new ones. 
c       EL_CON(new_el_num) equals the old element number 
      call gen_EL_CON_3D(EL_CON, new_NEL, new_MCP, new_NCP, new_OCP,
     &  uflag,vflag,wflag)


c...  Create new IEN and INN arrays
      call gen_new_IEN_INN_3D(new_MCP,new_NCP,new_OCP,new_IEN,
     &  new_INN,new_NEL,new_NNODZ)
      
      
      
      
      
c...  Also create new face connectivity arrays
      call gen_new_FACE_IEN_3D(new_NFACE, new_NNODZ,
     &  new_FACE_IEN, new_FACE_OR,new_NEL,
     &  new_MCP, new_NCP,new_OCP,new_IEN)
      
c
c...  Create new IBC array assign face data
c
      call gen_FACE_DATA_3D(new_NNODZ,new_NFACE,FACE_CON,new_FACE_OR, 
     &     new_FACE_IEN, new_IPER,  new_IBC, new_IBC_FACE, new_LD_FACE,
     &     new_MCP,new_NCP,new_OCP, new_INN, new_B_NET,new_U_KNOT,
     &     new_V_KNOT, new_W_KNOT,new_DIR_BC) 
 
c
c...  Assign element data
c
      call gen_EL_DATA_3D(new_NEL, EL_CON, new_LD_EL, new_EY, new_NU)


c
c...  Now we have assembled all of the new structures, we may replace
c       the old ones and write our results to refined_mesh.dat
c

c...  assign common variables to new values
      NNODZ = new_NNODZ
      MCP = new_MCP
      NCP = new_NCP
      OCP = new_OCP
      NFACE = new_NFACE
      NEL = new_NEL

c...  Write to file refined_mesh.dat
      call output_3D_2(new_U_KNOT,new_V_KNOT,new_W_KNOT,new_B_NET,
     &     new_IPER,new_IBC,new_DIR_BC,new_IBC_FACE,new_LD_FACE,
     &     new_LD_EL,new_EY,new_NU)

      stop

      end
