c     This routine links each face in an h-refined mesh with its parent face
c        in the unrefined mesh.
c
c     FACE_CON(i) = j indicates that element j in the OLD mesh is the parent
c        of element i in the new mesh.
c      
c       Feb. 10, 2004
c       J. Austin Cottrell
c       CAM Graduate Student
c       Institute for Computational Engineering Science
c       The University of Texas at Austin     

      subroutine gen_FACE_CON_3D(FACE_CON,new_NFACE,new_MCP,new_NCP,
     &  new_OCP,uflag,vflag,wflag)

      use aAdjKeep
      include "common.h"
      
      integer e, icount,i,j,jcount,new_MCP,new_NCP,new_NFACE
      integer  FACE_CON(new_NFACE), new_OCP,kcount,k,uflag,vflag,wflag
      
      

c     Each face orientation must be treated separately, though the basic
c       behaviour of each loop is the same. Note that uflag = 1 indicates
c       that we have inserted knots into U_KNOT. Likewise with vflag and
c       wflag.      

      e = 0    
      
      if (CLOSED_W_flag.ne.1) then
c...    Loop through orientation 1
        
        jcount = 0
        do j= 1,NCP-Q
          icount = 0
          do i = 1,MCP-P
            e = e+1
            FACE_CON(i+icount + (new_MCP-P)*(j-1+jcount)) = e
            if ((U_KNOT(i+P).ne.U_KNOT(i+P+1)).and.(uflag.eq.1)) then
              icount = icount+1
              FACE_CON(i+icount + (new_MCP-P)*(j-1+jcount)) = e
            endif
          enddo
          if ((V_KNOT(j+Q).ne.V_KNOT(j+1+Q)).and.(vflag.eq.1)) then
            jcount = jcount+1
            icount = 0
            e = (j-1)*(MCP-P)
            do i = 1,MCP-P
              e = e+1
              FACE_CON(i+icount + (new_MCP-P)*(j-1+jcount)) = e
              if ((U_KNOT(i+P).ne.U_KNOT(i+1+P)).and.(uflag.eq.1)) then
                icount = icount+1
                FACE_CON(i+icount + (new_MCP-P)*(j-1+jcount)) = e
              endif
            enddo
          endif
        enddo
      endif
      
      
      
      
      if (CLOSED_V_flag.ne.1) then
c...    Loop Through orientation 2
        
        kcount = 0
        do k= 1,OCP-R
          icount = 0
          do i = 1,MCP-P
            e = e+1
            FACE_CON(i+icount +
     &        (1-CLOSED_W_flag)*(new_MCP-P)*(new_NCP-Q)+
     &        (k-1+kcount)*(new_MCP-P)) = e
            if ((U_KNOT(i+P).ne.U_KNOT(i+P+1)).and.(uflag.eq.1)) then
              icount = icount+1
              FACE_CON(i+icount +
     &          (1-CLOSED_W_flag)*(new_MCP-P)*(new_NCP-Q)+
     &          (k-1+kcount)*(new_MCP-P)) = e
            endif
          enddo
          if ((W_KNOT(k+R).ne.W_KNOT(k+1+R)).and.(wflag.eq.1)) then
            kcount = kcount+1
            icount = 0
            e = (k-1)*(MCP-P)+(1-CLOSED_W_flag)*(MCP-P)*(NCP-Q)
            do i = 1,MCP-P
              e = e+1
              FACE_CON(i+icount +
     &          (1-CLOSED_W_flag)*(new_MCP-P)*(new_NCP-Q)+
     &          (k-1+kcount)*(new_MCP-p)) = e
              if ((U_KNOT(i+P).ne.U_KNOT(i+1+P)).and.(uflag.eq.1)) then
                icount = icount+1
                FACE_CON(i+icount +
     &            (1-CLOSED_W_flag)*(new_MCP-P)*(new_NCP-Q)+
     &            (k-1+kcount)*(new_MCP-P)) = e
              endif
            enddo
          endif
        enddo
      endif
      
      
      
      
      
      if (CLOSED_U_flag.ne.1) then
c...    Loop Through orientation 3
        
        kcount = 0
        do k= 1,OCP-R
          jcount = 0            
          do j = 1,NCP-Q
            e = e+1
            FACE_CON(j+jcount +
     &        (1-CLOSED_W_flag)*(new_MCP-P)*(new_NCP-Q)+
     &        (1-CLOSED_V_flag)*(new_MCP-p)*(new_OCP-R)+
     &        (k-1+kcount)*(new_NCP-Q)) = e
            if ((V_KNOT(j+Q).ne.V_KNOT(j+Q+1)).and.(vflag.eq.1)) then
              jcount = jcount+1
              FACE_CON(j+jcount +
     &          (1-CLOSED_W_flag)*(new_MCP-P)*(new_NCP-Q)+
     &          (1-CLOSED_V_flag)*(new_MCP-p)*(new_OCP-R)+
     &          (k-1+kcount)*(new_NCP-Q)) = e
            endif
          enddo
          if ((W_KNOT(k+R).ne.W_KNOT(k+1+R)).and.(wflag.eq.1)) then
            kcount = kcount+1
            jcount = 0
            e = (k-1)*(NCP-Q)+(1-CLOSED_W_flag)*(MCP-P)*(NCP-Q)+
     &        (1-CLOSED_V_flag)*(MCP-P)*(OCP-R)
            do j = 1,NCP-Q
              e = e+1
              FACE_CON(j+jcount +
     &          (1-CLOSED_W_flag)*(new_MCP-P)*(new_NCP-Q)+
     &          (1-CLOSED_V_flag)*(new_MCP-p)*(new_OCP-R)+
     &          (k-1+kcount)*(new_NCP-Q)) = e
              if ((V_KNOT(j+Q).ne.V_KNOT(j+1+Q)).and.(vflag.eq.1)) then
                jcount = jcount+1
                FACE_CON(j+jcount +
     &            (1-CLOSED_W_flag)*(new_MCP-P)*(new_NCP-Q)+
     &            (1-CLOSED_V_flag)*(new_MCP-p)*(new_OCP-R)+
     &            (k-1+kcount)*(new_NCP-Q)) = e
              endif
            enddo
          endif
        enddo
      endif
      
      
      
      
      if (CLOSED_V_flag.ne.1) then
c...    Loop Through orientation 4
        
        kcount = 0
        do k= 1,OCP-R
          icount = 0
          do i = 1,MCP-P
            e = e+1
            FACE_CON(i+icount +
     &        (1-CLOSED_W_flag)*(new_MCP-P)*(new_NCP-Q)+
     &        (1-CLOSED_V_flag)*(new_MCP-P)*(new_OCP-R)+
     &        (1-CLOSED_U_flag)*(new_NCP-Q)*(new_OCP-R)+
     &        (k-1+kcount)*(new_MCP-P)) = e
            if ((U_KNOT(MCP+1-i).ne.U_KNOT(MCP+2-i)).and.(uflag.eq.1))
     &        then
              icount = icount+1
              FACE_CON(i+icount +
     &          (1-CLOSED_W_flag)*(new_MCP-P)*(new_NCP-Q)+
     &          (1-CLOSED_V_flag)*(new_MCP-P)*(new_OCP-R)+
     &          (1-CLOSED_U_flag)*(new_NCP-Q)*(new_OCP-R)+
     &          (k-1+kcount)*(new_MCP-P)) = e
            endif
          enddo
          if ((W_KNOT(k+R).ne.W_KNOT(k+1+R)).and.(wflag.eq.1)) then
            kcount = kcount+1
            icount = 0
            e = (k-1)*(MCP-P)+(1-CLOSED_W_flag)*(MCP-P)*(NCP-Q)+
     &        (1-CLOSED_V_flag)*(MCP-P)*(OCP-R)+
     &        (1-CLOSED_U_flag)*(NCP-Q)*(OCP-R)
            do i = 1,MCP-P
              e = e+1
              FACE_CON(i+icount +
     &          (1-CLOSED_W_flag)*(new_MCP-P)*(new_NCP-Q)+
     &          (1-CLOSED_V_flag)*(new_MCP-P)*(new_OCP-R)+
     &          (1-CLOSED_U_flag)*(new_NCP-Q)*(new_OCP-R)+
     &          (k-1+kcount)*(new_MCP-P)) = e
              if ((U_KNOT(MCP+1-i).ne.U_KNOT(MCP+2-i)).and.
     &          (uflag.eq.1)) then
                icount = icount+1
                FACE_CON(i+icount +
     &            (1-CLOSED_W_flag)*(new_MCP-P)*(new_NCP-Q)+
     &            (1-CLOSED_V_flag)*(new_MCP-P)*(new_OCP-R)+
     &            (1-CLOSED_U_flag)*(new_NCP-Q)*(new_OCP-R)+
     &            (k-1+kcount)*(new_MCP-P)) = e
              endif
            enddo
          endif
        enddo
      endif
      
      

      
      if (CLOSED_U_flag.ne.1) then
c...    Loop Through orientation 5
        
        kcount = 0
        do k= 1,OCP-R
          jcount = 0            
          do j = 1,NCP-Q
            e = e+1
            FACE_CON(j+jcount +
     &        (1-CLOSED_W_flag)*(new_MCP-P)*(new_NCP-Q)+
     &        2*(1-CLOSED_V_flag)*(new_MCP-P)*(new_OCP-R)+
     &        (1-CLOSED_U_flag)*(new_NCP-Q)*(new_OCP-R)+
     &        (k-1+kcount)*(new_NCP-Q)) = e
            if ((V_KNOT(NCP+1-j).ne.V_KNOT(NCP+2-j)).and.(vflag.eq.1))
     &        then
              jcount = jcount+1
              FACE_CON(j+jcount +
     &          (1-CLOSED_W_flag)*(new_MCP-P)*(new_NCP-Q)+
     &          2*(1-CLOSED_V_flag)*(new_MCP-p)*(new_OCP-R)+
     &          (1-CLOSED_U_flag)*(new_NCP-Q)*(new_OCP-R)+
     &          (k-1+kcount)*(new_NCP-Q)) = e
            endif
          enddo
          if ((W_KNOT(k+R).ne.W_KNOT(k+1+R)).and.(wflag.eq.1)) then
            kcount = kcount+1
            jcount = 0
            e = (k-1)*(NCP-Q)+(1-CLOSED_W_flag)*(MCP-P)*(NCP-Q)+
     &        2*(1-CLOSED_V_flag)*(MCP-P)*(OCP-R)+
     &        (1-CLOSED_U_flag)*(NCP-Q)*(OCP-R)
            do j = 1,NCP-Q
              e = e+1
              FACE_CON(j+jcount +
     &          (1-CLOSED_W_flag)*(new_MCP-P)*(new_NCP-Q)+
     &          2*(1-CLOSED_V_flag)*(new_MCP-p)*(new_OCP-R)+
     &          (1-CLOSED_U_flag)*(new_NCP-Q)*(new_OCP-R)+
     &          (k-1+kcount)*(new_NCP-Q)) = e
              if ((V_KNOT(NCP+1-j).ne.V_KNOT(NCP+2-j)).and.
     &          (vflag.eq.1)) then
                jcount = jcount+1
                FACE_CON(j+jcount +
     &            (1-CLOSED_W_flag)*(new_MCP-P)*(new_NCP-Q)+
     &            2*(1-CLOSED_V_flag)*(new_MCP-p)*(new_OCP-R)+
     &            (1-CLOSED_U_flag)*(new_NCP-Q)*(new_OCP-R)+
     &            (k-1+kcount)*(new_NCP-Q)) = e
              endif
            enddo
          endif
        enddo
      endif


      
      if (CLOSED_W_flag.ne.1) then
c...    Loop through orientation 6
        
        jcount = 0
        do j= 1,NCP-Q
          icount = 0
          do i = 1,MCP-P
            e = e+1
            FACE_CON(i+icount +
     &        (1-CLOSED_W_flag)*(new_MCP-P)*(new_NCP-Q)+
     &        2*(1-CLOSED_U_flag)*(new_NCP-Q)*(new_OCP-R) +
     &        2*(1-CLOSED_V_flag)*(new_MCP-P)*(new_OCP-R) +
     &        (new_MCP-P)*(j-1+jcount)) = e
            if ((U_KNOT(i+P).ne.U_KNOT(i+P+1)).and.(uflag.eq.1)) then
              icount = icount+1
              FACE_CON(i+icount +
     &          (1-CLOSED_W_flag)*(new_MCP-P)*(new_NCP-Q)+
     &          2*(1-CLOSED_U_flag)*(new_NCP-Q)*(new_OCP-R) +
     &          2*(1-CLOSED_V_flag)*(new_MCP-P)*(new_OCP-R) +
     &          (new_MCP-P)*(j-1+jcount)) = e
            endif
          enddo
          if ((V_KNOT(j+Q).ne.V_KNOT(j+1+Q)).and.(vflag.eq.1)) then
            jcount = jcount+1
            icount = 0
            e = (1-CLOSED_W_flag)*(MCP-P)*(NCP-Q)+
     &        2*(1-CLOSED_U_flag)*(NCP-Q)*(OCP-R) +
     &        2*(1-CLOSED_V_flag)*(MCP-P)*(OCP-R) +
     &        (MCP-P)*(j-1)
            do i = 1,MCP-P
              e = e+1
              FACE_CON(i+icount +
     &          (1-CLOSED_W_flag)*(new_MCP-P)*(new_NCP-Q)+
     &          2*(1-CLOSED_U_flag)*(new_NCP-Q)*(new_OCP-R) +
     &          2*(1-CLOSED_V_flag)*(new_MCP-P)*(new_OCP-R) +
     &          (new_MCP-P)*(j-1+jcount)) = e
              if ((U_KNOT(i+P).ne.U_KNOT(i+P+1)).and.(uflag.eq.1)) then
                icount = icount+1
                FACE_CON(i+icount +
     &            (1-CLOSED_W_flag)*(new_MCP-P)*(new_NCP-Q)+
     &            2*(1-CLOSED_U_flag)*(new_NCP-Q)*(new_OCP-R) +
     &            2*(1-CLOSED_V_flag)*(new_MCP-P)*(new_OCP-R) +
     &            (new_MCP-P)*(j-1+jcount)) = e
              endif
            enddo
          endif
        enddo
      endif
      
      

      
      
      return
      end
