c
c       This subroutine generates the new_FACE_IEN matrix, which relates face 
c       numbers and local node numbers to the appropriate global node numbers. 
c       It will also generate new_FACE_OR which will contain the orientations
c       of a given face. This is the same as gen_FACE_IEN_3D, except for it
c       is for the refined mesh.	
c
c
c       This is for the 3D code.
c
c       gen_FACE_IEN_3D written January 15, 2004
c
c       modified for h-refinement
c       February 10, 2004
c
c       J. Austin Cottrell
c       CES Graduate Student
c       Institute for Computational Engineering Science
c       The University of Texas at Austin

 
	
	subroutine gen_new_FACE_IEN_3D(new_NFACE, new_NNODZ,
     &       new_FACE_IEN, new_FACE_OR,new_NEL,
     &       new_MCP, new_NCP,new_OCP,new_IEN)

	use aAdjKeep		! this f90 module contains all allocatables
c       
	include "common.h"	! common file defines all variables
c       
c...    Local variables
	integer :: i,j,face,e,new_NFACE,new_NNODZ,new_NEL,
     &        new_FACE_IEN(new_NFACE,(P+1)*(Q+1)*(R+1)),
     &        new_FACE_OR(new_NFACE),new_MCP,new_NCP,new_OCP,
     &        new_IEN(new_NEL,(P+1)*(Q+1)*(R+1))



c       
c...    Initialize matrices and variables
c       
	new_FACE_IEN = 0
	new_FACE_OR = 0
        face = 0    

	




c...    Orientation 1
	e = 0
	if (CLOSED_W_flag.ne.1) then
	   do j = 1,new_NCP
	      do i = 1,new_MCP
		 if ((i .ge. (P+1)).and.(j .ge. (Q+1))) then
		    face = face + 1     ! face number
		    new_FACE_OR(face) = 1   ! face orientation
		    e = e + 1           ! element number
		    new_FACE_IEN(face,:) = new_IEN(e,:)
		 endif
	      enddo
	   enddo
	endif



c...    Orientation 2
	if (CLOSED_V_flag.ne.1) then
	   do j = R+1,new_OCP
	      e = (new_MCP-P)*(new_NCP-Q)*(j-(R+1))
	      do i = P+1,new_MCP
		 e = e + 1	! element number
		 face = face+1
		 new_FACE_OR(face) = 2
		 new_FACE_IEN(face,:) = new_IEN(e,:)
	      enddo
	   enddo
	endif



c...    Orientation 3
	if (CLOSED_U_flag.ne.1) then
	   do j = R+1,new_OCP
	      e = (new_MCP-P)*(new_NCP-Q)*(j-(R+1))
	      do i = Q+1,new_NCP
		 face = face+1
		 new_FACE_OR(face) = 3
		 e = e + (new_MCP-P)
		 new_FACE_IEN(face,:) = new_IEN(e,:)
	      enddo
	   enddo
	endif



c...    Orientation 4
	if (CLOSED_V_flag.ne.1) then
	   do j = R+1,new_OCP
	      e = (new_MCP-P)*(new_NCP-Q)*(j-R)+1
	      do i = P+1,new_MCP
		 face = face+1
		 new_FACE_OR(face) = 4
		 e = e - 1 
		 new_FACE_IEN(face,:) = new_IEN(e,:)
	      enddo
	   enddo
	endif



c...    Orientation 5
	if (CLOSED_U_flag.ne.1) then
	   do j = R+1,new_OCP
	      e = (new_MCP-P)*(new_NCP-Q)*(j-R)+1
	      do i = Q+1,new_NCP
		 face = face+1
		 new_FACE_OR(face) = 5
		 e = e - (new_MCP-P)
		 new_FACE_IEN(face,:) = new_IEN(e,:)
	      enddo
	   enddo
	endif


c...    Orientation 6
	if (CLOSED_W_flag.ne.1) then
	   e = (new_MCP-P)*(new_NCP-Q)*(new_OCP-R-1)
	   do j = 1,new_NCP
	      do i = 1,new_MCP
		 if ((i .ge. (P+1)).and.(j .ge. (Q+1))) then
		    face = face + 1
		    new_FACE_OR(face) = 6
		    e = e + 1
		    new_FACE_IEN(face,:) = new_IEN(e,:)
		 endif
	      enddo
	   enddo
	endif

	return
	end
