      subroutine degree_elevate_solid(ml,pl,u_knotl,nl,ql,v_knotl,
     &  ol,rl,w_knotl,Pw,t,dir_u,dir_v,dir_w,dist_knots,Qw,mh,Uh,ph,nh,
     &  Vh,qh,oh,Wh,rh)

      
c
c purpose: This function degree elevates by t (in the specified direction)
c             a given NURBS solid and
c             refines projective control points Pw such that the surface
c             does not change shape.
c
c
c
c May 1, 2004
c J. Austin Cottrell
c CES Graduate Student
c Texas Institute for Computational Engineering Science
c University of Texas at Austin

      use aAdjKeep
      
      include "common.h"

      integer ml,pl,nl,ql,ol,rl,t,dist_knots,mh,ph,nh,qh,oh,rh,
     &  dir_u,dir_v,dir_w,i,j,k

      real*8 u_knotl(ml+pl+1),v_knotl(nl+ql+1),w_knotl(ol+rl+1),
     &  Uh(ml+pl+1+dir_u*(dist_knots*t)),Pw(ml,nl,ol,NSD+1),
     &  Vh(nl+ql+1+dir_v*(dist_knots*t)),
     &  Wh(ol+rl+1+dir_w*(dist_knots*t)),
     &  Qw(ml+dir_u*(dist_knots-1)*t,nl+dir_v*(dist_knots-1)*t,
     &     ol+dir_w*(dist_knots-1)*t,NSD+1)

      real*8, allocatable :: Pwtemp(:,:,:),Qwtemp(:,:,:)



      if (dir_u.eq.1) then
        Qw = 0.0;
        allocate(Pwtemp(ml,ol,NSD+1))
        allocate(Qwtemp(ml+(dist_knots-1)*t,ol,NSD+1))
        
        do i = 1,nl
          do j = 1,ml
            do k = 1,ol
              Pwtemp(j,k,:) = Pw(j,i,k,:)
            enddo
          enddo

          call degree_elevate_surface(ml,pl,u_knotl,ol,rl,w_knotl,
     &      Pwtemp,t,1,dist_knots,Qwtemp,mh,Uh,ph,
     &      oh,Wh,rh)
          
          Qw(:,i,:,:) = Qwtemp
        enddo
        qh = ql
        Vh = v_knotl
        nh = nl
        
        
        
        
        
      elseif (dir_v.eq.1) then
        
        Qw = 0.0
        allocate(Pwtemp(nl,ol,NSD+1))
        allocate(Qwtemp(nl+(dist_knots-1)*t,ol,NSD+1))
        
        do i = 1,ml
          do j = 1,nl
            do k = 1,ol
              Pwtemp(j,k,:) = Pw(i,j,k,:)
            enddo
          enddo
          
          call degree_elevate_surface(nl,ql,v_knotl,ol,rl,w_knotl,
     &      Pwtemp,t,1,dist_knots,Qwtemp,nh,Vh,qh,
     &      oh,Wh,rh)
          
          Qw(i,:,:,:) = Qwtemp
        enddo
        ph = pl
        Uh = u_knotl
        mh = ml


        
      elseif (dir_w.eq.1) then
        Qw = 0.0;
        allocate(Pwtemp(nl,ol,NSD+1))
        allocate(Qwtemp(nl,ol+(dist_knots-1)*t,NSD+1))
        
        do i = 1,ml
          do j = 1,nl
            do k = 1,ol
              Pwtemp(j,k,:) = Pw(i,j,k,:)
            enddo
          enddo
          
          call degree_elevate_surface(nl,ql,v_knotl,ol,rl,w_knotl,
     &      Pwtemp,t,0,dist_knots,Qwtemp,nh,Vh,qh,
     &      oh,Wh,rh)
          
          Qw(i,:,:,:) = Qwtemp
        enddo
        ph = pl
        Uh = u_knotl
        mh = ml
      endif

      return
      end
      
