
      use aAdjKeep
      include "common.h"
      


!
!       Driver for splitting one patch into many for
!       parallel processing at desired knot values
!      
!       June 16-Sept. 9, 2004 (got sidetracked for a few months)
!
!
!       J. Austin Cottrell
!       CAM Graduate Student
!       Institute for Computational Engineering Science
!       The University of Texas at Austin  




      integer dir,uflag,vflag,wflag,multiplicity,num_reps,rift_span,
     &  i,j,k,new_NEL_1,new_NEL_2,new_NFACE_1,new_NFACE_2,
     &  newMCP1,newMCP2,newNCP1,newNCP2,newOCP1,newOCP2,
     &  CLOSED_U_flag_1,CLOSED_V_flag_1,CLOSED_W_flag_1,
     &  CLOSED_U_flag_2,CLOSED_V_flag_2,CLOSED_W_flag_2
      integer, allocatable :: EL_CON_1(:), EL_CON_2(:), FACE_CON_1(:),
     &  FACE_CON_2(:),new_FACE_IEN_1(:,:),new_FACE_IEN_2(:,:),
     &  new_IEN_1(:,:),new_IEN_2(:,:),new_INN_1(:,:),new_INN_2(:,:),
     &  new_IBC_1(:,:),new_IBC_FACE_1(:,:),new_IBC_2(:,:),
     &  new_IBC_FACE_2(:,:),new_FACE_OR_1(:),new_FACE_OR_2(:),
     &  new_IPER_1(:),new_IPER_2(:)
      integer iknot, nknots, newu, newv, neww, meshf, pflag
      real*8 rift
      real*8, allocatable :: new_U_KNOT(:), new_V_KNOT(:), 
     &     new_W_KNOT(:),new_BW(:,:,:,:),new_B_NET(:,:,:,:),x_knot(:),
     &     UK1(:),VK1(:),WK1(:),B1(:,:,:,:),UK2(:),VK2(:),WK2(:),
     &     B2(:,:,:,:),new_LD_FACE_1(:,:),new_DIR_BC_1(:,:),
     &     new_LD_FACE_2(:,:),new_DIR_BC_2(:,:),new_LD_EL_1(:,:),
     &     new_EY_1(:),new_NU_1(:),new_LD_EL_2(:,:),new_EY_2(:),
     &     new_NU_2(:), x_knot_u(:), x_knot_v(:), x_knot_w(:)
      
  

c...  Get mesh data for original patch
      call input_3D
      NSHL = (P+1)*(Q+1)*(R+1)
      
c...  Form projective control net
      BW = B_NET
      do i = 1,NSD
        BW(:,:,:,i) = BW(:,:,:,i)*B_NET(:,:,:,NSD+1)
      enddo      

!--------------------------------------------------------------------
                                ! READ IN DECOMPOSITION INFO
      
      meshf = 14
      
c     
c...  Read in preliminary information
c     
      open (meshf, file='knot.dat', status='old')
      read (meshf,*)  uflag, vflag, wflag ! which directions to insert into
      read (meshf,*)  newu, newv, neww ! number of new knots in each direction
      read (meshf,*)  pflag
 

c     
c...  Read knot vectors
c     
      if (uflag.eq.1) then
        allocate(x_knot_u(newu))
        do i = 1,newu
          if (i < newu) then
            read (meshf,"(F13.9)", ADVANCE = "NO") x_knot_u(i) ! read x_knot_u
          else
            read (meshf,"(F13.9)", ADVANCE = "YES") x_knot_u(i) 
          endif
        enddo
      endif
      
      if (vflag.eq.1) then
        allocate(x_knot_v(newv))
        do i = 1,newv
          if (i < newv) then
            read (meshf,"(F13.9)", ADVANCE = "NO") x_knot_v(i) ! read x_knot_v
          else
            read (meshf,"(F13.9)", ADVANCE = "YES") x_knot_v(i) 
          endif
        enddo
      endif

      if (wflag.eq.1) then
        allocate(x_knot_w(neww))
        do i = 1,neww
          if (i < neww) then
            read (meshf,"(F13.9)", ADVANCE = "NO") x_knot_w(i) ! read x_knot_w
          else
            read (meshf,"(F13.9)", ADVANCE = "YES") x_knot_w(i) 
          endif
        enddo
      endif
      
!-------------------------------------------------------------------------
      
c...  Get orientation of splitting surface
c...  An number of knots to insert
      
      if (uflag.eq.1) then
        dir = 1
        nknots = newu
      elseif (vflag.eq.1) then
        dir = 2
        nknots = newv
      elseif (wflag.eq.1) then
        dir = 3
        nknots = neww
      endif
      
      do iknot = 1, nknots
        
        if (dir.eq.1) then
          rift = x_knot_u(iknot)
        elseif (dir.eq.2) then
          rift = x_knot_v(iknot)
        elseif (dir.eq.3) then
          rift = x_knot_w(iknot)
        endif
      
      
c...    We begin by just inserting rift into the original patch
c       until its multiplicity is equal to the polynomial degree
c       
c       Find the existing multiplicity of the knot to be inserted
        multiplicity = 0
        uflag = 0
        vflag = 0
        wflag = 0
        if (dir.eq.1) then
          uflag = 1
          do i = P+2,MCP
            if (U_KNOT(i).eq.rift) then
              multiplicity = multiplicity+1
            endif
          enddo
          num_reps = (P-multiplicity) ! number of times new knot is to
        elseif (dir.eq.2) then  !  to be inserted
          vflag = 1
          do i = Q+2,NCP
            if (V_KNOT(i).eq.rift) then
              multiplicity = multiplicity+1
            endif
          enddo
          num_reps = (Q-multiplicity)
        else
          wflag = 1
          do i = R+2,OCP
            if (W_KNOT(i).eq.rift) then
              multiplicity = multiplicity+1
            endif
          enddo
          num_reps = (R-multiplicity)
        endif
        
        
        allocate(x_knot(num_reps))
        x_knot = rift           ! new knot to be inserted
      
      
        allocate(new_U_KNOT(MCP+P+1+uflag*num_reps))
        allocate(new_V_KNOT(NCP+Q+1+vflag*num_reps))
        allocate(new_W_KNOT(OCP+R+1+wflag*num_reps))
        allocate(new_BW(MCP+uflag*num_reps,NCP+vflag*
     &    num_reps,OCP+wflag*num_reps,NSD+1))
        allocate(new_B_NET(MCP+uflag*num_reps,NCP+vflag*
     &    num_reps,OCP+wflag*num_reps,NSD+1))
        
        
        
        if (num_reps.ne.0) then
          
          call RefineKnotVectSolid_3D(MCP,P,U_KNOT,NCP,Q,V_KNOT,OCP,R,
     &    W_KNOT,BW,x_knot,uflag*num_reps,vflag*num_reps,wflag*num_reps,
     &    dir,new_BW,new_U_KNOT,new_V_KNOT,new_W_KNOT)
          
        else
          new_BW = BW
          new_U_KNOT = U_KNOT
          new_V_KNOT = V_KNOT
          new_W_KNOT = W_KNOT
        endif
        
        
        
c...    Form control net
        new_B_NET = new_BW
        do i = 1,NSD
          new_B_NET(:,:,:,i) = new_B_NET(:,:,:,i)/new_BW(:,:,:,NSD+1)
        enddo
        
        
c       
c...  At this point the mesh has been refined to the point that the
c       basis only preserves C^0 continuity across the splitting surface
c       It is ready to be split into separate patches.
c
c...  It is simplest to proceed based on orientation of 
c        the splitting surface
     
        CLOSED_U_flag_1 = 0
        CLOSED_V_flag_1 = 0
        CLOSED_W_flag_1 = 0
        CLOSED_U_flag_2 = 0
        CLOSED_V_flag_2 = 0
        CLOSED_W_flag_2 = 0
        
c...    Splitting in U
        
        if (dir.eq.1) then
          
          call find_span(P,MCP,rift,U_KNOT,rift_span)
          allocate(UK1(rift_span-multiplicity+P+1))
          allocate(UK2(MCP+2*(P+1) - rift_span))
          allocate(VK1(NCP+Q+1))
          allocate(VK2(NCP+Q+1))
          allocate(WK1(OCP+R+1))
          allocate(WK2(OCP+R+1))
          newMCP1 = rift_span-multiplicity
          newMCP2 = MCP-rift_span+P+1
          newNCP1 = NCP
          newNCP2 = NCP
          newOCP1 = OCP
          newOCP2 = OCP
          CLOSED_V_flag_1 = CLOSED_V_flag
          CLOSED_W_flag_1 = CLOSED_W_flag
          CLOSED_V_flag_2 = CLOSED_V_flag
          CLOSED_W_flag_2 = CLOSED_W_flag
          allocate(B1(newMCP1,NCP,OCP,NSD+1))
          allocate(B2(newMCP2,NCP,OCP,NSD+1))
          new_NEL_1 = (newMCP1-P)*(NCP-Q)*(OCP-R)
          new_NEL_2 = (newMCP2-P)*(NCP-Q)*(OCP-R)
          new_NFACE_1 = 2*(rift_span-multiplicity-P)*(NCP-Q)*
     &      (1-CLOSED_W_flag_1) +
     &      2*(NCP-Q)*(OCP-R)*(1-CLOSED_U_flag_1) +
     &      2*(rift_span-multiplicity-P)*(OCP-R)*(1-CLOSED_V_flag_1) 
          new_NFACE_2 = 2*(MCP-rift_span+1)*(NCP-Q)*
     &      (1-CLOSED_W_flag_2) +
     &      2*(NCP-Q)*(OCP-R)*(1-CLOSED_U_flag_2) +
     &      2*(MCP-rift_span+1)*(OCP-R)*(1-CLOSED_V_flag_2) 
          
          
c...      Define new knot vectors
          UK1 = 1d0*rift
          UK2 = 1d0*rift
          UK1(1:rift_span-multiplicity) = 
     &      U_KNOT(1:rift_span-multiplicity)
          UK2(P+2:MCP+2*(P+1) - rift_span) = 
     &      U_KNOT(rift_span+1:MCP+P+1)
          do k = 1,OCP
            do j = 1,NCP
              do i = 1,rift_span-multiplicity
                B1(i,j,k,:) = new_B_NET(i,j,k,:)
              enddo
              do i = 1,MCP-rift_span+p+1
                B2(i,j,k,:) =
     &            new_B_NET(rift_span-multiplicity-1+i,j,k,:)
              enddo
            enddo
          enddo
          VK1 = V_KNOT
          VK2 = V_KNOT
          WK1 = W_KNOT
          WK2 = W_KNOT
        endif
        
        
      
       
c...  Splitting in V
      
        if (dir.eq.2) then
          
          call find_span(Q,NCP,rift,V_KNOT,rift_span)
          allocate(VK1(rift_span-multiplicity+Q+1))
          allocate(VK2(NCP+2*(Q+1) - rift_span))
          allocate(UK1(MCP+P+1))
          allocate(UK2(MCP+P+1))
          allocate(WK1(OCP+R+1))
          allocate(WK2(OCP+R+1))
          allocate(B1(MCP,rift_span-multiplicity,OCP,NSD+1))
          allocate(B2(MCP,NCP-rift_span+Q+1,OCP,NSD+1))
          new_NEL_1 = (rift_span-multiplicity-Q)*(MCP-P)*(OCP-R)
          new_NEL_2 = (NCP+1 - rift_span)*(MCP-P)*(OCP-R)
          new_NFACE_1  = 2*(MCP-P)*(rift_span-multiplicity-Q)*
     &      (1-CLOSED_W_flag_1) +
     &      2*(rift_span-multiplicity-Q)*(OCP-R)*(1-CLOSED_U_flag_1) +
     &      2*(MCP-P)*(OCP-R)*(1-CLOSED_V_flag_1) 
      new_NFACE_2  = 2*(MCP-P)*(NCP-rift_span+1)*(1-CLOSED_W_flag_2) +
     &      2*(NCP-rift_span+1)*(OCP-R)*(1-CLOSED_U_flag_2) +
     &      2*(MCP-P)*(OCP-R)*(1-CLOSED_V_flag_2)
          newNCP1 = rift_span-multiplicity
          newNCP2 = NCP-rift_span+Q+1
          newMCP1 = MCP
          newMCP2 = MCP
          newOCP1 = OCP
          newOCP2 = OCP
          CLOSED_U_flag_1 = CLOSED_U_flag
          CLOSED_W_flag_1 = CLOSED_W_flag
          CLOSED_U_flag_2 = CLOSED_U_flag
          CLOSED_W_flag_2 = CLOSED_W_flag
          
          
c...      Define new knot vectors
          VK1 = 1d0*rift
          VK2 = 1d0*rift
          VK1(1:rift_span-multiplicity) = 
     &      V_KNOT(1:rift_span-multiplicity)
          VK2(Q+2:NCP+2*(Q+1) - rift_span) = 
     &      V_KNOT(rift_span+1:NCP+Q+1)
          do k = 1,OCP
            do i = 1,MCP
              do j = 1,rift_span-multiplicity
                B1(i,j,k,:) = new_B_NET(i,j,k,:)
              enddo
              do j = 1,NCP-rift_span+Q+1
                B2(i,j,k,:) =
     &            new_B_NET(i,rift_span-multiplicity-1+j,k,:)
              enddo
            enddo
          enddo         
          UK1 = U_KNOT
          UK2 = U_KNOT
          WK1 = W_KNOT
          WK2 = W_KNOT
        endif
        
        
        
        
c...    Splitting in W
        
        if (dir.eq.3) then
          
          call find_span(R,OCP,rift,W_KNOT,rift_span)
          allocate(WK1(rift_span-multiplicity+R+1))
          allocate(WK2(OCP+2*(R+1) - rift_span))
          allocate(UK1(MCP+P+1))
          allocate(UK2(MCP+P+1))
          allocate(VK1(NCP+Q+1))
          allocate(VK2(NCP+Q+1))
          allocate(B1(MCP,NCP,rift_span-multiplicity,NSD+1))
          allocate(B2(MCP,NCP,OCP-rift_span+R+1,NSD+1))
          new_NEL_1 = (rift_span-multiplicity-R)*(MCP-P)*(NCP-Q)
          new_NEL_2 = (OCP+(R+1) - rift_span-R)*(MCP-P)*(NCP-Q)
          newOCP1 = rift_span-multiplicity
          newOCP2 = OCP-rift_span+R+1
          newNCP1 = NCP
          newNCP2 = NCP
          newMCP1 = MCP
          newMCP2 = MCP
          CLOSED_V_flag_1 = CLOSED_V_flag
          CLOSED_U_flag_1 = CLOSED_U_flag
          CLOSED_V_flag_2 = CLOSED_V_flag
          CLOSED_U_flag_2 = CLOSED_U_flag
          new_NFACE_1 = 2*(MCP-P)*(NCP-Q)*(1-CLOSED_W_flag_1) +
     &      2*(NCP-Q)*(newOCP1-R)*(1-CLOSED_U_flag_1) +
     &      2*(MCP-P)*(newOCP1-R)*(1-CLOSED_V_flag_1)
          new_NFACE_2 = 2*(MCP-P)*(NCP-Q)*(1-CLOSED_W_flag_2) +
     &      2*(NCP-Q)*(newOCP2-R)*(1-CLOSED_U_flag_2) +
     &      2*(MCP-P)*(newOCP2-R)*(1-CLOSED_V_flag_2) 
          
          
          
c...      Define new knot vectors
          WK1 = 1d0*rift
          WK2 = 1d0*rift
          WK1(1:rift_span-multiplicity) = 
     &      W_KNOT(1:rift_span-multiplicity)
          WK2(R+2:OCP+2*(R+1) - rift_span) = 
     &      W_KNOT(rift_span+1:OCP+R+1)
          do j = 1,NCP
            do i = 1,MCP
              do k = 1,rift_span-multiplicity
                B1(i,j,k,:) = new_B_NET(i,j,k,:)
              enddo
              do k = 1,OCP-rift_span+R+1
                B2(i,j,k,:) =
     &            new_B_NET(i,j,rift_span-multiplicity-1+k,:)
              enddo
            enddo
          enddo         
          UK1 = U_KNOT
          UK2 = U_KNOT
          VK1 = V_KNOT
          VK2 = V_KNOT
        endif
        
        
        
c       
c...    At this point the two geometries have been defined.
c       Now the mesh data required for analysis must be assigned.
        
        
c       We need the connectivities from our original mesh in order to 
c       assign old data to the correct locations in the new mesh.
        call genIEN_INN_3D
        call gen_FACE_IEN_3D       
        
c       Generate EL_CON matrices connecting old elements to new ones. 
c       EL_CON_1(new_el_num) equals the old element number corresponding
c       to the new element in mesh 1. Likewise for EL_CON_2.
        allocate(EL_CON_1(new_NEL_1))
        allocate(EL_CON_2(new_NEL_2))
        call gen_EL_CON_3D_mp(EL_CON_1,EL_CON_2,new_NEL_1,new_NEL_2,
     &    rift_span,multiplicity,
     &    dir)      
        
        
c       Generate FACE_CON matrices connecting old faces to new ones. 
c       FACE_CON_1(new_el_num) equals the old element number corresponding
c       to the new element in mesh 1.
c       FACE_CON_1(new_el_num = 0 implies the face was originally
c       on the interior of the domain. Likewise for FACE_CON_2.
        
        allocate(FACE_CON_1(new_NFACE_1))
        allocate(FACE_CON_2(new_NFACE_2))
        call gen_FACE_CON_3D_mp(FACE_CON_1,FACE_CON_2,new_NFACE_1,
     &    new_NFACE_2,rift_span,multiplicity,dir)
        
        
c...    Create new IEN and INN arrays
        allocate(new_IEN_1((newMCP1-P)*(newNCP1-Q)*(newOCP1-R),
     &    (P+1)*(Q+1)*(R+1)))
        allocate(new_IEN_2((newMCP2-P)*(newNCP2-Q)*(newOCP2-R),
     &    (P+1)*(Q+1)*(R+1)))
        allocate(new_INN_1(newMCP1*newNCP1*newOCP1,NSD))
        allocate(new_INN_2(newMCP2*newNCP2*newOCP2,NSD))
        call gen_new_IEN_INN_3D_mp(newMCP1,newNCP1,newOCP1,new_IEN_1,
     &    new_INN_1)
        call gen_new_IEN_INN_3D_mp(newMCP2,newNCP2,newOCP2,new_IEN_2,
     &    new_INN_2)
        
        
c...    Also create new boundary connectivity arrays
	allocate(new_FACE_IEN_1(new_NFACE_1,(P+1)*(Q+1)*(R+1)))
        allocate(new_FACE_IEN_2(new_NFACE_2,(P+1)*(Q+1)*(R+1)))
        allocate(new_FACE_OR_1(new_NFACE_1))
        allocate(new_FACE_OR_2(new_NFACE_2))     
        call gen_new_FACE_IEN_3D_mp(new_NFACE_1,
     &    new_FACE_IEN_1, new_FACE_OR_1,
     &    newMCP1, newNCP1,newOCP1,new_IEN_1)
        call gen_new_FACE_IEN_3D_mp(new_NFACE_2,
     &    new_FACE_IEN_2, new_FACE_OR_2,
     &    newMCP2, newNCP2,newOCP2,new_IEN_2)
        
        
c       
c...    Create new IBC array assign edge data
c       
        allocate(new_IBC_1(newMCP1*newNCP1*newOCP1,NSD))
        allocate(new_IBC_FACE_1(new_NFACE_1,NSD))
        allocate(new_LD_FACE_1(new_NFACE_1,NSD))
        allocate(new_DIR_BC_1(newMCP1*newNCP1*newOCP1,NSD))
        allocate(new_IBC_2(newMCP2*newNCP2*newOCP2,NSD))
        allocate(new_IBC_FACE_2(new_NFACE_2,NSD))
        allocate(new_LD_FACE_2(new_NFACE_2,NSD))
        allocate(new_DIR_BC_2(newMCP2*newNCP2*newOCP2,NSD))
        allocate(new_IPER_1(newMCP1*newNCP1*newOCP1))
        allocate(new_IPER_2(newMCP2*newNCP2*newOCP2))
        call gen_FACE_DATA_3D_mp(newMCP1*newNCP1*newOCP1,
     &    new_NFACE_1,FACE_CON_1,
     &    new_FACE_OR_1,new_FACE_IEN_1,new_IPER_1,
     &    new_IBC_1, new_IBC_FACE_1,
     &    new_LD_FACE_1,newMCP1, newNCP1, newOCP1, new_INN_1,
     &    B1,UK1,VK1,WK1,new_DIR_BC_1)
        call gen_FACE_DATA_3D_mp(newMCP2*newNCP2*newOCP2,
     &    new_NFACE_2,FACE_CON_2,
     &    new_FACE_OR_2,new_FACE_IEN_2,new_IPER_2,
     &    new_IBC_2, new_IBC_FACE_2,
     &    new_LD_FACE_2,newMCP2, newNCP2, newOCP2, new_INN_2,
     &    B2,UK2,VK2,WK2,new_DIR_BC_2)
        
        
c       
c...    Assign element data
c       
        allocate(new_LD_EL_1(new_NEL_1,NSD))
        allocate(new_EY_1(new_NEL_1))
        allocate(new_NU_1(new_NEL_1))
        allocate(new_LD_EL_2(new_NEL_2,NSD))
        allocate(new_EY_2(new_NEL_2))
        allocate(new_NU_2(new_NEL_2))
        call gen_EL_DATA_3D_mp(new_NEL_1, EL_CON_1, new_LD_EL_1, 
     &    new_EY_1, new_NU_1)
        call gen_EL_DATA_3D_mp(new_NEL_2, EL_CON_2, new_LD_EL_2, 
     &    new_EY_2, new_NU_2)
        

      
c...    Write to file
        NNODZ = newMCP1*newNCP1*newOCP1
        MCP  = newMCP1
        NCP = newNCP1
        OCP = newOCP1
        NFACE = new_NFACE_1
        NEL = new_NEL_1
        CLOSED_U_flag = CLOSED_U_flag_1
        CLOSED_V_flag = CLOSED_V_flag_1
        CLOSED_W_flag = CLOSED_W_flag_1
        call output_mp(UK1,VK1,WK1,B1,new_IPER_1,
     &    new_IBC_1,new_DIR_BC_1,new_IBC_FACE_1,new_LD_FACE_1,
     &    new_LD_EL_1,new_EY_1,new_NU_1,iknot,pflag,nknots,dir)
        
        NNODZ = newMCP2*newNCP2*newOCP2
        MCP  = newMCP2
        NCP = newNCP2
        OCP = newOCP2
        NFACE = new_NFACE_2
        NEL = new_NEL_2
        CLOSED_U_flag = CLOSED_U_flag_2
        CLOSED_V_flag = CLOSED_V_flag_2
        CLOSED_W_flag = CLOSED_W_flag_2
        
        if (iknot.ne.nknots) then ! Remainder of the mesh => new mesh

          deallocate(U_KNOT)    
          deallocate(V_KNOT)
          deallocate(W_KNOT)
          deallocate(B_NET)
          deallocate(BW)
          deallocate(IPER)
          deallocate(IBC)
          deallocate(DIR_BC)
          deallocate(IBC_FACE)
          deallocate (LD_FACE)
          deallocate(LD_EL)
          deallocate(EY)
          deallocate(NU)
          
          deallocate(INN)
          deallocate(IEN)
          deallocate(FACE_IEN)
          deallocate(FACE_OR)
          

          
          allocate(U_KNOT(MCP+P+1))    
          allocate(V_KNOT(NCP+Q+1))
          allocate(W_KNOT(OCP+R+1))
          allocate(B_NET(MCP,NCP,OCP,NSD+1))
          allocate(BW(MCP,NCP,OCP,NSD+1))
          allocate(IPER(NNODZ))
          allocate(IBC(NNODZ,NSD))
          allocate(DIR_BC(NNODZ,NSD))
          allocate(IBC_FACE(NFACE,NSD))
          allocate (LD_FACE(NFACE,NSD))
          allocate(LD_EL(NEL,NSD))
          allocate(EY(NEL))
          allocate(NU(NEL))
          
          U_KNOT = UK2
          V_KNOT = VK2
          W_KNOT = WK2
          B_NET = B2
          IPER = new_IPER_2
          IBC = new_IBC_2
          DIR_BC = new_DIR_BC_2
          IBC_FACE = new_IBC_FACE_2
          LD_FACE = new_LD_FACE_2 
          LD_EL = new_LD_EL_2 
          EY = new_EY_2
          NU = new_NU_2

c...  Form projective control net
          
          BW = B_NET
          do i = 1,NSD
            BW(:,:,:,i) = BW(:,:,:,i)*B_NET(:,:,:,NSD+1)
          enddo      

c...      Deallocate Everything NEW

          deallocate(x_knot)
          deallocate(new_U_KNOT)
          deallocate(new_V_KNOT)
          deallocate(new_W_KNOT)
          deallocate(new_BW)
          deallocate(new_B_NET)

          deallocate(UK1)
          deallocate(UK2)
          deallocate(VK1)
          deallocate(VK2)
          deallocate(WK1)
          deallocate(WK2)

          deallocate(B1)
          deallocate(B2)

          deallocate(EL_CON_1)
          deallocate(EL_CON_2)

          deallocate(FACE_CON_1)
          deallocate(FACE_CON_2)

          deallocate(new_IEN_1)
          deallocate(new_IEN_2)
          deallocate(new_INN_1)
          deallocate(new_INN_2)

          deallocate(new_FACE_IEN_1)
          deallocate(new_FACE_IEN_2)
          deallocate(new_FACE_OR_1)
          deallocate(new_FACE_OR_2)

          deallocate(new_IBC_1)
          deallocate(new_IBC_FACE_1)
          deallocate(new_LD_FACE_1)
          deallocate(new_DIR_BC_1)
          deallocate(new_IBC_2)
          deallocate(new_IBC_FACE_2)
          deallocate(new_LD_FACE_2)
          deallocate(new_DIR_BC_2)
          deallocate(new_IPER_1)
          deallocate(new_IPER_2)

          deallocate(new_LD_EL_1)
          deallocate(new_EY_1)
          deallocate(new_NU_1)
          deallocate(new_LD_EL_2)
          deallocate(new_EY_2)
          deallocate(new_NU_2)
          
        else
          
          call output_mp(UK2,VK2,WK2,B2,new_IPER_2,
     &      new_IBC_2,new_DIR_BC_2,new_IBC_FACE_2,new_LD_FACE_2,
     &      new_LD_EL_2,new_EY_2,new_NU_2,iknot+1,pflag,nknots,dir)     
          
        endif
        
      enddo                     ! end loop overknots to insert
      
      end
