c     This routine links each face in the new patches with the
c        original faces in the unsplit mesh.
c
c     FACE_CON(i) = j indicates that element j in the OLD mesh is the parent
c        of element i in the new mesh.
c
c     modified for patch splitting 9/6/04 - jac3
c      
c       Feb. 10, 2004
c       J. Austin Cottrell
c       CAM Graduate Student
c       Institute for Computational Engineering Science
c       The University of Texas at Austin     

      subroutine gen_FACE_CON_3D_mp(FACE_CON_1,FACE_CON_2,new_NFACE_1,
     &  new_NFACE_2,rift_span,multiplicity,dir) 
      
      use aAdjKeep
      include "common.h"
      
      integer e, i,j,new_NFACE_1,
     &  new_NFACE_2,dir,
     &  rift_span,multiplicity,icount1,icount2
      integer  FACE_CON_1(new_NFACE_1),FACE_CON_2(new_NFACE_2)
      
      
      
c     Each splitting plane orientation must be treated separately.       
      
      e = 0    
      icount1 = 0
      icount2 = 0
      FACE_CON_1 = 0
      FACE_CON_2 = 0
      
c...  SPLITTING IN U DIRECTION
      if (dir.eq.1) then

      if (CLOSED_W_flag.ne.1) then        
c...    Loop through face orientation 1
        do j = 1,NCP-Q
          e = (j-1)*(MCP-P)
          do i = 1,rift_span - multiplicity-P
            e = e+1
            icount1 = icount1+1
            FACE_CON_1(icount1) = e
          enddo
          e = (j-1)*(MCP-P)+rift_span - P-1
          do i = rift_span, MCP
            e = e+1
            icount2 = icount2+1
            FACE_CON_2(icount2) = e
          enddo
        enddo
      endif
      
        
c...    Loop through face orientation 2
      if (CLOSED_V_flag.ne.1) then
        do j = 1,OCP-R
          e = (j-1)*(MCP-P)+(MCP-P)*(NCP-Q)*(1-CLOSED_W_flag)
          do i = 1,rift_span - multiplicity-P
            e = e+1
            icount1 = icount1+1
            FACE_CON_1(icount1) = e
          enddo
          e = (j-1)*(MCP-P)+rift_span - P-1+(MCP-P)*(NCP-Q)*
     &      (1-CLOSED_W_flag)
          do i = rift_span, MCP
            e = e+1
            icount2 = icount2+1
            FACE_CON_2(icount2) = e
          enddo
        enddo
      endif
        
        
c...    Loop through face orientation 3
      if (CLOSED_U_flag.ne.1) then
        do j = 1,OCP-R
          do i = 1,NCP-Q
            icount1 = icount1+1
            FACE_CON_1(icount1) = 0 ! This is the 'internal'
                                ! face for new mesh 1
          enddo
          e = (j-1)*(NCP-Q)+(MCP-P)*(NCP-Q)*(1-CLOSED_W_flag)+
     &      (MCP-P)*(OCP-R)*(1-CLOSED_V_flag)
          do i = 1, NCP-Q
            e = e+1
            icount2 = icount2+1
            FACE_CON_2(icount2) = e
          enddo
        enddo
      endif
        
        
c...    Loop through face orientation 4
      if (CLOSED_V_flag.ne.1) then
        do j = 1,OCP-R
          e = (j-1)*(MCP-P)+(MCP-P)*(NCP-Q)*(1-CLOSED_W_flag) +
     &      (MCP-P)*(OCP-R)*(1-CLOSED_V_flag)+
     &      (OCP-R)*(NCP-Q)*(1-CLOSED_U_flag) +
     &      (MCP - rift_span+multiplicity)
          do i = 1,rift_span - multiplicity-P
            e = e+1
            icount1 = icount1+1
            FACE_CON_1(icount1) = e
          enddo
          e = (j-1)*(MCP-P)+(MCP-P)*(NCP-Q)*(1-CLOSED_W_flag) +
     &      (MCP-P)*(OCP-R)*(1-CLOSED_V_flag)+
     &      (OCP-R)*(NCP-Q)*(1-CLOSED_U_flag)
          do i = rift_span, MCP
            e = e+1
            icount2 = icount2+1
            FACE_CON_2(icount2) = e
          enddo
        enddo
      endif
        
        
c...    Loop through face orientation 5
      if (CLOSED_U_flag.ne.1) then      
        do j = 1,OCP-R
          e = (j-1)*(NCP-Q)+(MCP-P)*(NCP-Q)*(1-CLOSED_W_flag) +
     &      2*(MCP-P)*(OCP-R)*(1-CLOSED_V_flag)+
     &      (OCP-R)*(NCP-Q)*(1-CLOSED_U_flag)
          do i = 1, NCP-Q
            e = e+1
            icount1 = icount1+1
            FACE_CON_1(icount1) = e
          enddo
          do i = 1,NCP-Q
            icount2 = icount2+1
            FACE_CON_2(icount2) = 0 ! This is the 'internal'
                                ! face for new mesh 2
          enddo
        enddo
      endif
        
        
c...    Loop through face orientation 6
      if (CLOSED_W_flag.ne.1) then      
        do j = 1,NCP-Q
          e = (j-1)*(MCP-P)+(MCP-P)*(NCP-Q)*(1-CLOSED_W_flag) +
     &      2*(MCP-P)*(OCP-R)*(1-CLOSED_V_flag) +
     &      2*(OCP-R)*(NCP-Q)*(1-CLOSED_U_flag)
          do i = 1,rift_span - multiplicity-P
            e = e+1
            icount1 = icount1+1
            FACE_CON_1(icount1) = e
          enddo
          e = (j-1)*(MCP-P)+rift_span-P-1 +
     &      (MCP-P)*(NCP-Q)*(1-CLOSED_W_flag) +
     &      2*(MCP-P)*(OCP-R)*(1-CLOSED_V_flag) +
     &      2*(OCP-R)*(NCP-Q)*(1-CLOSED_U_flag)
          do i = rift_span, MCP
            e = e+1
            icount2 = icount2+1
            FACE_CON_2(icount2) = e
          enddo
        enddo
      endif
        

        
c       
c...    SPLITTING IN V DIRECTION
c       
        
      elseif (dir.eq.2) then
        
c...    Loop through face orientation 1
      if (CLOSED_W_flag.ne.1) then        
        do j = 1,rift_span-multiplicity-Q
          e = (j-1)*(MCP-P)
          do i = 1,MCP-P
            e = e+1
            icount1 = icount1+1
            FACE_CON_1(icount1) = e
          enddo
        enddo
        do j = rift_span-Q, NCP-Q
          e = (j-1)*(MCP-P)
          do  i = 1, MCP-P
            e = e+1
            icount2 = icount2+1
            FACE_CON_2(icount2) = e
          enddo
        enddo
      endif
        
c...    Loop through face orientation 2
      if (CLOSED_V_flag.ne.1) then
        do j = 1,OCP-R
          e = (j-1)*(MCP-P)+(MCP-P)*(NCP-Q)*(1-CLOSED_W_flag)
          do i = 1,MCP-P
            e = e+1
            icount1 = icount1+1
            FACE_CON_1(icount1) = e
            icount2 = icount2+1
            FACE_CON_2(icount2) = 0 ! This is the 'internal'
                                ! face for new mesh 2
          enddo
        enddo
      endif
      
        
c...    Loop through face orientation 3
      if (CLOSED_U_flag.ne.1) then
        do j = 1,OCP-R
          e = (j-1)*(NCP-Q)+(MCP-P)*(NCP-Q)*(1-CLOSED_W_flag)+
     &      (MCP-P)*(OCP-R)*(1-CLOSED_V_flag)
          do i = 1,rift_span-multiplicity-Q
            e = e+1
            icount1 = icount1+1
            FACE_CON_1(icount1) = e
          enddo
          e = (j-1)*(NCP-Q)+(rift_span-Q-1)*(MCP-P) +
     &      (MCP-P)*(NCP-Q)*(1-CLOSED_W_flag)+
     &      (MCP-P)*(OCP-R)*(1-CLOSED_V_flag)
          do i = rift_span, NCP
            e = e+1
            icount2 = icount2+1
            FACE_CON_2(icount2) = e
          enddo
        enddo
      endif
        
   
      
c...    Loop through face orientation 4
      if (CLOSED_V_flag.ne.1) then
        do j = 1,OCP-R
          e = (j-1)*(MCP-P)+(MCP-P)*(NCP-Q)*(1-CLOSED_W_flag) +
     &      (MCP-P)*(OCP-R)*(1-CLOSED_V_flag)+
     &      (OCP-R)*(NCP-Q)*(1-CLOSED_U_flag) 
          do i = 1,MCP-P
            e = e+1
            icount1 = icount1+1
            FACE_CON_1(icount1) = 0 ! This is the 'internal'
                                ! face for new mesh 1
            icount2 = icount2+1
            FACE_CON_2(icount2) = e
          enddo
        enddo
      endif

        
c...    Loop through face orientation 5
      if (CLOSED_U_flag.ne.1) then
        do j = 1,OCP-R
          e = (j-1)*(NCP-Q)+(MCP-P)*(NCP-Q)*(1-CLOSED_W_flag) +
     &      2*(MCP-P)*(OCP-R)*(1-CLOSED_V_flag)+
     &      (OCP-R)*(NCP-Q)*(1-CLOSED_U_flag) +
     &      (NCP - rift_span+multiplicity)
          do i = 1,rift_span - multiplicity-Q
            e = e+1
            icount1 = icount1+1
            FACE_CON_1(icount1) = e
          enddo
          e = (j-1)*(NCP-Q)+(MCP-P)*(NCP-Q)*(1-CLOSED_W_flag) +
     &      2*(MCP-P)*(OCP-R)*(1-CLOSED_V_flag)+
     &      (OCP-R)*(NCP-Q)*(1-CLOSED_U_flag)
          do i = rift_span, NCP
            e = e+1
            icount2 = icount2+1
            FACE_CON_2(icount2) = e
          enddo
        enddo
      endif
      

c...    Loop through face orientation 6
      if (CLOSED_W_flag.ne.1) then
        do j = 1,rift_span-multiplicity-Q
          e = (j-1)*(MCP-P)+(MCP-P)*(NCP-Q)*(1-CLOSED_W_flag)+
     &      2*(MCP-P)*(OCP-R)*(1-CLOSED_V_flag)+
     &      2*(OCP-R)*(NCP-Q)*(1-CLOSED_U_flag)
          do i = 1,MCP-P
            e = e+1
            icount1 = icount1+1
            FACE_CON_1(icount1) = e
          enddo
        enddo
        do j = rift_span-Q, NCP-Q
          e = (j-1)*(MCP-P)+(MCP-P)*(NCP-Q)*(1-CLOSED_W_flag)+
     &      2*(MCP-P)*(OCP-R)*(1-CLOSED_V_flag)+
     &      2*(OCP-R)*(NCP-Q)*(1-CLOSED_U_flag)
          do i = 1, MCP-P
            e = e+1
            icount2 = icount2+1
            FACE_CON_2(icount2) = e
          enddo
        enddo
      endif



c       
c...    SPLITTING IN W DIRECTION
c       
        
      elseif (dir.eq.3) then
        
c...    Loop through face orientation 1
      if (CLOSED_W_flag.ne.1) then
        do j = 1,NCP-Q
          do i = 1,MCP-P
            e = e+1
            icount1 = icount1+1
            icount2 = icount2+1    
            FACE_CON_1(icount1) = e
            FACE_CON_2(icount2) = 0 ! This is the 'internal'
                                ! face for new mesh 2

          enddo
        enddo
      endif
        
        
c...    Loop through face orientation 2
      if (CLOSED_V_flag.ne.1) then
        do j = 1,rift_span-multiplicity-R
          e = (j-1)*(MCP-P)+(MCP-P)*(NCP-Q)*(1-CLOSED_W_flag)
          do i = 1,MCP-P
            e = e+1
            icount1 = icount1+1
            FACE_CON_1(icount1) = e
          enddo
        enddo
        do j = rift_span-R, OCP-R
          e = (j-1)*(MCP-P)+(MCP-P)*(NCP-Q)*(1-CLOSED_W_flag)
          do i = 1, MCP-P
            e = e+1
            icount2 = icount2+1
            FACE_CON_2(icount2) = e
          enddo
        enddo
      endif
        
        
c...    Loop through face orientation 3
      if (CLOSED_U_flag.ne.1) then
        do j = 1,rift_span-multiplicity-R
          e = (j-1)*(NCP-Q)+(MCP-P)*(NCP-Q)*(1-CLOSED_W_flag)+
     &      (MCP-P)*(OCP-R)*(1-CLOSED_V_flag)
          do i = 1,NCP-Q
            e = e+1
            icount1 = icount1+1
            FACE_CON_1(icount1) = e
          enddo
        enddo
        do j = rift_span-R, OCP-R
          e = (j-1)*(NCP-Q)+(MCP-P)*(NCP-Q)*(1-CLOSED_W_flag)+
     &      (MCP-P)*(OCP-R)*(1-CLOSED_V_flag)
          do i = 1, NCP-Q
            e = e+1
            icount2 = icount2+1
            FACE_CON_2(icount2) = e
          enddo
        enddo
      endif
        
   
      
c...    Loop through face orientation 4
      if (CLOSED_V_flag.ne.1) then
        do j = 1,rift_span-multiplicity-R
          e = (j-1)*(MCP-P)+(MCP-P)*(NCP-Q)*(1-CLOSED_W_flag)+
     &      (MCP-P)*(OCP-R)*(1-CLOSED_V_flag)+
     &      (OCP-R)*(NCP-Q)*(1-CLOSED_U_flag)
          do i = 1,MCP-P
            e = e+1
            icount1 = icount1+1
            FACE_CON_1(icount1) = e
          enddo
        enddo
        do j = rift_span-R, OCP-R
          e = (j-1)*(MCP-P)+(MCP-P)*(NCP-Q)*(1-CLOSED_W_flag)+
     &      (MCP-P)*(OCP-R)*(1-CLOSED_V_flag)+
     &      (OCP-R)*(NCP-Q)*(1-CLOSED_U_flag)
          do i = 1, MCP-P
            e = e+1
            icount2 = icount2+1
            FACE_CON_2(icount2) = e
          enddo
        enddo
      endif

        
c...    Loop through face orientation 5
      if (CLOSED_U_flag.ne.1) then
        do j = 1,rift_span-multiplicity-R
          e = (j-1)*(NCP-Q)+(MCP-P)*(NCP-Q)*(1-CLOSED_W_flag)+
     &      2*(MCP-P)*(OCP-R)*(1-CLOSED_V_flag)+
     &      (OCP-R)*(NCP-Q)*(1-CLOSED_U_flag)
          do i = 1,NCP-Q
            e = e+1
            icount1 = icount1+1
            FACE_CON_1(icount1) = e
          enddo
        enddo
        do j = rift_span-R, OCP-R
          e = (j-1)*(NCP-Q)+(MCP-P)*(NCP-Q)*(1-CLOSED_W_flag)+
     &      2*(MCP-P)*(OCP-R)*(1-CLOSED_V_flag)+
     &      (OCP-R)*(NCP-Q)*(1-CLOSED_U_flag)
          do i = 1, NCP-Q
            e = e+1
            icount2 = icount2+1
            FACE_CON_2(icount2) = e
          enddo
        enddo
      endif
      

c...    Loop through face orientation 6
      if (CLOSED_W_flag.ne.1) then
        do j = 1,NCP-Q
          e = (j-1)*(MCP-P)+(MCP-P)*(NCP-Q)*(1-CLOSED_W_flag)+
     &      2*(MCP-P)*(OCP-R)*(1-CLOSED_V_flag)+
     &      2*(OCP-R)*(NCP-Q)*(1-CLOSED_U_flag)
          do i = 1,MCP-P
            e = e+1
            icount1 = icount1+1
            icount2 = icount2+1
            FACE_CON_2(icount2) = e
            FACE_CON_1(icount1) = 0 ! This is the 'internal'
                                ! face for new mesh 1
          enddo
        enddo
      endif
        
      endif

  
      
      return
      end
