c
c     This subroutine modifies the local stiffness matrix and load vector
c       to account for constrained nodes. At present we only allow for no
c       constraint or for zero displacement. Future work will include 
c       allowing for prescribed nonzero displacements.
c
c     This is for the 3D version.
c
c
c     January 20, 2004
c
c
c     J. Austin Cottrell, III
c     Jurijs Bazilevs
c
c     CAM Graduate Students
c     Institute for Computational Engineering Science
c     The University of Texas at Austin

      subroutine BCLhs_3D(iel, xKebe, Rhs)
      
      use aAdjKeep
      
      include "common.h"
      
      integer iel, aa, bb, cc
      real*8 xKebe(NSD*NSD,NSHL,NSHL), Rhs(NSD,NSHL) 
      
      
      
c...  loop through local nodes
      
      do aa = 1, NSHL
        
        cc = IEN(iel,aa)        ! global node number of local node aa
        
        
c...    Check status of constraint in x direction
        if (IBC(cc,1).eq.1) then 
          
          
c...      update matrix to account for constraint
          
          do bb = 1, NSHL       ! project away row (X-dir)             
            
            xKebe(1,aa,bb) = 0d+0
            xKebe(2,aa,bb) = 0d+0
            xKebe(3,aa,bb) = 0d+0
            
          enddo
          
          do bb = 1, NSHL       !project away column (X-dir)
            
            xKebe(1,bb,aa) = 0d+0
            xKebe(4,bb,aa) = 0d+0
            xKebe(7,bb,aa) = 0d+0
            
          enddo
          
          xKebe(1,aa,aa) = 1d+0
          Rhs(1,aa) = 0d+0      ! rhs
          
        endif
        
        
c...    Check for constraint in y direction
        
        if (IBC(cc,2).eq.1) then
          
          
          do bb = 1, NSHL       ! project away row (Y-dir)             
            
            xKebe(4,aa,bb) = 0d+0
            xKebe(5,aa,bb) = 0d+0
            xKebe(6,aa,bb) = 0d+0
            
          enddo
          
          do bb = 1, NSHL       !project away column (Y-dir)
            
            xKebe(2,bb,aa) = 0d+0
            xKebe(5,bb,aa) = 0d+0
            xKebe(8,bb,aa) = 0d+0
            
          enddo
          
          xKebe(5,aa,aa) = 1d+0           
          Rhs(2,aa) = 0d+0      ! rhs
          
        endif
        
c...    Check for constraint in z direction
        
        if (IBC(cc,3).eq.1) then
          
          
          do bb = 1, NSHL       ! project away row (Y-dir)             
            
            xKebe(7,aa,bb) = 0d+0
            xKebe(8,aa,bb) = 0d+0
            xKebe(9,aa,bb) = 0d+0
            
          enddo
          
          do bb = 1, NSHL       !project away column (Y-dir)
            
            xKebe(3,bb,aa) = 0d+0
            xKebe(6,bb,aa) = 0d+0
            xKebe(9,bb,aa) = 0d+0
            
          enddo
          
          xKebe(9,aa,aa) = 1d+0           
          Rhs(3,aa) = 0d+0      ! rhs
          
        endif         
        
        
      enddo
      
      return
      end
      
      
