c     This subroutine calculates the contributions to the left hand
c      side forcing vector due to boundary faces.
c
c     This is for the 3D case.
c     
c     
c     July 1, 2003
c     J. Austin Cottrell
c     CES Graduate Student
c     Texas Institute for Computational Engineering Science
c     University of Texas at Austin
c
c     Modified from 2D "BdryEdjAss.f"  
c     January 20, 2004
c     J. Austin Cottrell


      subroutine FaceAssembly_3D
      
      use aAdjKeep

      include "common.h"
      
      
      integer ifac, nshb, igaussb, jgaussb, i, g, ni, nj, 
     &     nk, area_flag
      real*8  sigx,sigy,sigz,gp(NGAUSS), gw(NGAUSS), da, gwt
      real*8  Rhsb(NSD,NSHL), shb(NSHL), shbg(NSHL,NSD), nor(NSD),
     &  Pres, norsurf
    
c     shb will be the shape function array while shbg will hold the
c     gradients of the shape functions

      
      
c...  Loop over Faces

      norsurf = 0d+0
      
      do ifac = 1, NFACE
        area_flag = 0         ! initialize flag
        
c...    Check if face is of nonzero area
        
        ni = INN(FACE_IEN(ifac,1),1)
        nj = INN(FACE_IEN(ifac,1),2)
        nk = INN(FACE_IEN(ifac,1),3)
        
        if ((FACE_OR(ifac).eq.1).or.(FACE_OR(ifac).eq.6)) then
           da = (U_KNOT(ni+1) - U_KNOT(ni))* 
     &          (V_KNOT(nj+1) - V_KNOT(nj))/4d+0 ! change in gwt
                                ! due to mapping from
                                ! [-1,1] x [-1,1]
           if((U_KNOT(ni).eq.(U_KNOT(ni+1))).or.
     &          (V_KNOT(nj).eq.(V_KNOT(nj+1))) ) then
              area_flag = 1     ! indicates face has zero area
           endif
           
           
        else if ((FACE_OR(ifac).eq.2).or.(FACE_OR(ifac).eq.4)) then
           da = (U_KNOT(ni+1) - U_KNOT(ni))* 
     &          (W_KNOT(nk+1) - W_KNOT(nk))/4d+0 ! change in gwt
                                ! due to mapping from
                                ! [-1,1] x [-1,1]
           if((U_KNOT(ni).eq.(U_KNOT(ni+1))).or.
     &          (W_KNOT(nk).eq.(W_KNOT(nk+1))) ) then
              area_flag = 1     ! indicates face has zero area
           endif
           
        else if ((FACE_OR(ifac).eq.3).or.(FACE_OR(ifac).eq.5)) then
           da = (V_KNOT(nj+1) - V_KNOT(nj))* 
     &          (W_KNOT(nk+1) - W_KNOT(nk))/4d+0 ! change in gwt
                                ! due to mapping from
                                ! [-1,1] x [-1,1]
           if((V_KNOT(nj).eq.(V_KNOT(nj+1))).or.
     &          (W_KNOT(nk).eq.(W_KNOT(nk+1))) ) then
              area_flag = 1     ! indicates face has zero area
           endif

        endif
        
        
c...    If face has zero area, skip entirely
        if (area_flag.eq.0) then
          
          
c..       Check for loads applied to the faces
           if (((IBC_FACE(ifac,1).eq.2).or.(IBC_FACE(ifac,2).eq.2)).or.
     &          (IBC_FACE(ifac,3).eq.2)) then
            
                                ! skip boundary integrals if load not prescr.
            
            
c...        set up element parameters
         
            
      
c...          get Gauss Point/Weight Arrays
            
              call genGPandGW(gp,gw)
            
              Rhsb = 0d+0
              shb = 0d+0
              shbg = 0d+0

c...        Loop over integration points
            
            do igaussb = 1, NGAUSS
               do jgaussb = 1, NGAUSS
              
c...              Get Boundary Shape functions and set Jacobian
              
                  call eval_FACE_3D(ifac,gp(igaussb), gp(jgaussb),
     &                 ni, nj, nk, shb, shbg, nor)
              
c     eval_FACE_3D returns no gradient yet.
              

              
c...              Get boundary loads for this edge
              
                  sigx = LD_FACE(ifac,1)
                  sigy = LD_FACE(ifac,2)
                  sigz = LD_FACE(ifac,3)                  

c...  This would be a good place to insert a more sophisticated, problem
c      specific call to calculate loads on faces. The normal vectors
c      maybe calculated from the gradients.

              
c...              assemble local load vector
              
                  gwt = gw(igaussb)*gw(jgaussb)*da
              
ccccc                  call e3bRhs_3D(shb,sigx,sigy,sigz,gwt,Rhsb)
                  
c-----------------------------------------------------------------------c
c                 User-Defined Pressurization                           c
c                 Set Value of Pres                                     c

                
                    Pres = -1d+2
                  
                    call e3bRhs_3D_pres(shb,nor,gwt,Rhsb,Pres,norsurf)
      
                    
c                                                                       c
c-----------------------------------------------------------------------c
                  
              
               enddo
            enddo
            


            
c...        check if displacements are prescribed 
c...        and assemble into global force vector 
            
            do i = 1,NSHL
              
              g = FACE_IEN(ifac,i) ! global node number
              
                                ! in x-direction              
              
              if(IBC(g,1).eq.1) Rhsb(1,i) = 0d+0
                                ! in y-direction
              
              if(IBC(g,2).eq.1) Rhsb(2,i) = 0d+0    
                                ! in z-direction
              
              if(IBC(g,3).eq.1) Rhsb(3,i) = 0d+0 
              
                                ! assemble
              RHSG(g,1) = RHSG(g,1) + Rhsb(1,i)
              RHSG(g,2) = RHSG(g,2) + Rhsb(2,i)
              RHSG(g,3) = RHSG(g,3) + Rhsb(3,i)
              
            enddo
            
             
          endif
        endif
      enddo

      write(*,*) "Surface Area=    ", norsurf
      
      return
      end
      


      
      
