c
c  Subroutine e3Rhs.f consumes the forces in the x, y, z directions, as well
c     as the Gaussian quadrature weight and the shape functions, and assembles
c     the local load vector. 
c
c     This is the 3D version
c
c
c  January 20, 2004
c
c 
c
c  J. Austin Cottrell
c  Jurijs Bazilevs
c
c  CAM Graduate Students
c  Institute for Computational Engineering Science
c  The University of Texas at Austin


      subroutine e3Rhs_3D (forx, fory, forz, gwt, shg, Rhs, vol)
      
      use aAdjKeep
      
      include "common.h"
      
      integer aa
      
      real*8  gwt, forx, fory, forz, shg(NSHL),
     &  Rhs(NSD,NSHL), vol
      
      
      do aa = 1, NSHL
        
        Rhs(1,aa) = Rhs(1,aa) + forx*shg(aa)*DetJ*gwt
        
        Rhs(2,aa) = Rhs(2,aa) + fory*shg(aa)*DetJ*gwt
        
        Rhs(3,aa) = Rhs(3,aa) + forz*shg(aa)*DetJ*gwt
        
      enddo

      vol = vol + DetJ*gwt
      
      return
      end

      
      subroutine e3Rhs_strss_3D (yl, gwt, shg, shgradg,
     &  lambda, mu, Rhs)
      
      use aAdjKeep
      
      include "common.h"
      
      integer aa
      
      real*8  gwt, shg(NSHL), Rhs(NSD,NSHL), shgradg(NSHL,NSD),
     &  lambda, mu, Strn(6), yl(NSHL,NSD), Strss(6), trace,
     &  J2
      
      
      Strn = 0d+0
      
c...  Calculate Symmetric Strain Tensor - 3D
      
      do aa = 1, NSHL
        
        Strn(1) = Strn(1) + yl(aa,1)*shgradg(aa,1)
        Strn(2) = Strn(2) + yl(aa,2)*shgradg(aa,2)
        Strn(3) = Strn(3) + yl(aa,3)*shgradg(aa,3)
        
        Strn(4) = Strn(4) + 1d+0/2d+0*(yl(aa,1)*shgradg(aa,2) +
     &    yl(aa,2)*shgradg(aa,1))
        Strn(5) = Strn(5) + 1d+0/2d+0*(yl(aa,2)*shgradg(aa,3) +
     &    yl(aa,3)*shgradg(aa,2))
        Strn(6) = Strn(6) + 1d+0/2d+0*(yl(aa,1)*shgradg(aa,3) +
     &    yl(aa,3)*shgradg(aa,1))
        
      enddo
      
c...  Calculate Strss at an int. point      
      
      Strss(1) = (lambda+2d+0*mu)*Strn(1) +
     &  lambda*Strn(2) + lambda*Strn(3)
      Strss(2) = (lambda+2d+0*mu)*Strn(2) +
     &  lambda*Strn(1) + lambda*Strn(3)
      Strss(3) = (lambda+2d+0*mu)*Strn(3) +
     &  lambda*Strn(1) + lambda*Strn(2)
      
      Strss(4) = 2d+0*mu*Strn(4)
      Strss(5) = 2d+0*mu*Strn(5)
      Strss(6) = 2d+0*mu*Strn(6)

c...  Calculate energy at this integration point
      
      ENRGY = ENRGY + (Strn(1)*Strss(1) +
     &  Strn(2)*Strss(2) + Strn(3)*Strss(3) +
     &  2d+0*(Strn(4)*Strss(4) + Strn(5)*Strss(5) +
     &  Strn(6)*Strss(6)))*DetJ*gwt
      
      trace = Strss(1) + Strss(2) + Strss(3)
      trace = trace*(1d+0/3d+0)

      Strss(1) = Strss(1) - trace
      Strss(2) = Strss(2) - trace
      Strss(3) = Strss(3) - trace
      
c...  Calculate J2 at this integration point
      
      J2 = Strss(1)*Strss(1) +
     &  Strss(2)*Strss(2) + Strss(3)*Strss(3) +
     &  2d+0*(Strss(4)*Strss(4) + Strss(5)*Strss(5) +
     &  Strss(6)*Strss(6))


c...  Von Mises Stress

      J2 = sqrt(J2)
      
c...  Calculate contribution to RHS (Sxx Syy and Szz only)
      
      do aa = 1, NSHL
        
        Rhs(1,aa) = Rhs(1,aa) + shg(aa)*J2*DetJ*gwt
        Rhs(2,aa) = Rhs(2,aa) + shg(aa)*J2*DetJ*gwt
        Rhs(3,aa) = Rhs(3,aa) + shg(aa)*J2*DetJ*gwt
        
      enddo
      
      return
      end
      
